using System;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Diagnostics;
using System.Management;
using System.Threading;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
namespace CachingCoreTests
{
    /// <summary>
    /// Summary description for CachingInstrumentationTestFixture
    /// </summary>
    [TestClass]
    public class CachingInstrumentationTestFixture
    {
        private CacheManager itemCache;
        String wmiText = "";
        bool wmiLogged = false;
        public CachingInstrumentationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
         //Use TestInitialize to run code before running each test 
         [TestInitialize()]
         public void MyTestInitialize() { 
         itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            if (itemCache != null)
                itemCache.Flush();}
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TotalEntriesCounterTest()
        {
            string counterName = "Total Cache Entries";
            int initialCountDefault = GetCounterValue(counterName, "DefaultCacheManager");
            int initialCountIsolated = GetCounterValue(counterName, "IsolatedCacheManager");
            int loggedCountDefault = 0;
            int loggedCountIsolated = 0;
            
            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails itemDetails = new ItemDetails(10, "Toy10", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            loggedCountDefault = GetCounterValue(counterName, "DefaultCacheManager");
            loggedCountIsolated = GetCounterValue(counterName, "IsolatedCacheManager");
            Assert.AreEqual(loggedCountDefault - 1, initialCountDefault);
            Assert.AreEqual(loggedCountIsolated, initialCountIsolated);

            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            itemDetails = new ItemDetails(20, "Toy20", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            loggedCountDefault = GetCounterValue(counterName, "DefaultCacheManager");
            loggedCountIsolated = GetCounterValue(counterName, "IsolatedCacheManager"); 
            Assert.AreEqual(loggedCountDefault - 1, initialCountDefault);
            Assert.AreEqual(loggedCountIsolated - 1, initialCountIsolated); 

        }

        [TestMethod]
        public void ConfigFailWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM CacheConfigurationFailureEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start(); 
            
            try
            {
                itemCache = CacheFactory.GetCacheManager("NonExistingManager");
            }

            catch (System.Configuration.ConfigurationErrorsException)
            { }

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("CacheConfigurationFailureEvent") == -1)
            {
                Assert.Fail();
            }

        }

        [TestMethod]
        public void ConfigFailEventLogTest()
        {
            try
            {
                itemCache = CacheFactory.GetCacheManager("NonExistingManager");
            }

            catch (System.Configuration.ConfigurationErrorsException)
            { }

            string logName = EventLog.LogNameFromSourceName("Application", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;
                if (!(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("NonExistingManager")))
                    Assert.Fail("Event Not logged");
            }

        }

        [TestMethod]
        public void UpdatedEntriesCounterTest()
        {
            string counterName = "Updated Entries/sec";
            int initialCount = GetCounterValue(counterName, "DefaultCacheManager");
            int loggedCount = 0;

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            loggedCount = GetCounterValue(counterName, "DefaultCacheManager");
            Assert.AreEqual(loggedCount - 1, initialCount);
        }

        [TestMethod]
        public void CacheHitsCounterTest()
        {
            string counterName = "Cache Hits/sec";
            
            int loggedCount = 0;

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            int initialCount = GetCounterValue(counterName, "DefaultCacheManager");

            writtenItem = (ItemDetails)itemCache.GetData("1");
            loggedCount = GetCounterValue(counterName, "DefaultCacheManager");
            Assert.AreEqual(loggedCount - 1, initialCount);
        }

        [TestMethod]
        public void CacheAccessAttemptsCounterTest()
        {
            string counterName = "Total # of Cache Access Attempts";

            int loggedCount = 0;

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            int initialCount = GetCounterValue(counterName, "DefaultCacheManager");

            writtenItem = (ItemDetails)itemCache.GetData("1");
            loggedCount = GetCounterValue(counterName, "DefaultCacheManager");
            Assert.AreEqual(loggedCount - 1, initialCount);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            loggedCount = GetCounterValue(counterName, "DefaultCacheManager");
            Assert.AreEqual(loggedCount - 2, initialCount);

        }

        [TestMethod]
        public void CacheHitRatioCounterTest()
        {
            string counterName = "Cache Hit Ratio";
            int loggedCount = 0;

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            int initialCount = GetCounterValue(counterName, "DefaultCacheManager");

            writtenItem = (ItemDetails)itemCache.GetData("1");
            loggedCount = GetCounterValue(counterName, "DefaultCacheManager");
            Assert.AreEqual(loggedCount - 1, initialCount);
        }


        [TestMethod]
        public void CacheMissesCounterTest()
        {
            string counterName = "Cache Misses/sec";

            int loggedCount = 0;

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            int initialCount = GetCounterValue(counterName, "DefaultCacheManager");

            writtenItem = (ItemDetails)itemCache.GetData("2");
            loggedCount = GetCounterValue(counterName, "DefaultCacheManager");
            Assert.AreEqual(loggedCount - 1, initialCount);
        }
        
        // Only one instances is created for each counter. That is the default instance.
        // Total instance is not created.
        [TestMethod]
        public void CounterInstanceTest()
        {

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            string categoryName = "Enterprise Library Caching Counters";
            string instanceName = new AppDomainNameFormatter().CreateName("DefaultCacheManager");
            Assert.IsTrue(PerformanceCounterCategory.InstanceExists(instanceName, categoryName));

        }


        private int  GetCounterValue(string counterName, string instance)
        {
            string categoryName = "Enterprise Library Caching Counters";
            string instanceName = new AppDomainNameFormatter().CreateName(instance);
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            wmiLogged = true;
            wmiText = args.NewEvent.GetText(TextFormat.Mof);
        }
    }
}
