using System;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
namespace CachingCoreTests
{
    /// <summary>
    /// Summary description for CachingIsolatedStoreTestFixture
    /// </summary>
    [TestClass]
    public class CachingIsolatedStoreTestFixture
    {
        private CacheManager itemCache;
        public CachingIsolatedStoreTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            if (itemCache != null)
                itemCache.Flush();

            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManagerEncryption");
            if (itemCache != null)
                itemCache.Flush();

        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Default Add overload"
        [TestMethod]
        public void IsolatedStoreAddDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAddRemoveDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreAddDupKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAddNullKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreReadNullKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreRemoveNullKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreNonExistKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAddRemoveCountDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreContainsDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreAddNullObjectDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreAddNullValueDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25),writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreCheckContainsNullDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Absolute Time Expiry, Add overload"
        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddRemoveTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddDupKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            
            itemCache.Add("1", null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            object writtenItem = itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Sliding Time Expiry, Add overload"
        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddRemoveTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddDupKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "File Dependency Expiry, Add overload"
        [TestMethod]
        public void IsolatedStoreFileExpiryAddTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddRemoveTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddDupKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreFileExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreFileExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Extended Format, Add overload"
        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddRemoveTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddDupKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));


            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Never Expired, Add overload"
        [TestMethod]
        public void IsolatedStoreNeverExpiredAddTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddRemoveTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddDupKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        [TestMethod]
        [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void IsolatedStoreEncConfigErrorTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedConfigErrorEncryption");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            Assert.AreEqual(1, itemCache.Count);

        }

        [TestMethod]
        public void IsolatedStoreEncryptAddTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManagerEncryption");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreCryptoEncryptAddTest()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCrytpoEncryption");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

    }
}
