using System;
using Microsoft.Practices.EnterpriseLibrary.Caching;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
namespace CachingCoreTests
{
    /// <summary>
    /// Summary description for DifferentCacheManagersTestFixture
    /// </summary>
    [TestClass]
    public class DiffCacheManagersTestFixture
    {
        private CacheManager itemCache;
        public DiffCacheManagersTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            if (itemCache != null)
                itemCache.Flush();

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            if (itemCache != null)
                itemCache.Flush();

        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        [TestMethod]
        public void AddDefaultDiffCacheManagerTest()
        {
            ItemDetails writtenItem = new ItemDetails();

            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            
            itemCache = CacheFactory.GetCacheManager();
            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            itemDetails = new ItemDetails(7, "Toy7", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            
            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            writtenItem = (ItemDetails)itemCache.GetData("7");
            Assert.AreEqual("Toy7", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);
            Assert.AreEqual(3, itemCache.Count);

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);
            Assert.AreEqual(4, itemCache.Count);

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);
            
            itemCache = CacheFactory.GetCacheManager();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);
            Assert.AreEqual(4, itemCache.Count);

            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            Assert.AreEqual(0, itemCache.Count);

            itemCache = CacheFactory.GetCacheManager("IsolatedCacheManager");
            Assert.AreEqual(3, itemCache.Count);

            

        }
    }
}
