using System;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.IO;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
namespace CachingCoreTests
{
    /// <summary>
    /// Summary description for DiffParametersDicTestFixture
    /// </summary>
    [TestClass]
    public class DiffParametersDicTestFixture
    {
        private CacheManager itemCache;
        private CacheManagerFactory factory;
        private DictionaryConfigurationSource configSource;
        public DiffParametersDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            CachingDictionarySource sourceSection = new CachingDictionarySource();
            configSource = sourceSection.BuildDictionarySourceSection();

            factory = new CacheManagerFactory(configSource);
            itemCache = factory.Create("InMemoryCacheManager");
            if (itemCache != null)
                itemCache.Flush(); 
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Combination of different priority"

        [TestMethod]
        public void LowPriorityDefaultDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void HighPriorityDefaultDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));

        }
        [TestMethod]
        public void NormalPriorityDefaultDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void NonePriorityDefaultDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void NotRemovablePriorityDefaultDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion 

        #region "Combination of different priority and Absolutetime expiry"

        [TestMethod]
        public void LowPriorityAbsoluteTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(1)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void HighPriorityAbsoluteTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));

        }
        [TestMethod]
        public void NormalPriorityAbsoluteTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void NonePriorityAbsoluteTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void NotRemovablePriorityAbsoluteTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion 

        #region "Combination of different priorities and slidingtime"
        [TestMethod]
        public void LowPrioritySlidingtimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void HighPrioritySlidingtimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));
        }
        [TestMethod]
        public void NormalPrioritySlidingtimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void NonePrioritySlidingtimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void NotRemovablePrioritySlidingtimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion

        #region "Combination of different priorities and File dependency"



        [TestMethod]
        public void LowPriorityFileDependencyDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void HighPriorityFileDependencyDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));
        }
        [TestMethod]
        public void NormalPriorityFileDependencyDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void NonePriorityFileDependencyDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void NotRemovablePriorityFileDependencyDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion

        #region "Combination of different priorities and ExtendedFormat"
        [TestMethod]
        public void LowPriorityExtendedFormatTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void HighPriorityExtendedFormatTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));
        }
        [TestMethod]
        public void NormalPriorityExtendedFormatTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void NonePriorityExtendedFormatTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void NotRemovablePriorityExtendedFormatTimeDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }

        #endregion

        #region "Combination of different priority and Never Expired Items"

        [TestMethod]
        public void LowPriorityNeverExpiredDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void HighPriorityNeverExpiredDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));

        }
        [TestMethod]
        public void NormalPriorityNeverExpiredDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void NonePriorityNeverExpiredDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void NotRemovablePriorityNeverExpiredDicTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion

    }
}
