using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.Data;
using System.Data.SqlClient;
using System.Threading;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.Configuration;

namespace CachingDBThreadWaitTests
{
    /// <summary>
    /// Summary description for ScavengeDBDicTestFixture
    /// </summary>
    [TestClass]
    public class ScavengeDBDicTestFixture
    {
        String connStr;
        private CacheManager itemCache;
        private CacheManagerFactory factory;
        private DictionaryConfigurationSource configSource;
        public ScavengeDBDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            CachingDBDictionarySource sourceSection = new CachingDBDictionarySource();
            configSource = sourceSection.BuildDBDictionarySourceSection();

            connStr = "server=(local)\\SQLEXPRESS;initial catalog=caching;integrated security=SSPI";

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("begin delete from CacheData " +
                                            "end", conn);
            cmd.ExecuteNonQuery();

            conn.Close();

            factory = new CacheManagerFactory(configSource);
            itemCache = factory.Create("DBCacheManagerDic");
            if (itemCache != null)
                itemCache.Flush();
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Scavenging - DB Cache Manager"
        [TestMethod]
        public void DBScavengeDiffPriorityDicTest()
        {
            int count = 0;
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            Thread.Sleep(1000);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(21), writtenItem.Price);

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            Thread.Sleep(1000); 
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNotNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNotNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNotNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNotNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNotNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("7");
            Assert.IsNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("8");
            Assert.IsNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("9");
            Assert.IsNotNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("10");
            Assert.IsNotNull(writtenItem);
            writtenItem = (ItemDetails)itemCache.GetData("11");
            Assert.IsNotNull(writtenItem);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(8, count);

            dr.Close();
            conn.Close();

            
        }

        [TestMethod]
        public void DBScavengeSamePriorityDicTest()
        {
            int count = 0;
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            Thread.Sleep(1500);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(22), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("7");
            Assert.AreEqual("Toy7", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(21), writtenItem.Price);

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            Thread.Sleep(1000);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNotNull(writtenItem);

            writtenItem = (ItemDetails)itemCache.GetData("7");
            Assert.IsNotNull(writtenItem);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNotNull(writtenItem);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(8, count);

            dr.Close();
            conn.Close();


        }
        #endregion
    }
}
