using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;

using Microsoft.Practices.EnterpriseLibrary.Caching.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.Database.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;

namespace CachingDatabaseTests
{
    public class CachingDBDictionarySource
    {
        public DictionaryConfigurationSource BuildDBDictionarySourceSection()
        {
            DictionaryConfigurationSource sections = new DictionaryConfigurationSource();
            CacheManagerSettings settings = new CacheManagerSettings();

            settings.DefaultCacheManager = "DBCacheManagerDic";

            #region "Backing Stores"
            DataCacheStorageData dcsd = new DataCacheStorageData();
            dcsd.Name = "Data Cache Storage Dic";
            dcsd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.Database.DataBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching.Database");
            dcsd.DatabaseInstanceName = "CachingDatabase";
            dcsd.PartitionName = "Partition1";
            settings.BackingStores.Add(dcsd);

            dcsd = new DataCacheStorageData();
            dcsd.Name = "Data Cache Storage Encryption Dic";
            dcsd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.Database.DataBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching.Database");
            dcsd.DatabaseInstanceName = "CachingDatabase";
            dcsd.PartitionName = "Partition1";
            settings.BackingStores.Add(dcsd);

            #endregion

            #region "Cache Managers"
            CacheManagerData cmd = new CacheManagerData();
            cmd.Name = "DBCacheManagerDic";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "Data Cache Storage Dic";

            settings.CacheManagers.Add(cmd);

            cmd = new CacheManagerData();
            cmd.Name = "DBCacheManagerEncryptionDic";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "Data Cache Storage Encryption Dic";

            settings.CacheManagers.Add(cmd);

            #endregion

            sections.Add("cachingConfiguration", settings);

            #region "Data Configuration"
            DatabaseSettings dataSettings = new DatabaseSettings();
            dataSettings.DefaultDatabase = "CachingDatabase";

            ConnectionStringSettings connstring = new ConnectionStringSettings();
            connstring.Name = "CachingDatabase";
            connstring.ProviderName = "System.Data.SqlClient";
            connstring.ConnectionString = "server=(local)\\SQLEXPRESS;database=Caching;Integrated Security=true";


            ConnectionStringsSection connsection = new ConnectionStringsSection();
            connsection.ConnectionStrings.Add(connstring); 
            
            sections.Add("dataConfiguration", dataSettings);
            sections.Add("connectionStrings", connsection);
            #endregion

            InstrumentationConfigurationSection ics = new InstrumentationConfigurationSection(true, true, true);
            sections.Add("instrumentationConfiguration", ics);
            return sections;
        }
    }
}
