using System;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.Data;
using System.Data.SqlClient;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.Configuration;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
namespace CachingDatabaseTests
{
    /// <summary>
    /// Summary description for CachingDBStoreDicTestFixture
    /// </summary>
    [TestClass]
    public class CachingDBStoreDicTestFixture
    {
        private CacheManager itemCache;
        private CacheManagerFactory factory;
        private DictionaryConfigurationSource configSource;
        string connStr;
        public CachingDBStoreDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            CachingDBDictionarySource sourceSection = new CachingDBDictionarySource();
            configSource = sourceSection.BuildDBDictionarySourceSection();

            connStr = "server=(local)\\SQLEXPRESS;initial catalog=caching;integrated security=SSPI";

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("begin delete from CacheData " +
                                            "end", conn);
            cmd.ExecuteNonQuery();

            conn.Close();

            factory = new CacheManagerFactory(configSource);
            itemCache = factory.Create("DBCacheManagerDic");
            if (itemCache != null)
                itemCache.Flush();

        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Default Add overload"
        [TestMethod]
        public void DBStoreAddDefaultDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreAddRemoveDefaultDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreAddDupKeyDefaultDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAddNullKeyDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreReadNullKeyDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreRemoveNullKeyDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreNonExistKeyDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreAddRemoveCountDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreContainsDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreAddNullObjectDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreAddNullValueDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreCheckContainsNullDefaultDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Absolute Time Expiry, Add overload"
        [TestMethod]
        public void DBStoreAbsTimeExpiryAddDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddRemoveDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddDupKeyDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryContainsDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddNullObjectTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddNullValueTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryCheckContainsNullTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Sliding Time Expiry, Add overload"
        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddRemoveDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddDupKeyDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryContainsDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddNullObjectDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddNullValueDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryCheckContainsNullDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "File Dependency Expiry, Add overload"
        [TestMethod]
        public void DBStoreFileExpiryAddDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreFileExpiryAddRemoveDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreFileExpiryAddDupKeyDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreFileExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreFileExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreFileExpiryContainsDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreFileExpiryAddNullObjectDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreFileExpiryAddNullValueDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryCheckContainsNullDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Extended Format, Add overload"
        [TestMethod]
        public void DBStoreExtFormatExpiryAddDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddRemoveDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddDupKeyDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreExtFormatExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreExtFormatExpiryContainsDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddNullObjectDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddNullValueDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryCheckContainsNullDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Never Expired, Add overload"
        [TestMethod]
        public void DBStoreNeverExpiredAddDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreNeverExpiredAddRemoveDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreNeverExpiredAddDupKeyDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredAddNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredReadNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreNeverExpiredNonExistKeyDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreNeverExpiredAddRemoveCountDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreNeverExpiredContainsDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreNeverExpiredAddNullObjectDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreNeverExpiredAddNullValueDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredCheckContainsNullDicTest()
        {
            itemCache = factory.Create("DBCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "DB Store with Encryption"
        [TestMethod]
        public void DBStoreAbsTimeExpiryEncryptAddDicTest()
        {
            int count = 0;
            itemCache = factory.Create("DBCacheManagerEncryptionDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            Assert.AreEqual(1, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }
        #endregion

    }
}
