using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.Data;
using System.Data.SqlClient;

namespace CachingDatabaseTests
{
    /// <summary>
    /// Summary description for CachingDBStoreTestFixture
    /// </summary>
    [TestClass]
    public class CachingDBStoreTestFixture
    {
        String connStr;
        private CacheManager itemCache;
        public CachingDBStoreTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            connStr = "server=(local)\\SQLEXPRESS;initial catalog=caching;integrated security=SSPI";
            
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("begin delete from CacheData " +
                                            "end", conn);
            cmd.ExecuteNonQuery();

            conn.Close();

            itemCache = CacheFactory.GetCacheManager("DBCacheManagerOleDb");
            if (itemCache != null)
                itemCache.Flush();

            itemCache = CacheFactory.GetCacheManager("DBCacheManagerEncryption");
            if (itemCache != null)
                itemCache.Flush();

            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            if (itemCache != null)
                itemCache.Flush();
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        
        #region "Default Add overload"
        [TestMethod]
        public void DBStoreAddDefaultTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6') and partitionname = 'Partition1'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6') and partitionname = 'Partition1'", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        // Using Odbc driver require code changes to the application block.
        [TestMethod]
        [ExpectedException(typeof(System.Data.Odbc.OdbcException))]
        public void DBStoreAddDefaultODBCTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManagerODBC");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6') and partitionname = 'Partition2'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6') and partitionname = 'Partition2'", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        // Using OleDb driver require code changes to the application block.
        [TestMethod]
        [ExpectedException(typeof(System.Data.OleDb.OleDbException))]
        public void DBStoreAddDefaultOleDbTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManagerOleDb");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6') and partitionname = 'Partition3'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6') and partitionname = 'Partition3'", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }
        
        [TestMethod]
        public void DBStoreAddRemoveDefaultTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreAddDupKeyDefaultTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAddNullKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreReadNullKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreRemoveNullKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreNonExistKeyDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreAddRemoveCountDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreContainsDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreAddNullObjectDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreAddNullValueDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));

            itemCache.Add("2", null);
            object item = itemCache.GetData("2");
            Assert.IsNull(item);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreCheckContainsNullDefaultTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Absolute Time Expiry, Add overload"
        [TestMethod]
        public void DBStoreAbsTimeExpiryAddTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            
            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1; 
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddRemoveTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            
            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddDupKeyTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails) itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails) itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreAbsTimeExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreAbsTimeExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion
        
        #region "Sliding Time Expiry, Add overload"
        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddRemoveTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddDupKeyTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));

        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreSlidingTimeExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreSlidingTimeExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "File Dependency Expiry, Add overload"
        [TestMethod]
        public void DBStoreFileExpiryAddTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreFileExpiryAddRemoveTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreFileExpiryAddDupKeyTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreFileExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreFileExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreFileExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCachinginDB.txt"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreFileExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreFileExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreFileExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Extended Format, Add overload"
        [TestMethod]
        public void DBStoreExtFormatExpiryAddTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddRemoveTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddDupKeyTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreExtFormatExpiryNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreExtFormatExpiryContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreExtFormatExpiryAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreExtFormatExpiryCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Never Expired, Add overload"
        [TestMethod]
        public void DBStoreNeverExpiredAddTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreNeverExpiredAddRemoveTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(6, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreNeverExpiredAddDupKeyTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1','2','3','4','5','6')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredAddNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredReadNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredRemoveNullKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void DBStoreNeverExpiredNonExistKeyTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void DBStoreNeverExpiredAddRemoveCountTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void DBStoreNeverExpiredContainsTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void DBStoreNeverExpiredAddNullObjectTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void DBStoreNeverExpiredAddNullValueTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void DBStoreNeverExpiredCheckContainsNullTest()
        {
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "DB Store with Encryption"
        [TestMethod]
        public void DBStoreAbsTimeExpiryEncryptAddTest()
        {
            int count = 0;
            itemCache = CacheFactory.GetCacheManager("DBCacheManagerEncryption");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();
            conn.Close();

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            Assert.AreEqual(1, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            conn.Open();
            count = 0;
            cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }
        #endregion

    }
}
