using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
using Ignore = NUnit.Framework.IgnoreAttribute;    
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.IO;
using System.Threading;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
namespace CachingThreadWaitTests
{
    /// <summary>
    /// Summary description for RefreshActionDicTestFixture
    /// </summary>
    [TestClass]
    public class RefreshActionDicTestFixture
    {
        public static bool removed = false;
        private CacheManager itemCache;
        private CacheManagerFactory factory;
        private DictionaryConfigurationSource configSource;
        public RefreshActionDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            CachingDictionarySource sourceSection = new CachingDictionarySource();
            configSource = sourceSection.BuildDictionarySourceSection();

            factory = new CacheManagerFactory(configSource);
            itemCache = factory.Create("InMemoryCacheManager");
            if (itemCache != null)
                itemCache.Flush();

            itemCache = factory.Create("IsolatedCacheManagerDic");
            if (itemCache != null)
                itemCache.Flush();
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Refresh action for Expiration"
        [TestMethod]
        public void DefaultExpiryRefreshActionDicTest()
        {
            removed = false;
            itemCache = factory.Create("InMemoryCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), new FileDependency("FileForCaching.txt"));
            Assert.AreEqual(false, removed);
            File.SetLastWriteTime("FileForCaching.txt", DateTime.Now);
            Thread.Sleep(2000);
            Assert.AreEqual(true, removed);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
        }


        [TestMethod]
        public void IsolatedExpiryRefreshActionDicTest()
        {
            removed = false;
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), new FileDependency("FileForCaching.txt"));
            Assert.AreEqual(false, removed);
            File.SetLastWriteTime("FileForCaching.txt", DateTime.Now);
            Thread.Sleep(2000);
            Assert.AreEqual(true, removed);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);


        }

        #endregion

        #region "Refresh action for Scavenging"
        [TestMethod]
        public void DefaultRefreshActionScavengingDicTest()
        {
            removed = false;
            itemCache = factory.Create("InMemoryCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            Assert.AreEqual(false, removed);

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            Thread.Sleep(1000);
            

            Assert.AreEqual(true, removed);
            writtenItem = (ItemDetails)itemCache.GetData("8");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void NullRefreshActionScavengingDicTest()
        {
            itemCache = factory.Create("InMemoryCacheManager");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            Thread.Sleep(2000);
            
            writtenItem = (ItemDetails)itemCache.GetData("8");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedRefreshActionScavengingDicTest()
        {
            removed = false;
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            Assert.AreEqual(false, removed);

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            Thread.Sleep(1000);
            Assert.AreEqual(true, removed);

            writtenItem = (ItemDetails)itemCache.GetData("8");
            Assert.IsNull(writtenItem);

        }

        #endregion

        [Serializable]
        public class ItemRefreshAction : ICacheItemRefreshAction
        {
            public void Refresh(string removedKey, object expiredValue, CacheItemRemovedReason removalReason)
            {
                removed = true;

            }
        }
    }
}
