using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Configuration;
using System.Security.Cryptography;
namespace CryptoFuncTests
{
    [TestClass]
    public class CryptographerTestFixtureDictionarySource
    {
        private DictionaryConfigurationSource configSource;
        private byte[] PlainByteMessage;
        private String PlainStrMessage;
        private const String DESInstance = "DESCryptoServiceProvider";
        private const String RijndaelManaged = "RijndaelManaged";
        private const String RC2CryptoServiceProvider = "RC2CryptoServiceProvider";
        private const String TripleDESCryptoServiceProvider = "TripleDESCryptoServiceProvider";
        //private const String SHA1Managed = "SHA1Managed";
        public CryptographerTestFixtureDictionarySource()
        {
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
        }
        [TestInitialize()]
        public void Initialize()
        {
            configSource = BuildSymmetrySourceSection();
        }
        [TestCleanup()]
        public void Cleanup()
        {
        }
        # region EncryptionSymmetic Tests
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestForNullCryptoInstance()
        {
            CryptoHelper.Decrypt(null, configSource, PlainByteMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestForNullPlainMessageonBytes()
        {
            CryptoHelper.Encrypt(DESInstance, configSource, (byte[])null);
        }
        [TestMethod]
        
        [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestForCryptoInstanceNotPresentInConfig()
        {
            //try
            //{
                CryptoHelper.Encrypt("NotPresentCryptoInstance", configSource, PlainByteMessage);
            //}
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        # endregion
        # region DecryptSymmetric Tests
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDecryptForNullCryptoInstance()
        {
            CryptoHelper.Decrypt((string)null, configSource, PlainByteMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDecryptForNullCipherMessageAndNullCryptoInstance()
        {
            CryptoHelper.Decrypt((string)null, configSource, (byte[])null);
        }
        [TestMethod]
       [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestDecryptForCryptoInstanceNotPresentInConfig()
        {
            //try
            //{
                CryptoHelper.Decrypt("CryptoInstanceNotPresent", configSource, PlainByteMessage);
            //}
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        [TestMethod]
        public void TestDecryptForValidInputs()
        {
            byte[] encryptedMessage = CryptoHelper.Encrypt(DESInstance, configSource, PlainByteMessage);
            byte[] DecryptedMessage = CryptoHelper.Decrypt(DESInstance, configSource, encryptedMessage);
            Assert.IsTrue(CryptographyUtility.CompareBytes(DecryptedMessage, PlainByteMessage));
        }
        #endregion
        # region Test for encrypting and Decrypting data in form of byte array, using various Symmetric Algorithm
        [TestMethod]
        public void TestOfEncryptionAndDecryptionForAllProvidersUsingByteArrayInputs()
        {
            byte[] cipher1 = CryptoHelper.Encrypt(DESInstance, configSource, PlainByteMessage);
            byte[] cipher2 = CryptoHelper.Encrypt(RC2CryptoServiceProvider, configSource, cipher1);
            byte[] cipher3 = CryptoHelper.Encrypt(RijndaelManaged, configSource, cipher2);
            byte[] cipher4 = CryptoHelper.Encrypt(TripleDESCryptoServiceProvider, configSource, cipher3);
            byte[] Plain4 = CryptoHelper.Decrypt(TripleDESCryptoServiceProvider, configSource, cipher4);
            byte[] Plain3 = CryptoHelper.Decrypt(RijndaelManaged, configSource, Plain4);
            byte[] Plain2 = CryptoHelper.Decrypt(RC2CryptoServiceProvider, configSource, Plain3);
            byte[] Plain1 = CryptoHelper.Decrypt(DESInstance, configSource, Plain2);
            //Test for DESInstance
            Assert.IsTrue(CryptographyUtility.CompareBytes(PlainByteMessage, Plain1));
            //Test for RC2CryptoServiceProvider
            Assert.IsTrue(CryptographyUtility.CompareBytes(cipher1, Plain2));
            //Test for RijndaelManaged
            Assert.IsTrue(CryptographyUtility.CompareBytes(cipher2, Plain3));
            //Test for TripleDESCryptoServiceProvider
            Assert.IsTrue(CryptographyUtility.CompareBytes(cipher3, Plain4));
        }
        # endregion
        # region Default Test
        [TestMethod]
       [ExpectedException(typeof(ConfigurationErrorsException))]
        public void NullSectionDictionarySourceTest()
        {
           // try
           // {
               
                DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            
                CryptographySettings settings = null;
                DictSource.Add("securityCryptographyConfiguration", settings);
                SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(DictSource);
                ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(DESInstance);
        //    }
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        [TestMethod]
       [ExpectedException(typeof(ArgumentNullException))]
        public void NullDictionarySource()
        {
          //  try
           // {
                SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory((IConfigurationSource)null);
                ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(DESInstance);
          //  }
         //   catch (InvalidOperationException e)
          //  {
               // Assert.IsInstanceOfType(e.InnerException, typeof(ArgumentNullException));
          //   }
        }
        [TestMethod]
        public void TestDefaultInstance()
        {
            DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            setting.DefaultSymmetricCryptoProviderName = DESInstance;
            SymmetricAlgorithmProviderData DESCryptoProvider = new SymmetricAlgorithmProviderData();
            DESCryptoProvider.Name = DESInstance;
            DESCryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.DESCryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            DESCryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            byte[] key = new byte[8];
            CryptographyUtility.GetRandomBytes(key);
            DESCryptoProvider.ProtectedKeyFilename = @"DesCryptoserviceProvider.key";
            DESCryptoProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
            setting.SymmetricCryptoProviders.Add(DESCryptoProvider);
            DictSource.Add("securityCryptographyConfiguration", setting);
            //Here the default instance is DESInstance
            SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(DictSource);
            ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(DESInstance);
            ISymmetricCryptoProvider providerDefault = (ISymmetricCryptoProvider)factory.CreateDefault();
            //both the provider uses the same algorithm and same key here
            byte[] CipherTextDesDefault = providerDefault.Encrypt(PlainByteMessage);
            // encrypt using default provider
            //decrypt using DESInstance
            byte[] PlainDes = provider.Decrypt(CipherTextDesDefault);
            Assert.IsTrue(CryptographyUtility.CompareBytes(PlainByteMessage, PlainDes));
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDefaultInstanceWhichIsNotSet()
        {
            DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            //Default Instance Not set
            //setting.DefaultSymmetricCryptoProviderName = DESInstance;
            SymmetricAlgorithmProviderData DESCryptoProvider = new SymmetricAlgorithmProviderData();
            DESCryptoProvider.Name = DESInstance;
            DESCryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.DESCryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            DESCryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            DESCryptoProvider.ProtectedKeyFilename = @"DesCryptoserviceProvider.key";
            DESCryptoProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
           
            setting.SymmetricCryptoProviders.Add(DESCryptoProvider);
            DictSource.Add("securityCryptographyConfiguration", setting);
            //Here the default instance is DESInstance
            SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(DictSource);
            ISymmetricCryptoProvider providerDefault = (ISymmetricCryptoProvider)factory.CreateDefault();
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDefaultInstanceWhichIsSetToNull()
        {
            DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            //DEfault InstanceNot set
            setting.DefaultSymmetricCryptoProviderName = null;
            SymmetricAlgorithmProviderData DESCryptoProvider = new SymmetricAlgorithmProviderData();
            DESCryptoProvider.Name = DESInstance;
            DESCryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.DESCryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            DESCryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            DESCryptoProvider.ProtectedKeyFilename = @"DesCryptoserviceProvider.key";
            DESCryptoProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
            setting.SymmetricCryptoProviders.Add(DESCryptoProvider);
            DictSource.Add("securityCryptographyConfiguration", setting);
            //Here the default instance is DESInstance
            SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(DictSource);
            ISymmetricCryptoProvider providerDefault = (ISymmetricCryptoProvider)factory.CreateDefault();
        }
        [TestMethod]
     [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestDefaultInstanceWhichIsNotPresentInConfigurationSource()
        {
            //try
            //{
                DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
                CryptographySettings setting = new CryptographySettings();
                //DEfault Instance Not set
                setting.DefaultSymmetricCryptoProviderName = "DefaulInstanceTypeIsMissing";
                SymmetricAlgorithmProviderData DESCryptoProvider = new SymmetricAlgorithmProviderData();
                DESCryptoProvider.Name = DESInstance;
                DESCryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.DESCryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
                DESCryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
                DESCryptoProvider.ProtectedKeyFilename = @"DesCryptoserviceProvider.key";
                DESCryptoProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
                setting.SymmetricCryptoProviders.Add(DESCryptoProvider);
                DictSource.Add("securityCryptographyConfiguration", setting);
                SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(DictSource);
                ISymmetricCryptoProvider providerDefault = (ISymmetricCryptoProvider)factory.CreateDefault();
            //}
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        #endregion
       // # region Provider Creation using Symmetric Provider Data
       // [TestMethod]
       // public void TestOfCreationOfProviderUsingSymmetricProviderData()
       // {
       //     SymmetricAlgorithmProviderData DESCryptoProvider = new SymmetricAlgorithmProviderData();
       //     DESCryptoProvider.Name = DESInstance;
       //     DESCryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.DESCryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
       //     DESCryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
       //     byte[] key = new byte[8];
       //     CryptographyUtility.GetRandomBytes(key);
       //     DESCryptoProvider.Key = key;
       //     SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory();
       //     ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(DESCryptoProvider);
       //     Assert.IsNotNull(provider);
            
       // }
       // [TestMethod]
       // public void TestOfCreationOfProviderUsingNullSymmetricProviderData()
       // {
       //     SymmetricAlgorithmProviderData DESCryptoProvider = null;
       //     SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory();
       //     ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(DESCryptoProvider);
            
       // }
       //# endregion
        # region HelperMethod
        private bool CompareByteArrays(byte[] Array1, byte[] Array2)
        {
            if (Array1.GetUpperBound(0) != Array2.GetUpperBound(0))
            {
                return (false);
            }
            for (int index1 = 0; index1 < Array1.GetUpperBound(0); index1++)
            {
                if (Array1[index1] != Array2[index1])
                {
                    return (false);
                }
            }
            return (true);
        }
        # endregion
        #region Build the DictionaryConfiguration Source
        private DictionaryConfigurationSource BuildSymmetrySourceSection()
        {
            DictionaryConfigurationSource section = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            setting.DefaultSymmetricCryptoProviderName = DESInstance;
            SymmetricAlgorithmProviderData DESCryptoProvider = new SymmetricAlgorithmProviderData();
            DESCryptoProvider.Name = DESInstance;
            DESCryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.DESCryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            DESCryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            DESCryptoProvider.ProtectedKeyFilename = @"DesCryptoserviceProvider.key";
            DESCryptoProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
            SymmetricAlgorithmProviderData RijndaelManagedProvider = new SymmetricAlgorithmProviderData();
            RijndaelManagedProvider.Name = RijndaelManaged;
            RijndaelManagedProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.RijndaelManaged, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            RijndaelManagedProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            RijndaelManagedProvider.ProtectedKeyFilename = @"Rijndeal.key";
            RijndaelManagedProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
            SymmetricAlgorithmProviderData RC2CryptoProvider = new SymmetricAlgorithmProviderData();
            RC2CryptoProvider.Name = RC2CryptoServiceProvider;
            RC2CryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.RC2CryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            RC2CryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            RC2CryptoProvider.ProtectedKeyFilename = @"RC2CryptoServiceProvider.key";
            RC2CryptoProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
            SymmetricAlgorithmProviderData TDESCryptoProvider = new SymmetricAlgorithmProviderData();
            TDESCryptoProvider.Name = TripleDESCryptoServiceProvider;
            TDESCryptoProvider.AlgorithmType = Type.GetType("System.Security.Cryptography.TripleDESCryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            TDESCryptoProvider.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.SymmetricAlgorithmProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            TDESCryptoProvider.ProtectedKeyFilename = @"tripleDesCryptoServiceProvider.key";
            TDESCryptoProvider.ProtectedKeyProtectionScope = DataProtectionScope.LocalMachine;
            setting.SymmetricCryptoProviders.Add(DESCryptoProvider);
            setting.SymmetricCryptoProviders.Add(RijndaelManagedProvider);
            setting.SymmetricCryptoProviders.Add(RC2CryptoProvider);
            setting.SymmetricCryptoProviders.Add(TDESCryptoProvider);
            section.Add("securityCryptographyConfiguration", setting);
            return section;
        }
        #endregion
    }
}
