using System;
using System.Text;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Collections.Generic;
namespace CryptoFuncTests
{
  
    [TestClass]
    public class CryptographerTestFixtureSystemSource
    {
        private byte[] PlainByteMessage;
        private String PlainStrMessage;
        private const String DESInstance = "DESCryptoServiceProvider";
        private const String RijndaelManaged = "RijndaelManaged";
        private const String RC2CryptoServiceProvider = "RC2CryptoServiceProvider";
        private const String TripleDESCryptoServiceProvider = "TripleDESCryptoServiceProvider";
        private const String SHA1Managed = "SHA1Managed";
        public CryptographerTestFixtureSystemSource()
        {
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
        }
        [TestInitialize()]
        public void Initialize()
        {
        }
        [TestCleanup()]
        public void Cleanup()
        {
            
        }
        # region EncryptionSymmetic Tests
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestForNullCryptoInstance()
        {
            CryptoHelper.Decrypt(null, null, PlainByteMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestForNullPlainMessage()
        {
            CryptoHelper.Encrypt(DESInstance, (String)null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestForNullPlainMessageonBytes()
        {
           
            CryptoHelper.Encrypt(DESInstance, null, (byte[])null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestForEmptyPlainMessage()
        {
            CryptoHelper.Encrypt(DESInstance, String.Empty);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestForNullPlainMessageAndNullCryptoInstance()
        {
            CryptoHelper.Encrypt(DESInstance, (String)null);
        }
        [TestMethod]
       [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestForCryptoInstanceNotPresentInConfig()
        {
        //    try
        //    {
                CryptoHelper.Encrypt("NotPresentCryptoInstance", null, PlainByteMessage);
            //}
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        [TestMethod]
        public void TestEncryptionForValidInputs()
        {
            String encryptedStrMessage = CryptoHelper.Encrypt(DESInstance, PlainStrMessage);
            Assert.IsFalse(encryptedStrMessage.Equals(PlainStrMessage));
        }
        # endregion
        # region DecryptSymmetric Tests
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDecryptForNullCryptoInstance()
        {
            CryptoHelper.Decrypt((string)null, null, PlainByteMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDecryptForNullCipherMessage()
        {
            CryptoHelper.Decrypt(DESInstance, (String)null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDecryptForNullCipherMessageAndNullCryptoInstance()
        {
            CryptoHelper.Decrypt((string)null, null, (byte[])null);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDecryptForEmptyCipherMessage()
        {
            CryptoHelper.Decrypt(DESInstance, String.Empty);
        }
        [TestMethod]
       [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestDecryptForCryptoInstanceNotPresentInConfig()
        {
            //try
            //{
                CryptoHelper.Decrypt("CryptoInstanceNotPresent", null, PlainByteMessage);
            //}
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }

        
        [TestMethod]
        public void TestDecryptForValidInputs()
        {
            String encryptedStrMessage = CryptoHelper.Encrypt(DESInstance, PlainStrMessage);
            String DecryptedMessage = CryptoHelper.Decrypt(DESInstance, encryptedStrMessage);
            Assert.IsTrue(DecryptedMessage == PlainStrMessage);
        }
        #endregion
        # region Test for encrypting and Decrypting data in form of string array, using various Symmetric Algorithm
        [TestMethod]
        public void TestOfEncryptionAndDecryptionForAllProvidersUsingStringInputs()
        {
            String cipher1 = CryptoHelper.Encrypt(DESInstance, PlainStrMessage);
            String cipher2 = CryptoHelper.Encrypt(RC2CryptoServiceProvider, cipher1);
            String cipher3 = CryptoHelper.Encrypt(RijndaelManaged, cipher2);
            String cipher4 = CryptoHelper.Encrypt(TripleDESCryptoServiceProvider, cipher3);
            String Plain4 = CryptoHelper.Decrypt(TripleDESCryptoServiceProvider, cipher4);
            String Plain3 = CryptoHelper.Decrypt(RijndaelManaged, Plain4);
            String Plain2 = CryptoHelper.Decrypt(RC2CryptoServiceProvider, Plain3);
            String Plain1 = CryptoHelper.Decrypt(DESInstance, Plain2);
            //Test for DESInstance
            Assert.AreEqual(PlainStrMessage, Plain1);
            //Test for RC2CryptoServiceProvider
            Assert.AreEqual(cipher1, Plain2);
            //Test for RijndaelManaged
            Assert.AreEqual(cipher2, Plain3);
            //Test for TripleDESCryptoServiceProvider
            Assert.AreEqual(cipher3, Plain4);
        }
        #endregion
        # region Test for encrypting and Decrypting data in form of byte array, using various Symmetric Algorithm
        [TestMethod]
        public void TestOfEncryptionAndDecryptionForAllProvidersUsingByteArrayInputs()
        {
            byte[] cipher1 = CryptoHelper.Encrypt(DESInstance, null, PlainByteMessage);
            byte[] cipher2 = CryptoHelper.Encrypt(RC2CryptoServiceProvider, null, cipher1);
            byte[] cipher3 = CryptoHelper.Encrypt(RijndaelManaged, null, cipher2);
            byte[] cipher4 = CryptoHelper.Encrypt(TripleDESCryptoServiceProvider, null, cipher3);
            byte[] Plain4 = CryptoHelper.Decrypt(TripleDESCryptoServiceProvider, null, cipher4);
            byte[] Plain3 = CryptoHelper.Decrypt(RijndaelManaged, null, Plain4);
            byte[] Plain2 = CryptoHelper.Decrypt(RC2CryptoServiceProvider, null, Plain3);
            byte[] Plain1 = CryptoHelper.Decrypt(DESInstance, null, Plain2);
            //Test for DESInstance
            Assert.IsTrue(this.CompareByteArrays(PlainByteMessage, Plain1) == true);
            //Test for RC2CryptoServiceProvider
            Assert.IsTrue(this.CompareByteArrays(cipher1, Plain2) == true);
            //Test for RijndaelManaged
            Assert.IsTrue(this.CompareByteArrays(cipher2, Plain3) == true);
            //Test for TripleDESCryptoServiceProvider
            Assert.IsTrue(this.CompareByteArrays(cipher3, Plain4) == true);
        }
        #endregion
        # region Default Test
     
        [TestMethod]
        public void DefaultCrptoInstanceTest()
        {
            // The default crypto instance in configuration file is set to 
            // DesCryptoServiceProvider
            // In this test, Plain test is encrypted using default crypto instance
            // and decrypted using the known crypto instance(i.e. by mentioning DESinstance explicitly
            SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(new SystemConfigurationSource());
            ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(DESInstance);
          
            ISymmetricCryptoProvider providerDefault = (ISymmetricCryptoProvider)factory.CreateDefault();
            byte[] EncryptedMessage = providerDefault.Encrypt(PlainByteMessage);
            byte[] DecryptedMessage = provider.Decrypt(EncryptedMessage);
            Assert.IsTrue(CryptographyUtility.CompareBytes(PlainByteMessage, DecryptedMessage));
        }

        [TestMethod]
        public void TestingRijndael128bitEncryption()
        {
            SymmetricCryptoProviderFactory fact = new SymmetricCryptoProviderFactory(new SystemConfigurationSource());
            ISymmetricCryptoProvider prov = fact.Create("RijndaelManaged128");
            byte[] plain = new byte[16];
            CryptographyUtility.GetRandomBytes(plain);
            byte[] cipher = prov.Encrypt(plain);
            byte[] deciphered = prov.Decrypt(cipher);
            CryptographyUtility.CompareBytes(plain, deciphered);

        }

        [TestMethod]
        public void TestingRijndael192bitEncryption()
        {
            SymmetricCryptoProviderFactory fact = new SymmetricCryptoProviderFactory(new SystemConfigurationSource());
            ISymmetricCryptoProvider prov = fact.Create("RijndaelManaged192");
            byte[] plain = new byte[16];
            CryptographyUtility.GetRandomBytes(plain);
            byte[] cipher = prov.Encrypt(plain);
            byte[] deciphered = prov.Decrypt(cipher);
            CryptographyUtility.CompareBytes(plain, deciphered);
        }

        [TestMethod]
        public void TestingRijndael256bitEncryption()
        {
            SymmetricCryptoProviderFactory fact = new SymmetricCryptoProviderFactory(new SystemConfigurationSource());
            ISymmetricCryptoProvider prov = fact.Create("RijndaelManaged256");
            byte[] plain = new byte[16];
            CryptographyUtility.GetRandomBytes(plain);
            byte[] cipher = prov.Encrypt(plain);
            byte[] deciphered = prov.Decrypt(cipher);
            CryptographyUtility.CompareBytes(plain, deciphered);
        }

        [TestMethod]
        public void RijndaelManagedMachineModeStorageEncryption()
        {
            SymmetricCryptoProviderFactory fact = new SymmetricCryptoProviderFactory(new SystemConfigurationSource());
            ISymmetricCryptoProvider prov = fact.Create("RijndaelManagedMachineModeStorage");
            byte[] plain = new byte[16];
            CryptographyUtility.GetRandomBytes(plain);
            byte[] cipher = prov.Encrypt(plain);
            byte[] deciphered = prov.Decrypt(cipher);
            CryptographyUtility.CompareBytes(plain, deciphered);
        }

        # endregion
        # region HelperMethod
        private bool CompareByteArrays(byte[] Array1, byte[] Array2)
        {
            if (Array1.GetUpperBound(0) != Array2.GetUpperBound(0))
            {
                return (false);
            }
            for (int index1 = 0; index1 < Array1.GetUpperBound(0); index1++)
            {
                if (Array1[index1] != Array2[index1])
                {
                    return (false);
                }
            }
            return (true);
        }
        # endregion
    }
}
