using System;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Security.Cryptography;
namespace CryptoFuncTests
{
   
    [TestClass]
    public class DPAPIFixtureDictionarySource
    {
        private byte[] PlainByteMessage;
        private String PlainStrMessage;
        private const String DPAPISymmetricCryptographyProvider = "DPAPISymmetricCryptographyProvider";
        private const String DPAPISymmetricCryptographyProviderMachineMode = "DPAPISymmetricCryptographyProviderMachineMode";
        //private const String MachineModeFile = @"c:\MachineModeDpapi.Txt";
        //private const String UserModeFile = @"c:\UserModeDpapi.Txt";
        private DictionaryConfigurationSource configSource;
        public DPAPIFixtureDictionarySource()
        {
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
        }
        [TestInitialize]
        public void  InitializeConfigSource()
        {
            configSource = BuildSymmetrySourceSection();
        }
        # region EncryptionSymmetic Tests For DPAPIUserMode
        
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDPAPIForNullPlainMessageonBytes()
        {
            CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider,configSource, (byte[])null);
        }
    
          
        [TestMethod]
        public void TestDPAPIEncryptionForValidInputsInByteArrays()
        {
            byte[] encryptedbytMessage = CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider,configSource, PlainByteMessage);
          
            Assert.IsFalse(CryptographyUtility.CompareBytes(encryptedbytMessage,PlainByteMessage));
        }
        # endregion
        //# region EncryptionSymmetic Tests For DPAPISymmetricCryptographyProviderMachineMode
        //[TestMethod]
        //[ExpectedException(typeof(ArgumentNullException))]
        //public void TestDPAPIMachineModeForNullPlainMessageonBytes()
        //{
        //    CryptoHelper.Encrypt(DPAPISymmetricCryptographyProviderMachineMode,configSource, (byte[])null);
        //}
       
        //# endregion
        #region DecryptSymmetric Tests For Dpapi User Mode
   
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestDpapiDecryptForNullCipherMessage()
        {
            CryptoHelper.Decrypt(DPAPISymmetricCryptographyProvider,configSource, (byte[])null);
        }
  
      
        [TestMethod]
        public void TestDpapiDecryptForValidInputsAsByte()
        {
           byte[] encryptedBytMessage = CryptoHelper.Encrypt(DPAPISymmetricCryptographyProvider,configSource,PlainByteMessage);
           
            byte[] decryptedBytes=CryptoHelper.Decrypt(DPAPISymmetricCryptographyProvider,configSource,encryptedBytMessage);
            Assert.IsTrue(CryptographyUtility.CompareBytes(decryptedBytes, PlainByteMessage));
        }
        #endregion
        //#region DecryptSymmetric Tests For Dpapi Machine Mode
     
        //[TestMethod]
        //[ExpectedException(typeof(ArgumentNullException))]
        //public void TestDpapiMachineModeDecryptForNullCipherMessage()
        //{
        //    CryptoHelper.Decrypt(DPAPISymmetricCryptographyProviderMachineMode,configSource, (byte[])null);
        //}
        //[TestMethod]
        //public void TestDPAPIMachineModeEncryptionForValidInputsInByteArrays()
        //{
        //    byte[] encryptedbytMessage = CryptoHelper.Encrypt(DPAPISymmetricCryptographyProviderMachineMode, configSource, PlainByteMessage);
        //    Assert.IsFalse(CryptographyUtility.CompareBytes(encryptedbytMessage, PlainByteMessage));
        //}
     
        //[TestMethod]
        //public void TestDpapiDecryptMachineModeForValidInputsAsByte()
        //{
        //    byte[] encryptedBytMessage = CryptoHelper.Encrypt(DPAPISymmetricCryptographyProviderMachineMode, configSource, PlainByteMessage);
        //    Assert.IsTrue(CryptographyUtility.CompareBytes((CryptoHelper.Decrypt(DPAPISymmetricCryptographyProviderMachineMode, configSource, encryptedBytMessage)), PlainByteMessage));
        //}
        //# endregion
        
        #region Build the DictionaryConfiguration Source
        private DictionaryConfigurationSource BuildSymmetrySourceSection()
        {
            DictionaryConfigurationSource section = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            DpapiSymmetricCryptoProviderData dpApi = new DpapiSymmetricCryptoProviderData();
            dpApi.Scope = DataProtectionScope.CurrentUser;
            dpApi.Name = DPAPISymmetricCryptographyProvider;
            dpApi.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.DpapiSymmetricCryptoProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            setting.SymmetricCryptoProviders.Add(dpApi);
            DpapiSymmetricCryptoProviderData dpApiMachineMode = new DpapiSymmetricCryptoProviderData();
            dpApiMachineMode.Scope = DataProtectionScope.LocalMachine;
            byte[] entropy = new byte[16];
            CryptographyUtility.GetRandomBytes(entropy);
           
            dpApiMachineMode.Name = DPAPISymmetricCryptographyProviderMachineMode;
            dpApiMachineMode.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.DpapiSymmetricCryptoProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cryptography");
            setting.SymmetricCryptoProviders.Add(dpApiMachineMode);
           section.Add("securityCryptographyConfiguration", setting);
            return section;
        }
        #endregion
    }
}
