using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace CryptoFuncTests
{
    /// <summary>
    /// Summary description for CryptographerHashTestFixture
    /// Test using different hash providers for creating and comparing hash, returns the comparison false. 
    /// Use CreateHash method for creating hash for data in form of string or byte array, using various Hash and Keyed Hash Algorithms. With salt
    /// Use CompareHash method for comparing data in form of string or byte array with existing hash, using various Hash and Keyed Hash Algorithms. With Salt 
    /// Use CompareHash method for comparing data in form of string or byte array with existing hash, using various Hash and Keyed Hash Algorithms. Without Salt 
    /// </summary>
    [TestClass]
    public class CryptographerHashTestFixtureSystemSource
    {
        private byte[] PlainByteMessage;
        private String PlainStrMessage;
        private const String HMACMD5KeyedHashNoSalt = "HMACMD5KeyedHashNoSalt";
        private const String HMACMD5KeyedHash = "HMACMD5KeyedHash";
        private const String SHA1ManagedNoSalt = "SHA1ManagedNoSalt";
        private const String SHA1Managed = "SHA1Managed";
        private const String DPAPISymmetricCryptographyProvider = "DPAPISymmetricCryptographyProvider";
       
        public CryptographerHashTestFixtureSystemSource()
        {
          
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
           
        }
        
        [TestInitialize()]
        public void Initialize()
        {
        
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
        }
        
        [TestCleanup()]
        public void Cleanup()
        {
          
        }
        #region TestCases for hashing

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHashingtForNullCryptoInstance()
        {
            CryptoHelper.CreateHash((String)null, null, PlainByteMessage);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHashingForEmptyCryptoInstance()
        {
            CryptoHelper.CreateHash(String.Empty, null, PlainByteMessage);
        }
    
        [TestMethod]
        [ExpectedException(typeof(System.NullReferenceException))]
        public void TestHashingForNullPlainMessageInByte()
        {
            CryptoHelper.CreateHash(SHA1Managed, null, (byte[])null);
        }
     
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestHashingForNullPlainMessage()
        {
            CryptoHelper.CreateHash(SHA1Managed, (String)null);
        }
    
        [TestMethod]
     [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestHahingForCryptoInstanceNotPresentInConfig()
        {
            //try
            //{
                CryptoHelper.CreateHash("InstanceNotpresent", null, PlainByteMessage);
            //}
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        #endregion
        # region Tests for CompareHash
   
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareHashForNullHashInstance()
        {
            CryptoHelper.CompareHash(null, null, PlainByteMessage, Cryptographer.CreateHash(SHA1Managed, PlainByteMessage));
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareHashForEmptyCryptoInstance()
        {
            CryptoHelper.CompareHash(String.Empty, null, PlainByteMessage, Cryptographer.CreateHash(SHA1Managed, PlainByteMessage));
        }
     
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCompareHashForNullPlainMessageInByte()
        {
            CryptoHelper.CompareHash(SHA1Managed, null, (byte[])null, PlainByteMessage);
        }
     
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCompareHashForNullPlainMessage()
        {
            CryptoHelper.CompareHash(SHA1Managed, (String)null, PlainStrMessage);
        }
       
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCompareHashForNullHashMessage()
        {
            CryptoHelper.CompareHash(SHA1Managed, null, PlainByteMessage, (byte[])null);
        }
       
        [TestMethod]
        public void TestCompareHashForValidInput()
        {
            byte[] hash = CryptoHelper.CreateHash(SHA1Managed, null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash(SHA1Managed, null, PlainByteMessage, hash) == true);
        }
        #endregion
        # region Tests for all the Hash Providers
      
        [TestMethod]
        public void TestHashingForCryptoSHA512ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA512ManagedWithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA512ManagedWithNoSalt", null, PlainByteMessage, hash));
        }
      
        [TestMethod]        public void TestHashingForCryptoSHA512ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA512ManagedWithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA512ManagedWithSalt", null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoSHA384ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA384ManagedWithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA384ManagedWithNoSalt",null, PlainByteMessage, hash));
        }
      
        [TestMethod]
        public void TestHashingForCryptoSHA384ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA384ManagedWithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA384ManagedWithSalt",null,PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoSHA256ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA256ManagedWithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA256ManagedWithNoSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoSHA256ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA256ManagedWithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA256ManagedWithSalt",null, PlainByteMessage, hash));
        }
        
        [TestMethod]
        public void TestHashingForCryptoSHA1ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA1ManagedWithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA1ManagedWithNoSalt",null, PlainByteMessage, hash));
        }
        
        [TestMethod]
        public void TestHashingForCryptoSHA1ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA1ManagedWithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA1ManagedWithSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoRIPEMD160ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("RIPEMD160ManagedWithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("RIPEMD160ManagedWithNoSalt",null, PlainByteMessage, hash));
        }
        
        [TestMethod]
        public void TestHashingForCryptoRIPEMD160ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("RIPEMD160ManagedWithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("RIPEMD160ManagedWithSalt",null, PlainByteMessage, hash));
        }
        
        [TestMethod]
        public void TestHashingForCryptoMD5CryptoServiceProviderWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MD5CryptoServiceProviderWithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MD5CryptoServiceProviderWithNoSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoMD5CryptoServiceProviderWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MD5CryptoServiceProviderWithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MD5CryptoServiceProviderWithSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoMACTripleDESWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MACTripleDESWithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MACTripleDESWithNoSalt",null, PlainByteMessage, hash));
        }
        
        [TestMethod]
        public void TestHashingForCryptoMACTripleDESWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MACTripleDESWithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MACTripleDESWithSalt",null, PlainByteMessage, hash));
        }
        
        [TestMethod]
        public void TestHashingForCryptoHMACSHA512WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA512WithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA512WithNoSalt",null, PlainByteMessage, hash));
        }
        
        [TestMethod]
        public void TestHashingForCryptoHMACSHA512WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA512WithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA512WithSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACSHA384WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA384WithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA384WithNoSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACSHA384WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA384WithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA384WithSalt",null, PlainByteMessage, hash));
        }
      
        [TestMethod]
        public void TestHashingForCryptoHMACSHA256WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA256WithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA256WithNoSalt",null, PlainByteMessage, hash));
        }
      
        [TestMethod]
        public void TestHashingForCryptoHMACSHA256WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA256WithSalt", null,PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA256WithSalt", null,PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACMD5WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACMD5WithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(Cryptographer.CompareHash("HMACMD5WithNoSalt", PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACMD5WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACMD5WithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACMD5WithSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACRIPEMD160WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACRIPEMD160WithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACRIPEMD160WithNoSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACRIPEMD160WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACRIPEMD160WithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACRIPEMD160WithSalt",null, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA1WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA1WithNoSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA1WithNoSalt",null, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACSHA1WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA1WithSalt",null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA1WithSalt",null, PlainByteMessage, hash));
        }


        [TestMethod]
        public void TestHashingForCryptoHMACSHA512MachineModeStorage()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA512MachineMode", null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA512MachineMode", null, PlainByteMessage, hash));
        }  
        #endregion
        # region  Testing CreateHash and Compare method for creating hash for data in form of string or byte array, using various Hash and Keyed Hash Algorithms. With salt
       
        [TestMethod]
        public void TestingCreateHashCompareHashMethodUsingDataInTheFormOfStringUsingVariousAlgorithms()
        {
            String Hash1 = CryptoHelper.CreateHash(SHA1Managed, PlainStrMessage);
            String Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt, Hash1);
            String Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash, Hash2);
            String Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt, Hash3);
            // Test for CreateHash
            Assert.IsTrue(Hash1 != PlainStrMessage);
            Assert.IsTrue(Hash2 != Hash1);
            Assert.IsTrue(Hash3 != Hash2);
            Assert.IsTrue(Hash4 != Hash3);
        }
      
        [TestMethod]
        public void TestCompareHashMethodUsingDataInTheFormOfStringUsingVariousAlgorithmsForValidComparision()
        {
            String Hash1 = CryptoHelper.CreateHash(SHA1Managed, PlainStrMessage);
            String Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt, Hash1);
            String Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash, Hash2);
            String Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt, Hash3);
            Assert.IsTrue(CryptoHelper.CompareHash(SHA1Managed,PlainStrMessage,Hash1));
            Assert.IsTrue(CryptoHelper.CompareHash(SHA1ManagedNoSalt,Hash1,Hash2));
            Assert.IsTrue(CryptoHelper.CompareHash(HMACMD5KeyedHash,Hash2,Hash3));
            Assert.IsTrue(CryptoHelper.CompareHash(HMACMD5KeyedHashNoSalt,Hash3,Hash4));
            
      
        }
        //In this test MessageDigest is  computed with one Method And Compared Using Another Method
       
        [TestMethod]
        public void TestCompareHashMethodUsingDataInTheFormOfStringUsingVariousAlgorithmsForInValidComparision()
        {
         
            
            String Hash1 = CryptoHelper.CreateHash(SHA1Managed, PlainStrMessage);
            String Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt, Hash1);
            String Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash, Hash2);
            String Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt, Hash3);
            Assert.IsFalse(CryptoHelper.CompareHash(HMACMD5KeyedHashNoSalt, PlainStrMessage, Hash1));
            Assert.IsFalse(CryptoHelper.CompareHash(HMACMD5KeyedHash, Hash1, Hash2));
            Assert.IsFalse(CryptoHelper.CompareHash(SHA1ManagedNoSalt, Hash2, Hash3));
            Assert.IsFalse(CryptoHelper.CompareHash(SHA1Managed, Hash3, Hash4));
        }
       
        [TestMethod]
        public void TestingCreateHashCompareHashMethodUsingDataInTheFormOfBytesUsingVariousAlgorithms()
        {
            byte[] Hash1 = CryptoHelper.CreateHash(SHA1Managed,null, PlainByteMessage);
            byte[] Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt,null, Hash1);
            byte[] Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash,null, Hash2);
            byte[] Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt,null, Hash3);
            // Test for CreateHash
            Assert.IsFalse(this.CompareByteArrays(Hash1, PlainByteMessage));
            Assert.IsFalse(this.CompareByteArrays(Hash2, Hash1));
            Assert.IsFalse(this.CompareByteArrays(Hash3, Hash2));
            Assert.IsFalse(this.CompareByteArrays(Hash4, Hash3));
        }
       
        [TestMethod]
        public void TestCompareHashMethodUsingDataInTheFormOfBytesUsingVariousAlgorithmsForValidComparision()
        {
            byte[] Hash1 = CryptoHelper.CreateHash(SHA1Managed,null, PlainByteMessage);
            byte[] Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt,null, Hash1);
            byte[] Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash,null, Hash2);
            byte[] Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt,null, Hash3);
            Assert.IsTrue(CryptoHelper.CompareHash(SHA1Managed,null, PlainByteMessage, Hash1));
            Assert.IsTrue(CryptoHelper.CompareHash(SHA1ManagedNoSalt,null, Hash1, Hash2));
            Assert.IsTrue(CryptoHelper.CompareHash(HMACMD5KeyedHash,null, Hash2, Hash3));
            Assert.IsTrue(CryptoHelper.CompareHash(HMACMD5KeyedHashNoSalt,null, Hash3, Hash4));
        }
        //In this test MessageDigest is  computed with one Method And Compared Using Another Method
       
        [TestMethod]
        public void TestCompareHashMethodUsingDataInTheFormOfBytesUsingVariousAlgorithmsForInValidComparision()
        {
            byte[] Hash1 = Cryptographer.CreateHash(SHA1Managed, PlainByteMessage);
            byte[] Hash2 = Cryptographer.CreateHash(SHA1ManagedNoSalt, Hash1);
            byte[] Hash3 = Cryptographer.CreateHash(HMACMD5KeyedHash, Hash2);
            byte[] Hash4 = Cryptographer.CreateHash(HMACMD5KeyedHashNoSalt, Hash3);
            Assert.IsFalse(Cryptographer.CompareHash(HMACMD5KeyedHashNoSalt, PlainByteMessage, Hash1));
            Assert.IsFalse(Cryptographer.CompareHash(HMACMD5KeyedHash, Hash1, Hash2));
            Assert.IsFalse(Cryptographer.CompareHash(SHA1ManagedNoSalt, Hash2, Hash3));
            Assert.IsFalse(Cryptographer.CompareHash(SHA1Managed, Hash3, Hash4));
            
        }     
        
        # endregion
       
        # region HelperMethod
        private bool CompareByteArrays(byte[] Array1, byte[] Array2)
        {
            if (Array1.GetUpperBound(0) != Array2.GetUpperBound(0))
            {
                return (false);
            }
            for (int index1 = 0; index1 < Array1.GetUpperBound(0); index1++)
            {
                if (Array1[index1] != Array2[index1])
                {
                    return (false);
                }
            }
            return (true);
        }
        # endregion
    }
}
