﻿// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Configuration; 
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql; 
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration; 
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;    
using System.Reflection;
namespace DataFuncTests
{
    /// <summary>
    ///This is a test class for Microsoft.Practices.EnterpriseLibrary.Data.ConnectionString and is intended
    ///to contain all Microsoft.Practices.EnterpriseLibrary.Data.ConnectionString Unit Tests
    ///</summary>
    [TestClass()]
    public class CommonTestsFixture
    {
        string connStr;
        [TestInitialize()]
        public void MyTestInitialize()
        {
            connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString;
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Delete from TestNullTable", conn);
            cmd.ExecuteNonQuery();
            conn.Close();

        }   

       #region Connection String Tests

        [TestMethod()]
        public void ConnectionStringConstructorTest()
        {
            string connectionString =ConfigurationManager.ConnectionStrings["OracleTest"].ConnectionString;
            string userIdTokens = "user id=,uid=";
            string passwordTokens = "password=,pwd=";
            ConnectionString target = new ConnectionString(connectionString, userIdTokens, passwordTokens);
            Assert.AreEqual<string>(connectionString,target.ToString());
            Assert.AreEqual("testone", target.Password);
            Assert.AreEqual("testone", target.UserName);
        }

        [TestMethod()]
        public void CreateNewConnectionStringTest()
        {
            string connectionString = ConfigurationManager.ConnectionStrings["OracleTest"].ConnectionString;
            
            string userIdTokens = "user id=,uid=";
            string passwordTokens = "password=,pwd=";
            ConnectionString target = new ConnectionString(connectionString, userIdTokens, passwordTokens);
            ConnectionString actual = target.CreateNewConnectionString(connectionString);
            Assert.AreEqual("testone",actual.Password);
            Assert.AreEqual("testone",actual.UserName);

           
        }
       [TestMethod()]
        public void ConnectionStringWithoutCredentialsTest()
        {
            string connectionString = "server=newtest;user id=testone;password=testone;"; // TODO: Initialize to an appropriate value
            string userIdTokens = "testone";
            string passwordTokens = "testone"; 
            ConnectionString target = new ConnectionString(connectionString, userIdTokens, passwordTokens);
            string sAc1 = target.ToStringNoCredentials();
           Assert.AreEqual(connectionString, sAc1);
           string cString1 = "server=newtest;user id=u;password=p;";
           ConnectionString target1 = new ConnectionString(cString1, "u", "p");
           string sAc2 = target1.ToStringNoCredentials();
           Assert.AreEqual("server=newtest;", sAc2);
       }
        #endregion

       #region Null Values tests
       [TestMethod]
       public void NullValueWithDbTypeTest()
       {
           Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
           int? age = null;
           DbCommand dbIns = db.GetStoredProcCommand("TestNullType");
           db.AddInParameter(dbIns, "@eno", DbType.Int32, 1);
           db.AddInParameter(dbIns, "@ename", DbType.String, "Narayan");
           db.AddInParameter(dbIns, "@age", DbType.Int32, age);
           db.ExecuteNonQuery(dbIns);
           int noofRows = (int)db.ExecuteScalar(CommandType.Text, "select count(eno) from TestNullTable");
           Assert.AreEqual(noofRows, 1);
       }

       [TestMethod]
       public void NullValueWithSqlDbTypeTest()
       {
           SqlDatabase db = (SqlDatabase)DatabaseFactory.CreateDatabase("DataSQLTest");
           int? age = null;
           DbCommand dbIns = db.GetStoredProcCommand("TestNullType");
           db.AddInParameter(dbIns, "@eno", SqlDbType.Int, 1);
           db.AddInParameter(dbIns, "@ename", SqlDbType.VarChar, "Narayan");
           db.AddInParameter(dbIns, "@age", SqlDbType.Int, age);
           db.ExecuteNonQuery(dbIns);
           int noofRows = (int)db.ExecuteScalar(CommandType.Text, "select count(eno) from TestNullTable");
           Assert.AreEqual(noofRows, 1);
       }

       [TestMethod]
       public void InsertNullValueSqlTypeTest()
       {
           SqlDatabase db = (SqlDatabase)DatabaseFactory.CreateDatabase("DataSQLTest");

           DbCommand dbIns = db.GetStoredProcCommand("TestNullType");
           db.AddInParameter(dbIns, "@eno", SqlDbType.Int, 1);
           db.AddInParameter(dbIns, "@ename", SqlDbType.VarChar, "Narayan");
           db.AddInParameter(dbIns, "@age", SqlDbType.Int);
           db.ExecuteNonQuery(dbIns);
           int noofRows = (int)db.ExecuteScalar(CommandType.Text, "select count(eno) from TestNullTable");
           Assert.AreEqual(noofRows, 1);
       }
       #endregion

       #region Dictionary Source test
        [TestMethod]
        public void DictionarySourceTest()
        {
            DictionaryConfigurationSource source = new DictionaryConfigurationSource();

            DatabaseSettings setting = new DatabaseSettings();
            setting.DefaultDatabase = "SqlTest";
            ConnectionStringsSection connectionSection = new ConnectionStringsSection();
            connectionSection.ConnectionStrings.Add(new ConnectionStringSettings("SqlTest", "server=(local)\\sqlexpress;database=TestDatabase;Integrated Security=true", "System.Data.SqlClient"));
            source.Add("connectionStrings", connectionSection);
            source.Add("dataConfiguration", setting);

            DatabaseProviderFactory factory = new DatabaseProviderFactory(source);
            SqlDatabase db = (SqlDatabase)factory.Create("SqlTest");
            StringBuilder readerDataActual = new StringBuilder();
            string sqlText = "select ItemDescription From Items where ItemId = 1 ";
            using (IDataReader dataReader = db.ExecuteReader(CommandType.Text, sqlText))
            {
                while (dataReader.Read())
                {
                    readerDataActual.Append(dataReader["ItemDescription"].ToString().Trim());
                }
            }
            Assert.AreEqual("Digital Image Pro", readerDataActual.ToString());
        }

       #endregion
   }


}
