using System;
//using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Data.Sql;
using System.Data.Odbc;
using System.Xml;
using System.Data.Common;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Configuration;

namespace DataFuncTests.GenericDatabaseODBCFixturesFixtures
{
    /// <summary>
    /// Tests ExecuteReader of the DataBase Class
    /// </summary>
    [TestClass]
    public class ExecuteReaderTestFixture
    {
        String connStr;
        String itemsXMLfile;
        public ExecuteReaderTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString; 
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("begin Delete from Items where itemId > 3 " +
                                                   "Update Items set ItemDescription='Digital Image Pro',QtyInHand = 25, QtyRequired=100 where ItemId = 1 " +
                                                   "Update Items set ItemDescription='Excel 2003',QtyInHand = 95, QtyRequired=100 where ItemId = 2 " +
                                                   "Update Items set ItemDescription='Infopath',QtyInHand = 34, QtyRequired=100 where ItemId = 3 " +
                                                   "delete from CustomersOrders where CustomerId > 3 " +
                                                   "Update CustomersOrders set QtyOrdered=100 " +
                                            "end", conn);
            cmd.ExecuteNonQuery();
            conn.Close();
            itemsXMLfile = "Items.xml";
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "DB Command"
        [TestMethod]
        public void DBCommandQueryTest()
        {
            //Expected Data
            DataSet dsExpectedData = new DataSet();
            dsExpectedData.ReadXml(itemsXMLfile);
            int count = 0;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            string sqlCommand = "Select ItemDescription from Items order by ItemId";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
                while (dataReader.Read())
                {
                    Assert.AreEqual(dsExpectedData.Tables[0].Rows[count][0].ToString(), dataReader["ItemDescription"].ToString().Trim());
                    count++;
                }
            }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        public void DBCommandSPNoParmTest()
        {
            //Expected Data
            DataSet dsExpectedData = new DataSet();
            dsExpectedData.ReadXml(itemsXMLfile);
            int count = 0;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            string sqlCommand = "ItemsGet";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
                while (dataReader.Read())
                {
                    // Get the value of the 'Name' column in the DataReader
                    Assert.AreEqual(dataReader["ItemDescription"].ToString().Trim(), dsExpectedData.Tables[0].Rows[count][0]);
                    count++;
                }
            }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        public void DBCommandSPINParmTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            string sqlCommand = "{ Call ItemGetByPrice(?) }";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            this.CreateCommandParameter(db,dbCommandWrapper, "@Price", DbType.Int16, ParameterDirection.Input, 90);
            StringBuilder readerDataActual = new StringBuilder();
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
                while (dataReader.Read())
                {
                    readerDataActual.Append(dataReader["ItemDescription"].ToString().Trim());
                }
            }
            Assert.AreEqual("Excel 2003", readerDataActual.ToString());
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        public void DBCommandSPOutParmTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            string sqlCommand = "{ Call ItemPriceGetbyId(?,?) }";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            this.CreateCommandParameter(db,dbCommandWrapper, "@Id", System.Data.DbType.Int16, ParameterDirection.Input, 1);
            this.CreateCommandParameter(db,dbCommandWrapper, "@Price", System.Data.DbType.Double, ParameterDirection.Output, 32);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
            }
            Assert.AreEqual(Convert.ToDouble(38.95), Convert.ToDouble(dbCommandWrapper.Parameters["@Price"].Value));
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        public void DBCommandSPUpdateParmTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            string sqlCommand = "{ Call ItemQuantityUpdateById(?,?) }";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
            this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 55);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
                while (dataReader.Read())
                {
                    if (Convert.ToInt16(dataReader["ItemId"]) == 1)
                    {
                        Assert.AreEqual(55, Convert.ToInt32(dataReader["QtyInHand"]));
                    }

                }

            }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        [ExpectedException(typeof(OdbcException))]
        public void DBCommandNoSPPresentTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            string sqlCommand = "{ Call UpdateItem1(?,?) }";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            { }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }
        #endregion

        #region "Db Command, Transaction"
        [TestMethod]
        public void DBCommandTransSqlNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();

                string sqlCommand = "begin update Items set QtyInHand= 88 where ItemId = 2  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);

                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {

                }
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("select * from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
        }

        [TestMethod]
        public void DBCommandTransSqlCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand = 45 where ItemId = 1  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {

                }
                transaction.Commit();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(45), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBCommandTransSqlRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();

                string sqlCommand = "begin update Items set QtyInHand = 67 where ItemId = 2  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {
                }
                transaction.Rollback();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBCommandTransSqlCommitRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand = 111 where ItemId = 1  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {

                }
                transaction.Commit();
                transaction = connection.BeginTransaction();
                sqlCommand = "begin update Items set QtyInHand = 55 where ItemId = 2 select * from Items end";
                dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {

                }
                transaction.Rollback();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(111), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBCommandTransSqlRollbackOnErrorTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            DbTransaction transaction = null;
            try
            {
                using (DbConnection connection = db.CreateConnection())
                {
                    connection.Open();
                    transaction = connection.BeginTransaction();
                    string sqlCommand = "begin update Items set QtyInHand = 45 where ItemId = 2  select * from Items where ItemId = 'Test' end";
                    DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                    using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                    {

                    }
                    transaction.Commit();
                }
            }
            catch
            {
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBCommandTransSPNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 67);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {

                }
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBCommandTransSPCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "{ Call ItemQuantityUpdateById(?,?) }";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db, dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 88);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {

                }
                transaction.Commit();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(88), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBCommandTransSPRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();

                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 65);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {
                }
                transaction.Rollback();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBCommandTransSPCommitRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();

                string sqlCommand = "{ Call ItemQuantityUpdateById(?,?) }";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 44);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {
                }
                transaction.Commit();
                transaction = connection.BeginTransaction();
                sqlCommand = "{ Call ItemQuantityUpdateById(?,?) }";
                dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 72);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper, transaction))
                {
                }
                transaction.Rollback();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(44), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }
        #endregion

        #region "ExecuteReader(Commandtype)"
        [TestMethod]
        public void ExecuteReaderCommandtypeSqlTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            StringBuilder readerDataActual = new StringBuilder();
            string sqlText = "select ItemDescription From Items where ItemId = 1 ";
            using (IDataReader dataReader = db.ExecuteReader(CommandType.Text, sqlText))
            {
                while (dataReader.Read())
                {
                    readerDataActual.Append(dataReader["ItemDescription"].ToString().Trim());
                }
            }
            Assert.AreEqual("Digital Image Pro", readerDataActual.ToString());
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
        }


        [TestMethod]
        public void ExecuteReaderCommandtypeSpTest()
        {
            //Expected Data
            DataSet dsExpectedData = new DataSet();
            dsExpectedData.ReadXml(itemsXMLfile);
            int count = 0;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerDataActual = new StringBuilder();
            string spName = "ItemsGet";
            using (IDataReader dataReader = db.ExecuteReader(CommandType.StoredProcedure, spName))
            {
                while (dataReader.Read())
                {
                    Assert.AreEqual(dsExpectedData.Tables[0].Rows[count][0].ToString(), dataReader["ItemDescription"].ToString().Trim());
                    count++;
                }
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
        }

        [TestMethod]
        [ExpectedException(typeof(OdbcException))]

        public void ExecuteReaderNotExistingSPTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            StringBuilder readerDataActual = new StringBuilder();
            string spName = "something";
            using (IDataReader dataReader = db.ExecuteReader(CommandType.StoredProcedure, spName))
            {
                while (dataReader.Read())
                {
                    readerDataActual.Append(dataReader["Name"]);
                    readerDataActual.Append(Environment.NewLine);
                }
            }
        }
        #endregion

        #region "Transaction, Command Type, Command Name"
        [TestMethod]
        public void ExecuteReaderTypeTextStringTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (IDataReader dataReader = db.ExecuteReader(transaction, CommandType.Text, "begin update Items set QtyInHand = 75 where ItemId = 1 select * from Items end "))
                {
                }
                transaction.Commit();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(75), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void ExecuteReaderTypeTextStringNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (IDataReader dataReader = db.ExecuteReader(transaction, CommandType.Text, "begin update Items set QtyInHand = 67 where ItemId = 2 select * from Items end "))
                {

                }
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]

        public void ExecuteReaderTypeTextStringRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (IDataReader dataReader = db.ExecuteReader(transaction, CommandType.Text, "begin update Items set QtyInHand = 18 where ItemId = 2 select * from Items end "))
                {
                }
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void ExecuteReaderTranTextSpCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (IDataReader dataReader = db.ExecuteReader(transaction, CommandType.StoredProcedure, "ItemQuantityUpdateById"))
                {
                }
                transaction.Commit();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where itemid=1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(50, Convert.ToInt32(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void ExecuteReaderTranTextSpNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (IDataReader dataReader = db.ExecuteReader(transaction, CommandType.StoredProcedure, "ItemQuantityIncrease"))
                {
                }
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyRequired from Items", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(100, Convert.ToInt32(dr["QtyRequired"].ToString()));
            }
            dr.Close();
            conn.Close();
        }
        [TestMethod]
        public void ExecuteReaderTranTypeSpRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTestODBC");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (IDataReader dataReader = db.ExecuteReader(transaction, CommandType.StoredProcedure, "ItemQuantityIncrease"))
                {
                }
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyRequired from Items", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(100, Convert.ToInt32(dr["QtyRequired"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        #endregion

        # region Helper Method
        private void CreateCommandParameter(Database db, DbCommand comm, string Parametername, DbType dbType, ParameterDirection dir, object value)
        {

            if (dir == ParameterDirection.Input)
            {
                db.AddInParameter(comm, Parametername, dbType, value);
            }
            else if (dir == ParameterDirection.Output)
            {
                db.AddOutParameter(comm, Parametername, dbType, (int)value);
            }

            //DbParameter cmd = comm.CreateParameter();
            //cmd.ParameterName = Parametername;
            //cmd.Value = value;
            //cmd.Direction = dir;
            //cmd.DbType = dbType;
            //return (cmd);
        }

        # endregion



    }
}
