using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;
using System.Data.Common;
using Microsoft.Practices.EnterpriseLibrary.Data.Oracle;     
namespace DataFuncTests.OracleDatabaseFixtures
{
    /// <summary>
    /// Summary description for ExecuteDataSetTestFixture
    /// </summary>
    [TestClass]
    public class ExecuteDataSetTestFixture
    {
        private static DataSet dsCustomers = null;
        private static DataSet dsProducts = null;
        public ExecuteDataSetTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        [TestInitialize()]
        public void Initialize()
        {
            dsCustomers = new DataSet();
            dsProducts = new DataSet();
            string curPath = AppDomain.CurrentDomain.BaseDirectory;
            string customerFile = System.IO.Path.Combine(curPath, "Customers.xml");
            string productFile = System.IO.Path.Combine(curPath, "Products.xml");
            dsCustomers.ReadXml(customerFile);
            dsProducts.ReadXml(productFile);
        }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod] 
        public void SqlQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            string sqlText = "select * From Customers where customerId in ('BLAUS','BLONP','BOLID') order by CustomerId";

            using (DataSet ds = db.ExecuteDataSet(CommandType.Text, sqlText))
            {
                int rows = dsCustomers.Tables[0].Rows.Count;
                int columns = dsCustomers.Tables[0].Columns.Count;
                
                Assert.IsTrue(rows==dsCustomers.Tables[0].Rows.Count);
                Assert.IsTrue(columns ==dsCustomers.Tables[0].Columns.Count);
                for(int i=0;i<rows;i++)
                {
                    for(int j=0;j<columns;j++) 
                    {
                        Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables[0].Rows[i][j].ToString().Trim());
                    }
                }

            }
        }

        [TestMethod]
        public void StoredProcQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            string spName = "GetCustomersView";
            using (DataSet ds = db.ExecuteDataSet(CommandType.StoredProcedure, spName))
            {
                int rows = ds.Tables[0].Rows.Count;
                int columns = ds.Tables[0].Columns.Count;

                Assert.IsTrue(rows == dsCustomers.Tables[0].Rows.Count);
                Assert.IsTrue(columns == dsCustomers.Tables[0].Columns.Count);
                for (int i = 0; i < rows; i++)
                {
                    for (int j = 0; j < columns; j++)
                    {
                        Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables[0].Rows[i][j].ToString().Trim());
                    }
                }
            }
        }

        [TestMethod]
        public void CommandWrapperSqlQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            string sqlCommand = "select * from Customers where customerId ='BLAUS'";
            DbCommand dbCommand = db.GetSqlStringCommand(sqlCommand);

            using (DataSet ds = db.ExecuteDataSet(dbCommand))
            {
                int columns = dsCustomers.Tables[0].Columns.Count;
                for (int j = 0; j < columns; j++)
                {
                   Assert.AreEqual(dsCustomers.Tables[0].Rows[0][j].ToString().Trim(), ds.Tables[0].Rows[0][j].ToString().Trim());
                }
            }
        }

        [TestMethod] 
        public void CommandWrapperStoredProcTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DbCommand dbCommand = db.GetStoredProcCommand("GetCustomerByID");
            db.AddInParameter(dbCommand,"vCustomerID", DbType.String, "BLAUS");

            using (DataSet ds = db.ExecuteDataSet(dbCommand))
            {
                int columns = dsCustomers.Tables[0].Columns.Count;
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[0][j].ToString().Trim(), ds.Tables[0].Rows[0][j].ToString().Trim());
                }
            }
        }

        [TestMethod]
        public void CommandWrapperStoredProcOutParamTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DbCommand dbCommand = db.GetStoredProcCommand("GetCustomerOut") ;
			db.AddInParameter(dbCommand, "vCustomerID",DbType.String ,"BLONP");
			db.AddOutParameter(dbCommand, "vName",DbType.String,50);
            using (DataSet ds = db.ExecuteDataSet(dbCommand))
            {
                int columns = dsCustomers.Tables[0].Columns.Count;
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[1][j].ToString().Trim(), ds.Tables[0].Rows[0][j].ToString().Trim());
                }
                Assert.AreEqual(dsCustomers.Tables[0].Rows[1]["CompanyName"].ToString() , (string)db.GetParameterValue(dbCommand, "vName"));
            }
        }

        [TestMethod]
        public void OverLoadWithParamTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            using (DataSet ds = db.ExecuteDataSet("GetCustomerByID", new object[] { "BOLID", 1 }))
            {
                int columns = dsCustomers.Tables[0].Columns.Count;
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[2][j].ToString().Trim(), ds.Tables[0].Rows[0][j].ToString().Trim());
                }
            }
        }


        [TestMethod]
        public void TransactionRollbackStoredProcWithParamTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet dsAddCountry = null;
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.ExecuteDataSet(transaction, "UpdateCountryListAll", new object[] { "US", "United States of America","" });
                    dsAddCountry = db.ExecuteDataSet(transaction, "AddCountryListAll", new object[] { "IN","India" ,""});
                    transaction.Commit();

                }
                catch (Exception)
                {
                    transaction.Rollback();
                    Assert.IsFalse("United States of America" == (string)db.ExecuteScalar(CommandType.Text , "select CountryName from Country where CountryCode='US'") );
                    Assert.IsNull(dsAddCountry);
                }
            }

        }

        

        [TestMethod]
        public void TransactionCommitStoredProcWithParamTest()
        {
            OracleDatabase db = (OracleDatabase)DatabaseFactory.CreateDatabase("OracleTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    int initialCount = Convert.ToInt32(db.ExecuteScalar(CommandType.Text, "select count(countrycode) from Country"));
                    db.ExecuteDataSet(transaction, "AddCountryListAll", new object[] { "TEMP", "Temporary", "" });
                    db.ExecuteDataSet(transaction, "DeleteCountryListAll", new object[] { "TEMP", "" });
                    transaction.Commit();
                    int finalCount = Convert.ToInt32(db.ExecuteScalar(CommandType.Text, "select count(countrycode) from Country"));
                    Assert.IsTrue(finalCount == initialCount);
                }
                catch (Exception e)
                {
                    transaction.Rollback();
                    Assert.Fail("Transaction Rolled Back : " + e.Message);
                }
            }

        }

        public void SqlQueryTestUsingCommand()
        {
            OracleDatabase db = (OracleDatabase) DatabaseFactory.CreateDatabase("OracleTest");
            string sqlText = "select * From Customers where customerId in ('BLAUS','BLONP','BOLID') order by CustomerId";
            DbCommand dbCmdSql = db.GetSqlStringCommand(sqlText);
            using (DataSet ds = db.ExecuteDataSet(dbCmdSql))
            {
                int rows = dsCustomers.Tables[0].Rows.Count;
                int columns = dsCustomers.Tables[0].Columns.Count;

                Assert.IsTrue(rows == dsCustomers.Tables[0].Rows.Count);
                Assert.IsTrue(columns == dsCustomers.Tables[0].Columns.Count);
                for (int i = 0; i < rows; i++)
                {
                    for (int j = 0; j < columns; j++)
                    {
                        Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables[0].Rows[i][j].ToString().Trim());
                    }
                }

            }
        }

       

    }
}
