using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;
using System.Data.Common; 

namespace DataFuncTests.OracleDatabaseFixtures
{
    /// <summary>
    /// Summary description for ExecuteScalarTestFixture
    /// </summary>
    [TestClass]
    public class ExecuteScalarTestFixture
    {
        public ExecuteScalarTestFixture()
        {
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void SqlQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            string countryName = (string)db.ExecuteScalar(CommandType.Text, "select CountryName from Country where CountryCode='IN'");
            Assert.AreEqual("India", countryName);   
        }

        [TestMethod]
        public void CommandWrapperSqlQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DbCommand dbCommand = db.GetSqlStringCommand("select CountryName from Country where CountryCode='US'"); 
            string countryName = (string)db.ExecuteScalar(dbCommand);
            Assert.AreEqual("UnitedStates", countryName);   
        }

        [TestMethod]
        public void CommandWrapperStoredProcTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DbCommand dbCommand = db.GetStoredProcCommand("GetProductName");
            db.AddInParameter(dbCommand, "vProductId", DbType.Int32, 1);
            db.AddOutParameter(dbCommand, "vResult", DbType.String , 100);
            db.ExecuteScalar(dbCommand);
            Assert.AreEqual("Product1", db.GetParameterValue(dbCommand, "vResult"));
        }

        [TestMethod]
        public void TransactionCommitSqlQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");

            int debitId = Convert.ToInt32(db.ExecuteScalar(CommandType.Text, "select max(DebitId) from Debits"));

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    debitId = debitId + 1;
                    string query = "INSERT INTO DEBITS ( DebitId, AccountId, Amount )  VALUES ( " + debitId.ToString() + " , 5, 5000 )";
                    db.ExecuteScalar(transaction, CommandType.Text, query);
                    query = "DELETE FROM DEBITS where DebitId =  " + debitId.ToString();
                    db.ExecuteScalar(transaction, CommandType.Text, query);
                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    Assert.Fail("The transaction is rolled back.");
                }
                finally
                {
                    connection.Close();
                }
                string result = "select * from Debits where DebitID = " +  debitId ;
                using (DataSet ds = db.ExecuteDataSet(CommandType.Text, result))
                {
                    Assert.IsTrue(0 == ds.Tables[0].Rows.Count);
                }
            }
        }

        [TestMethod]
        public void TransactionRollbackSqlQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");

            int debitId = Convert.ToInt32(db.ExecuteScalar(CommandType.Text, "select max(DebitId),count(DebitID) from Debits"));

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    string query = "DELETE FROM DEBITS WHERE DEBITID = " + debitId.ToString();
                    db.ExecuteScalar(transaction, CommandType.Text, query);
                    debitId = debitId + 1;
                    query = "INSERT INTO DEBITS ( DebitId, AccountId, Amount )  VALUES ( " + debitId.ToString() + " , 5, 5000 )";
                    db.ExecuteScalar(transaction, CommandType.Text, query);
                    db.ExecuteScalar(transaction, CommandType.Text, query);
                    transaction.Commit();
                }
                catch (Exception )
                {
                    transaction.Rollback();
                }
                finally
                {
                    connection.Close();
                }
                string result = "select * from Debits where DebitID in ( " + (debitId -1) + "," + debitId + "," + (debitId + 1) + ")";

                using (DataSet ds = db.ExecuteDataSet(CommandType.Text, result))
                {
                    Assert.IsTrue(1 == ds.Tables[0].Rows.Count);
                    Assert.IsTrue((debitId - 1) == Convert.ToInt32(ds.Tables[0].Rows[0]["DebitId"])); 
                }
            }
        }


        [TestMethod]
        public void TransactionRollbackStoredProcWithParamTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.ExecuteScalar(transaction, "AddCountry", new object[] { "SA", "South Africa" });
                    db.ExecuteScalar(transaction, "DeleteCountry", new object[] { "SA" });
                    transaction.Commit();
                }
                catch (Exception )
                {
                    transaction.Rollback();
                    Assert.Fail("Transaction Rolled back");
                }
                finally
                {
                    connection.Close();
                }

                using (DataSet result = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode = 'SA'"))
                {
                    Assert.IsTrue(result.Tables[0].Rows.Count == 0);
                }

            }
        }

        [TestMethod]
        public void TransactionRollbackStoredProcParamTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.ExecuteScalar(transaction, "DeleteCountry", new object[] { "IN"});
                    db.ExecuteScalar(transaction, "UpdateCountry", new object[] { "US", "United States of America" });
                    db.ExecuteScalar(transaction, "AddCountry", new object[] { "SA", "South Africa" });
                    db.ExecuteScalar(transaction, "AddCountry", new object[] { "SA", "South Africa" });
                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                }
                finally
                {
                    connection.Close();
                }

                using (DataSet result = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('SA','IN','US') order by CountryCode "))
                {
                    Assert.IsTrue(result.Tables[0].Rows.Count == 2);
                    Assert.AreEqual("India", result.Tables[0].Rows[0]["CountryName"]);
                    Assert.IsFalse("United States of America" == (string)result.Tables[0].Rows[1]["CountryName"]);      
                }

            }
        }

        [TestMethod]
        public void TransactionCommitStoredProcTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();

                int lastEmployeeId = Convert.ToInt32(db.ExecuteScalar(CommandType.Text, "select max(EmployeeID) from Employees"));   
                DbCommand addEmployee = db.GetStoredProcCommand("AddEmployees");
                DbCommand deleteEmployee = db.GetStoredProcCommand("DeleteEmployees");

                db.AddInParameter(addEmployee, "vEmployeeId", DbType.Int32, lastEmployeeId + 1);
                db.AddInParameter(addEmployee, "vFirstName", DbType.String, "First Name");
                db.AddInParameter(addEmployee, "vLastName", DbType.String, "Last Name");
                db.AddInParameter(addEmployee, "vReportsTo", DbType.Int32, lastEmployeeId);

                db.AddInParameter(deleteEmployee, "vEmployeeId", DbType.Int32, lastEmployeeId + 1);

                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.ExecuteNonQuery(addEmployee, transaction);
                    db.ExecuteNonQuery(deleteEmployee, transaction);

                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                    Assert.Fail("Transaction Rolled back");
                }
                finally
                {
                    connection.Close();
                }

                using (DataSet result = db.ExecuteDataSet(CommandType.Text, "select * from Employees where EmployeeId = " + (lastEmployeeId + 1)))
                {
                    Assert.IsTrue(result.Tables[0].Rows.Count == 0);
                }

            }
        }

        [TestMethod]
        public void TransactionRollbackStoredProcTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();

                DbCommand deleteEmployee = db.GetStoredProcCommand("DeleteEmployees");
                DbCommand addEmployee = db.GetStoredProcCommand("AddEmployees");

                db.AddInParameter(deleteEmployee, "vEmployeeId", DbType.Int32, 107);

                db.AddInParameter(addEmployee, "vEmployeeId", DbType.Int32, 1);
                db.AddInParameter(addEmployee, "vFirstName", DbType.String, "Unknown");
                db.AddInParameter(addEmployee, "vLastName", DbType.String, "Unknown");
                db.AddInParameter(addEmployee, "vReportsTo", DbType.Int32, null);

                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.ExecuteNonQuery(deleteEmployee, transaction);
                    db.ExecuteNonQuery(addEmployee, transaction);
                    transaction.Commit();
                }
                catch (Exception)
                {
                    transaction.Rollback();
                }
                finally
                {
                    connection.Close();
                }

                using (DataSet result = db.ExecuteDataSet(CommandType.Text, "select * from Employees where EmployeeId in (1,107) order by employeeid"))
                {
                    Assert.IsTrue(result.Tables[0].Rows.Count == 2);
                    Assert.IsFalse("Unknown" == (string)result.Tables[0].Rows[0]["FirstName"]);
                    Assert.IsFalse("Unknown" == (string)result.Tables[0].Rows[0]["LastName"]);
                }

            }
        }

    }
}
