using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Oracle;
using System.Data;
using System.Data.Common;
using System.Data.OracleClient;  
namespace DataFuncTests.OracleDatabaseFixtures
{
    /// <summary>
    /// Summary description for LoadDataSetTestFixture
    /// </summary>
    [TestClass]
    public class LoadDataSetTestFixture
    {
        private static DataSet dsCustomers = null;
        private static DataSet dsProducts = null;

        public LoadDataSetTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
         [TestCleanup()]
         public void Cleanup() 
         {
             Database db = DatabaseFactory.CreateDatabase("OracleTest");
             db.ExecuteNonQuery(CommandType.Text, "delete from Country where CountryCode in ('NZ','ZIM')");   
         }
        //
        // Use TestInitialize to run code before running each test 
         [TestInitialize()]
         public void MyTestInitialize() 
         {
             dsCustomers = new DataSet();
             dsProducts = new DataSet();
             string curPath = AppDomain.CurrentDomain.BaseDirectory;
             string customerFile = System.IO.Path.Combine(curPath, "Customers.xml");
             string productFile = System.IO.Path.Combine(curPath, "Products.xml");    
             dsCustomers.ReadXml(customerFile);
             dsProducts.ReadXml(productFile);
         }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void SqlQueryLoadSingleTableTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();
            db.LoadDataSet(CommandType.Text, "select * from Customers where CustomerId in ('BLAUS','BLONP','BOLID') order by CustomerId", ds, new string[] { "Customers" });
            int rows = ds.Tables[0].Rows.Count;
            int columns = ds.Tables[0].Columns.Count;
            Assert.IsTrue(rows == dsCustomers.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsCustomers.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables[0].Rows[i][j].ToString().Trim());
                }
            }

        }

        [TestMethod]
        public void SqlQueryLoadMultipleTableTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();
            db.LoadDataSet(CommandType.Text, "select * from Customers where CustomerId in ('BLAUS','BLONP','BOLID') order by CustomerId", ds, new string[] { "Customers" });
            db.LoadDataSet(CommandType.Text, " select * from Products where ProductId in (1,2,3,4,5) order by productid", ds, new string[] {  "Products" });
            int rows = ds.Tables["Customers"].Rows.Count;
            int columns = ds.Tables["Customers"].Columns.Count;
            Assert.IsTrue(rows == dsCustomers.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsCustomers.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Customers"].Rows[i][j].ToString().Trim());
                }
            }
            rows = ds.Tables["Products"].Rows.Count;
            columns = ds.Tables["Products"].Columns.Count;
            Assert.IsTrue(rows == dsProducts.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsProducts.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsProducts.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Products"].Rows[i][j].ToString().Trim());
                }
            }
        }

        [TestMethod] 
        public void StoredProcLoadMultipleTableTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();
            db.LoadDataSet(CommandType.StoredProcedure, "GetCustomersView", ds, new string[] { "Customers" });
            db.LoadDataSet(CommandType.StoredProcedure, "GetProductsView", ds, new string[] { "Products" });
            int rows = ds.Tables["Customers"].Rows.Count;
            int columns = ds.Tables["Customers"].Columns.Count;
            Assert.IsTrue(rows == dsCustomers.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsCustomers.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Customers"].Rows[i][j].ToString().Trim());
                }
            }
            rows = ds.Tables["Products"].Rows.Count;
            columns = ds.Tables["Products"].Columns.Count;
            Assert.IsTrue(rows == dsProducts.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsProducts.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsProducts.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Products"].Rows[i][j].ToString().Trim());
                }
            }
        }

        [TestMethod]
        public void StoredProcParamLoadMultipleTableTest()
        {
            OracleDatabase db = (OracleDatabase)DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();
            db.LoadDataSet("GetCustomersAndProductsView", ds, new string[] { "Customers", "Products" }, new object[] {1,1 });
            int rows = ds.Tables["Customers"].Rows.Count;
            int columns = ds.Tables["Customers"].Columns.Count;
            Assert.IsTrue(rows == dsCustomers.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsCustomers.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Customers"].Rows[i][j].ToString().Trim());
                }
            }
            rows = ds.Tables["Products"].Rows.Count;
            columns = ds.Tables["Products"].Columns.Count;
            Assert.IsTrue(rows == dsProducts.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsProducts.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsProducts.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Products"].Rows[i][j].ToString().Trim());
                }
            }
        }

        [TestMethod]
        public void CommandWrapperLoadMultipleTableTest()
        {
            OracleDatabase db = (OracleDatabase)DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();
            OracleCommand dbCommand = (OracleCommand) db.GetStoredProcCommand("GetCustomersAndProductsView");
            db.AddParameter(dbCommand, "cur_OUT", OracleType.Cursor, 100, ParameterDirection.Output, true, 1, 1, null, DataRowVersion.Default, null);
            db.AddParameter(dbCommand, "cur_Products", OracleType.Cursor, 100, ParameterDirection.Output, true, 1, 1, null, DataRowVersion.Default, null);
            db.LoadDataSet(dbCommand, ds, new string[] {"Customers","Products" });
            int rows = ds.Tables["Customers"].Rows.Count;
            int columns = ds.Tables["Customers"].Columns.Count;
            Assert.IsTrue(rows == dsCustomers.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsCustomers.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsCustomers.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Customers"].Rows[i][j].ToString().Trim());
                }
            }
            rows = ds.Tables["Products"].Rows.Count;
            columns = ds.Tables["Products"].Columns.Count;
            Assert.IsTrue(rows == dsProducts.Tables[0].Rows.Count);
            Assert.IsTrue(columns == dsProducts.Tables[0].Columns.Count);
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < columns; j++)
                {
                    Assert.AreEqual(dsProducts.Tables[0].Rows[i][j].ToString().Trim(), ds.Tables["Products"].Rows[i][j].ToString().Trim());
                }
            }
        }

        [TestMethod]
        public void TransactionCommitStoredProcTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet dsCountry = new DataSet(); 
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.LoadDataSet(transaction,"AddCountryListAll", dsCountry, new string[] { "Country" }, new object[] { "NZ", "NewZealand","" });
                    transaction.Commit();  
                }
                catch(Exception e)
                {
                    transaction.Rollback();
                    Assert.Fail("The transaction is rolled back : " + e.Message);  
                }
                dsCountry.Tables[0].PrimaryKey = new DataColumn[] { dsCountry.Tables[0].Columns["CountryCode"] };   
                DataRow result = dsCountry.Tables[0].Rows.Find("NZ");
                Assert.IsNotNull(result);
                Assert.AreEqual("NewZealand", result["CountryName"].ToString());    
            }
        }

        [TestMethod]
        public void TransactionRollbackStoredProcTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet dsCountry = new DataSet();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.LoadDataSet(transaction, "DeleteCountryListAll", dsCountry, new string[] { "Country" }, new object[] { "US","" });
                    db.LoadDataSet(transaction, "AddCountryListAll", dsCountry, new string[] { "Country" }, new object[] { "IN", "India","" });
                    transaction.Commit();
                }
                catch
                {
                    transaction.Rollback();
                }
                string countryName = (string)db.ExecuteScalar(CommandType.Text, "select countryName from Country where CountryCode='US'");
                Assert.AreEqual(countryName, "UnitedStates");    
            }
        }

        [TestMethod]
        public void TransactionCommitCommandWrapperTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet dsCountry = new DataSet();
            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountryListAll");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String,"ZIM");
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String,"Zimbabwe");
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountryListAll");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String,"ZIM");

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.LoadDataSet(dbAddCountry, dsCountry, "Country",transaction);
                    dsCountry.Clear();  
                    db.LoadDataSet(dbDeleteCountry, dsCountry, "Country",transaction);   
                    transaction.Commit();
                }
                catch (Exception e)
                {
                    transaction.Rollback();
                    Assert.Fail("The transaction is rolled back : " + e.Message);
                }
                dsCountry.Tables[0].PrimaryKey = new DataColumn[] { dsCountry.Tables[0].Columns["CountryCode"] };
                DataRow result = dsCountry.Tables[0].Rows.Find("ZIM");
                Assert.IsNull(result);
            }
        }

        [TestMethod]
        public void TransactionRollbackCommandWrapperTest()
        {
            OracleDatabase db = (OracleDatabase)DatabaseFactory.CreateDatabase("OracleTest");
            DataSet dsCountry = new DataSet();
            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountryListAll");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String, "SCO");
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String, "Scotland");
            DbCommand dbAddCountry1 = db.GetStoredProcCommand("AddCountryListAll");
            db.AddInParameter(dbAddCountry1, "vCountryCode", DbType.String, "SCO");
            db.AddInParameter(dbAddCountry1, "vCountryName", DbType.String, "Scotland");

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.LoadDataSet(dbAddCountry, dsCountry, "Country",transaction);
                    db.LoadDataSet(dbAddCountry1, dsCountry, "Country",transaction);
                    transaction.Commit();
                }
                catch (Exception )
                {
                    transaction.Rollback();
                }
            }
            string countryName = (string)db.ExecuteScalar(CommandType.Text, "select countryName from Country where CountryCode='SCO'");
            Assert.IsNull(countryName);    
        }

    }
}
