using System;
//using Microsoft.VisualStudio.TestTools.UnitTesting;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Data;
using System.Data.SqlClient;
using System.Xml;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Data.Common;
using System.Configuration;
using System.Data.OracleClient;  
namespace DataFuncTests.SqlDatabaseFixtures
{
    /// <summary>
    /// Tests the ExecuteXmlReader Method of the Database 
    /// </summary>
    [TestClass]
    public class ExecuteXMLReaderTestFixture
    {
        string ExecuteXMLfile;
        string connStr;
        public ExecuteXMLReaderTestFixture()
        {
            ExecuteXMLfile = "executexml.xml";
        }
        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            ExecuteXMLfile = "executexml.xml";
           connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString;
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("begin Delete from Items where itemId > 3 " +
                                                  "Update Items set ItemDescription='Digital Image Pro',QtyInHand = 25, QtyRequired=100 where ItemId = 1 " +
                                                   "Update Items set ItemDescription='Excel 2003',QtyInHand = 95, QtyRequired=100 where ItemId = 2 " +
                                                   "Update Items set ItemDescription='Infopath',QtyInHand = 34, QtyRequired=100 where ItemId = 3 " +
                                                   "delete from CustomersOrders where CustomerId > 3 " +
                                                   "Update CustomersOrders set QtyOrdered=100 " +
                                            "end", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            dr.Close();
            conn.Close();
        }
        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        #region "ExecuteXmlReader - Command - Sql/SP

        [TestMethod]
        public void ExecuteXmlReaderSqlCommandTest()
        {
            // Use a named database instance that refers to a SQL Server database.
            SqlDatabase dbSQL = DatabaseFactory.CreateDatabase("DataSQLTest") as SqlDatabase;
            // Use "FOR XML AUTO" to have SQL return XML data
            string sqlCommand = "Select ItemDescription from Items FOR XML AUTO";
            System.Data.Common.DbCommand dbCommandWrapper = dbSQL.GetSqlStringCommand(sqlCommand) as DbCommand;
            XmlReader productsReader = null;
            productsReader = dbSQL.ExecuteXmlReader(dbCommandWrapper);
            DataSet da = new DataSet();
            da.ReadXml(ExecuteXMLfile);
            int count = da.Tables[0].Rows.Count;
            int index = 0;
            // Iterate through the XmlReader and put the data into our results.
            while (!productsReader.EOF)
            {
                if (productsReader.IsStartElement())
                {
                    Assert.AreEqual(productsReader.ReadOuterXml().ToString().Trim(), da.Tables[0].Rows[index][0].ToString().Trim());
                    if (index == (count - 1))
                        break;
                    index++;
                }
            }
        }

        [TestMethod]
        public void ExecuteXmlReaderSpCommandTest()
        {
            // Use a named database instance that refers to a SQL Server database.
            SqlDatabase dbSQL = DatabaseFactory.CreateDatabase("DataSQLTest") as SqlDatabase;
            // Use "FOR XML AUTO" to have SQL return XML data
            string spName = "itemsgetxml";
            DbCommand dbCommandWrapper = dbSQL.GetStoredProcCommand(spName) as DbCommand;
            XmlReader productsReader = null;
            productsReader = dbSQL.ExecuteXmlReader(dbCommandWrapper);
            DataSet da = new DataSet();
            da.ReadXml(ExecuteXMLfile);
            int count = da.Tables[0].Rows.Count;
            int index = 0;
            // Iterate through the XmlReader and put the data into our results.
            while (!productsReader.EOF)
            {
                if (productsReader.IsStartElement())
                {
                    Assert.AreEqual(productsReader.ReadOuterXml().ToString().Trim(), da.Tables[0].Rows[index][0].ToString().Trim());
                    if (index == (count - 1))
                        break;
                    index++;
                }
            }
        }

        [TestMethod]
        public void SpCmdTimeOutTest()
        {
            // Use a named database instance that refers to a SQL Server database.
            SqlDatabase dbSQL = DatabaseFactory.CreateDatabase("DataSQLTest") as SqlDatabase;
            // Use "FOR XML AUTO" to have SQL return XML data
            string spName = "itemsgetTimeOutxml";
            DbCommand dbCommandWrapper = dbSQL.GetStoredProcCommand(spName) as DbCommand;
            dbCommandWrapper.CommandTimeout = 5;
            XmlReader productsReader = null;
            string exceptionMessage = "";
            try
            {
                productsReader = dbSQL.ExecuteXmlReader(dbCommandWrapper);
            }
            catch (SqlException e)
            {
                exceptionMessage = e.Message;
            }
            if (exceptionMessage.IndexOf("Timeout expired") == -1)
            {
                Assert.Fail();
            }
        }

        #endregion

        #region "ExecuteXmlReader-command(sql,sp),transaction(commit)"
        [TestMethod]
        public void ExecuteXmlReaderSqlCommandTransactionCommitTest()
        {
            // Use a named database instance that refers to a SQL Server database.
            SqlDatabase dbSQL = DatabaseFactory.CreateDatabase("DataSQLTest") as SqlDatabase;
            // Use "FOR XML AUTO" to have SQL return XML data
            string sqlCommand = "Select ItemDescription from Items FOR XML AUTO";
            DbCommand dbCommandWrapper = dbSQL.GetSqlStringCommand(sqlCommand) as DbCommand;
            XmlReader productsReader = null;
            using (DbConnection connection = dbSQL.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                productsReader = dbSQL.ExecuteXmlReader(dbCommandWrapper, transaction);
                transaction.Commit();
                connection.Close();
            }
            DataSet da = new DataSet();
            da.ReadXml(ExecuteXMLfile);
            int count = da.Tables[0].Rows.Count;
            int index = 0;
            // Iterate through the XmlReader and put the data into our results.
            while (!productsReader.EOF)
            {
                if (productsReader.IsStartElement())
                {
                    Assert.AreEqual(productsReader.ReadOuterXml().ToString().Trim(), da.Tables[0].Rows[index][0].ToString().Trim());
                    if (index == (count - 1))
                        break;
                    index++;
                }
            }
        }

        [TestMethod]
        public void ExecuteXmlReaderSpCommandTransactionCommitTest()
        {
            // Use a named database instance that refers to a SQL Server database.
            SqlDatabase dbSQL = DatabaseFactory.CreateDatabase("DataSQLTest") as SqlDatabase;
            // Use "FOR XML AUTO" to have SQL return XML data
            string spName = "itemsgetxml";
            DbCommand dbCommandWrapper = dbSQL.GetStoredProcCommand(spName) as DbCommand;
            XmlReader productsReader = null;
            using (DbConnection connection = dbSQL.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                productsReader = dbSQL.ExecuteXmlReader(dbCommandWrapper, transaction);
                transaction.Commit();
                connection.Close();
            }
            DataSet da = new DataSet();
            da.ReadXml(ExecuteXMLfile);
            int count = da.Tables[0].Rows.Count;
            int index = 0;
            // Iterate through the XmlReader and put the data into our results.
            while (!productsReader.EOF)
            {
                if (productsReader.IsStartElement())
                {
                    Assert.AreEqual(productsReader.ReadOuterXml().ToString().Trim(), da.Tables[0].Rows[index][0].ToString().Trim());
                    if (index == (count - 1))
                        break;
                    index++;
                }
            }
        }
        #endregion

        #region Exception Test
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]  
        public void NonSqlCommandExecuteReader()
        {
            SqlDatabase dbSQL = DatabaseFactory.CreateDatabase("DataSQLTest") as SqlDatabase;
            dbSQL.ExecuteXmlReader(new OracleCommand());
        }
        #endregion

    }
}
