using System;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Diagnostics;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Data;
using System.Data.SqlClient;
using System.Data.Common;
namespace DataFuncTests.DataSQLServerTests
{
    /// <summary>
    /// Summary description for PerfMonitorTestFixture
    /// </summary>
    [TestClass]
    public class PerfMonitorTestFixture
    {
        public PerfMonitorTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            //  TODO: Add test initialization code
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        [TestMethod]
        public void PerfMonCmdFailedTestFixture()
        {
            string counterName = "Commands Failed/sec";
            int initialCount = GetCounterValue(counterName);
            int loggedCount = 0;
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string sqlCommand = "Select ItemDescription from Item order by ItemId";
            try
            {
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
                {
                }
            }
            catch (Exception)
            {
            }

            loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 1, initialCount);    
        }

        [TestMethod]
        public void PerfMonCmdExecutedTestFixture()
        {
            string counterName = "Commands Executed/sec";
            int initialCount = GetCounterValue(counterName);
            int loggedCount = 0;
             
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string sqlCommand = "Select ItemDescription from Items order by ItemId";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
            }
            loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 1, initialCount);    
        }

        [TestMethod]
        public void PerfMonConnFailedTestFixture()
        {
            string counterName = "Connections Failed/sec";
            int initialCount = GetCounterValue(counterName);
            int loggedCount=0;
            try
            {
                Database db = DatabaseFactory.CreateDatabase("DBDoesNotExist");
                string sqlCommand = "select ItemDescription from items where ItemID=1";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                object actualResult = db.ExecuteScalar(CommandType.Text, sqlCommand);
            }
            catch
            {
            }
            loggedCount = GetCounterValue(counterName);
                
            Assert.AreEqual(loggedCount - 1, initialCount);
        }

        [TestMethod]
        public void PerfMonConnOpenTestFixture()
        {
            string counterName = "Connections Opened/sec";
            int initialCount = GetCounterValue(counterName); 
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            int loggedCount;

            string sqlCommand = "Select ItemDescription from Items order by ItemId";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
                loggedCount = GetCounterValue(counterName);
                
            }
            Assert.AreEqual(loggedCount - 1, initialCount);
            
        }
        private int GetCounterValue(string counterName)
        {
            //Verify the perf counter
            string categoryName = "Enterprise Library Data Counters";
            string instanceName = new AppDomainNameFormatter().CreateName("Total");
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter(categoryName,counterName,instanceName))
                { 

                    return (int)counter.RawValue;
                }
            }
            return 0;
        }
        



        # region Helper Method

        private void CreateCommandParameter(Database db, DbCommand comm, string Parametername, DbType dbType, ParameterDirection dir, string SrcColumn, DataRowVersion rowVersion)
        {

            if (dir == ParameterDirection.Input)
            {
                db.AddInParameter(comm, Parametername, dbType, SrcColumn, rowVersion);
            }

            //DbParameter cmd = comm.CreateParameter();
            //cmd.ParameterName = Parametername;
            ////  cmd.Value = value;
            //cmd.SourceColumn = SrcColumn;
            //cmd.Direction = dir;
            //cmd.DbType = dbType;
            //cmd.SourceVersion = rowVersion;
            //return (cmd);
        }
        # endregion


        
    }
}
