using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.IO;

namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for CustomHandlerTestFixture
    /// </summary>
    [TestClass]
    public class CustomHandlerTestFixture
    {
        private const string fileName = "CustomHandler.txt";
        private bool fileLogged;
        public CustomHandlerTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
         //Use TestInitialize to run code before running each test 
         [TestInitialize()]
         public void MyTestInitialize() 
         {
             if (File.Exists(fileName))
                 File.Delete(fileName);
         }
        
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void CustomThrowNewTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: Security Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Security.SecurityException(exceptionMessage);
                rethrow = ExceptionPolicy.HandleException(originalException, "Custom Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual(exceptionMessage, e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains(exceptionMessage) == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void WrapCustomThrowNewTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: Security Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Security.SecurityException(exceptionMessage);
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Custom Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("Wrapped Exception: Security with Verification", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
                Assert.AreEqual("Original Exception: Security Exception", e.InnerException.Message);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Wrapped Exception: Security with Verification") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void CustomWrapThrowNewTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: Security Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Security.SecurityException(exceptionMessage);
                rethrow = ExceptionPolicy.HandleException(originalException, "Custom Wrap Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("Wrapped Exception: Security with Verification", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
                Assert.AreEqual("Original Exception: Security Exception", e.InnerException.Message);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains(exceptionMessage) == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void ReplaceCustomThrowNewTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: Security Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Security.SecurityException(exceptionMessage);
                rethrow = ExceptionPolicy.HandleException(originalException, "Replace Custom Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("Replaced Exception: Replace Security with Verification", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Replaced Exception: Replace Security with Verification") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void CustomReplaceThrowNewTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: Security Exception";
            bool rethrow = false;
            
            try
            {
                Exception originalException = new System.Security.SecurityException(exceptionMessage);
                rethrow = ExceptionPolicy.HandleException(originalException, "Custom Replace Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("Replaced Exception: Replace Security with Verification", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains(exceptionMessage) == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }
    }
}
