using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;

namespace ExceptionHandlingCoreTests
{
    public class DictionarySourceSection
    {
        public DictionaryConfigurationSource BuildDictionarySourceSection()
        {
            DictionaryConfigurationSource section = new DictionaryConfigurationSource();
            ExceptionHandlingSettings setting = new ExceptionHandlingSettings();

            #region "Replace Handler Policies"
            ExceptionPolicyData epd = new ExceptionPolicyData("Replace ThrowNew Diff Assembly Dic");
            ExceptionTypeData etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            ReplaceHandlerData rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security Exception replaced by Application Exception";
            rhd.ReplaceExceptionType = Type.GetType("System.ApplicationException, mscorlib");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace Handler without Type Dic");
            etd = new ExceptionTypeData("ReadOnlyException", Type.GetType("System.Data.ReadOnlyException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"), PostHandlingAction.None);
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: A recoverable error occurred while attempting to access the database.";
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Invalid Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name ="Replace Handler";
            rhd.ExceptionMessage = "Security Exception replaced by Application Exception";
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace ThrowNew Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData("Replace Handler", "Replaced Exception: Security Exception Replaced with Verification Exception",Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"));
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace But Exception Type Not in Policy Dict");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: Security Exception Replaced with Verification Exception";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace Rethrow Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.NotifyRethrow;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Rethrow the exception";
            rhd.ReplaceExceptionType = Type.GetType("System.ApplicationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace with None Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: User is not authorized to peform the requested action.";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace Multiple Exception Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "File Not Found replaced with Application Exception";
            rhd.ReplaceExceptionType = Type.GetType("System.ApplicationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            
            etd = new ExceptionTypeData();
            etd.Name = "StrongTypingException";
            etd.Type = Type.GetType("System.Data.StrongTypingException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Exception replaced with Not Implemented Exception";
            rhd.ReplaceExceptionType = Type.GetType("System.NotImplementedException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace Replace Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler Verification";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: Replace Security with Verification";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);

            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler HostProtection";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: Replace Verification with HostProtection";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.HostProtectionException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace Wrap Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler Verification";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: Replace Security with Verification";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);

            WrapHandlerData whd = new WrapHandlerData();
            whd.Name = "Wrap Handler Xml Syntax";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrapped Exception: Wrap Verification with Xml Syntax";
            whd.WrapExceptionType = Type.GetType("System.Security.XmlSyntaxException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace Log To File ThrowNew");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 6 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            
           
            epd.ExceptionTypes.Add(etd);

            setting.ExceptionPolicies.Add(epd);

            #endregion

            #region "Wrap Handler Policies"
            epd = new ExceptionPolicyData("Wrap ThrowNew Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "DBConcurrencyException";
            etd.Type = Type.GetType("System.Data.DBConcurrencyException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrapped Exception 1: A recoverable error occurred while attempting to access the database.";
            whd.WrapExceptionType = Type.GetType("System.Data.DataException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap ThrowNew Different Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "DBConcurrencyException";
            etd.Type = Type.GetType("System.Data.DBConcurrencyException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData("Wrap Handler","Wrapped Exception 2: A recoverable error occurred while attempting to access the database.", Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"));
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Rethrow Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.NotifyRethrow;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "File Not Found Exception wrapped with different assembly";
            whd.WrapExceptionType = Type.GetType("System.ApplicationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap with None Dic");
            etd = new ExceptionTypeData();
            etd.Name = "ReadOnlyException";
            etd.Type = Type.GetType("System.Data.ReadOnlyException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrapped Exception: A recoverable error occurred while attempting to access the database.";
            whd.WrapExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Handler without Type Dic");
            etd = new ExceptionTypeData();
            etd.Name = "ReadOnlyException";
            etd.Type = Type.GetType("System.Data.ReadOnlyException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrapped Exception: A recoverable error occurred while attempting to access the database.";
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Multiple Exception Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "File Not Found wrapped with Application Exception";
            whd.WrapExceptionType = Type.GetType("System.ApplicationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);

            etd = new ExceptionTypeData();
            etd.Name = "StrongTypingException";
            etd.Type = Type.GetType("System.Data.StrongTypingException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Exception wrapped with Not Implemented Exception";
            whd.WrapExceptionType = Type.GetType("System.NotImplementedException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);

            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Wrap ThrowNew Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler Verification";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrapping level 1: Security Exception Wrapped with Verification Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler Xml Syntax";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrapping level 2: Verification Exception Wrapped with XML Syntax Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.XmlSyntaxException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Wrap ThrowNew Diff Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler IO";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security Exception Wrapped with IO Exception";
            whd.WrapExceptionType = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler Application";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "IO Exception wrapped with Application Exception";
            whd.WrapExceptionType = Type.GetType("System.ApplicationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Wrap Rethrow Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.NotifyRethrow;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler Verification";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security Exception wrapped with Verification Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler Xml";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Verification Exception wrapped with XML Syntax Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.XmlSyntaxException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Wrap Rethrow Diff Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.NotifyRethrow;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler IO";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrap Security with IO";
            whd.WrapExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler Application";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrap IO with Application";
            whd.WrapExceptionType = Type.GetType("System.ApplicationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Replace Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "File Not Found Exception wrapped with Security Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security Exception wrapped with DataException";
            rhd.ReplaceExceptionType = Type.GetType("System.Data.DataException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            #endregion

            #region "Custom Handler Policies"
            epd = new ExceptionPolicyData("Custom Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "DBConcurrencyException";
            etd.Type = Type.GetType("System.Data.DBConcurrencyException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            CustomHandlerData chd = new CustomHandlerData("Custom Handler", Type.GetType("ExceptionHandlingCoreTests.CustomExceptionHandler, ExceptionHandlingCoreTests"));
            etd.ExceptionHandlers.Add(chd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap Custom Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "File Not Found Exception wrapped with Security Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            chd = new CustomHandlerData();
            chd.Name = "Custom Handler";
            chd.Type = Type.GetType("ExceptionHandlingCoreTests.CustomExceptionHandler, ExceptionHandlingCoreTests");
            etd.ExceptionHandlers.Add(chd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace Custom Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler Verification";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: Replace Security with Verification";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);

            chd = new CustomHandlerData();
            chd.Name = "Custom Handler";
            chd.Type = Type.GetType("ExceptionHandlingCoreTests.CustomExceptionHandler, ExceptionHandlingCoreTests");
            etd.ExceptionHandlers.Add(chd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Custom Replace Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            chd = new CustomHandlerData();
            chd.Name = "Custom Handler";
            chd.Type = Type.GetType("ExceptionHandlingCoreTests.CustomExceptionHandler, ExceptionHandlingCoreTests");
            etd.ExceptionHandlers.Add(chd); 
            
            rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler Verification";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: Replace Security with Verification";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Custom Wrap Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;

            chd = new CustomHandlerData();
            chd.Name = "Custom Handler";
            chd.Type = Type.GetType("ExceptionHandlingCoreTests.CustomExceptionHandler, ExceptionHandlingCoreTests");
            etd.ExceptionHandlers.Add(chd); 
            
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "File Not Found Exception wrapped with Security Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);

            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            #endregion

            #region "Misc Test Cases"
            epd = new ExceptionPolicyData("No Exception Type Dic");
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("No Handler with Rethrow Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.NotifyRethrow;
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("No Handler with Throw New Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("No Handler with None Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            
            #endregion

                       
            section.Add("exceptionHandling", setting);
            InstrumentationConfigurationSection ics = new InstrumentationConfigurationSection(true, true, true);
            section.Add("instrumentationConfiguration", ics);
            return section;
        }
    }
}
