using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;
using System.Configuration;
using System.Messaging;


namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for WrapHandlerTestFixture
    /// </summary>
    [TestClass]
    public class WrapHandlerTestFixture
    {
        public WrapHandlerTestFixture()
        {
        }

        [TestInitialize()]
        public void Initialize()
        {

        }

        [TestCleanup()]
        public void Cleanup()
        {
        }



        #region "Wrap Test Cases"
        [TestMethod]
        public void WrapThrowNewSameAssemblyTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Data.DBConcurrencyException("Original Exception: Concurrency Exception wrapped by Data Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap ThrowNew Same Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Wrapped Exception 1: A recoverable error occurred while attempting to access the database.", e.Message);
                Assert.AreEqual("Original Exception: Concurrency Exception wrapped by Data Exception", e.InnerException.Message);
                Assert.AreEqual("DataException", e.GetType().Name);
                Assert.AreEqual("DBConcurrencyException", e.InnerException.GetType().Name);


            }
            
            Assert.AreEqual(true, exceptionRaised);

        }


        [TestMethod]
        public void WrapThrowNewDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Data.DBConcurrencyException("Original Exception: Concurrency Exception wrapped by IO Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap ThrowNew Different Assembly");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {

                exceptionRaised = true;
                Assert.AreEqual("Wrapped Exception 2: A recoverable error occurred while attempting to access the database.", e.Message);
                Assert.AreEqual("Original Exception: Concurrency Exception wrapped by IO Exception", e.InnerException.Message);
                Assert.AreEqual("FileNotFoundException", e.GetType().Name);
                Assert.AreEqual("DBConcurrencyException", e.InnerException.GetType().Name);
            }
            Assert.AreEqual(true, exceptionRaised);


        }

        [TestMethod]
        public void WrapPostHandlingRethrowTest()
        {
            bool rethrow = false;
            Exception originalException = new System.IO.FileNotFoundException("Original Exception: File Not found exception");
            rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Rethrow");
            Assert.IsTrue(rethrow);


        }


        [TestMethod]
        public void WrapwithPostHandlingNoneTest()
        {

            Exception originalException = new System.Data.ReadOnlyException("Original Exception: Read Only Exception wrapped by Security Exception");
            bool rethrow = false;
            rethrow = ExceptionPolicy.HandleException(originalException, "Wrap with None");

            if (rethrow) throw originalException;
            Assert.IsFalse(rethrow);

        }


        [TestMethod]
        [ExpectedException(typeof(ExceptionHandlingException))]
        public void WrapAndHandlingExceptionOccursInChainTest()
        {
            bool rethrow = false;

            try
            {
                Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred which is not handled");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Handling Exception Occurs In Chain Test");
                if (rethrow) throw originalException;

            }
            catch (ExceptionHandlingException ex)
            {
                bool isLastHandlerWrap = ex.Message.Contains("Wrap Handler");
                string logName = EventLog.LogNameFromSourceName("Application", ".");
                using (EventLog eventlog = new EventLog(logName))
                {
                    int entryCount = eventlog.Entries.Count;


                    Assert.IsTrue(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("POLICY NAME: Wrap And Handling Exception Occurs In Chain Test"));
                    Assert.IsTrue(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("OFFENDING EXCEPTION:"));
                    Assert.IsTrue(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("ORIGINAL EXCEPTION:"));

                }
                throw;

            }
            Assert.Fail();


        }

        [TestMethod]
        public void WrapMultipleExceptionHandlingTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {

                Exception originalException = new System.Data.StrongTypingException("Original Exception: Strong Typing Exception wrapped with Not Implemented Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Multiple Exception Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Exception wrapped with Not Implemented Exception", e.Message);
                Assert.AreEqual("Original Exception: Strong Typing Exception wrapped with Not Implemented Exception", e.InnerException.Message);
                Assert.AreEqual("NotImplementedException", e.GetType().Name);
                Assert.AreEqual("StrongTypingException", e.InnerException.GetType().Name);
            }


            Assert.AreEqual(true, exceptionRaised);
            exceptionRaised = false;

            rethrow = false;
            try
            {
                Exception originalException = new System.IO.FileNotFoundException("Original Exception: File Not Found wrapped with Application Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Multiple Exception Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("File Not Found wrapped with Application Exception", e.Message);
                Assert.AreEqual("Original Exception: File Not Found wrapped with Application Exception", e.InnerException.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
                Assert.AreEqual("FileNotFoundException", e.InnerException.GetType().Name);
            }
            Assert.AreEqual(true, exceptionRaised);
        }

        #endregion

        #region "Multiple Handlers"

        [TestMethod]
        public void WrapWrapThrowNewSameAssemblyTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by Verification Exception which is wrapped by XMLSyntaxException");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Wrap ThrowNew Same Assembly");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Wrapping level 2: Verification Exception Wrapped with XML Syntax Exception", e.Message);
                Assert.AreEqual("Wrapping level 1: Security Exception Wrapped with Verification Exception", e.InnerException.Message);
                Assert.AreEqual("Original Exception: Security Exception wrapped by Verification Exception which is wrapped by XMLSyntaxException", e.InnerException.InnerException.Message);
                Assert.AreEqual("XmlSyntaxException", e.GetType().Name);
                Assert.AreEqual("VerificationException", e.InnerException.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.InnerException.GetType().Name);
            }
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void WrapWrapThrowNewDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is wrapped by Application Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Wrap ThrowNew Diff Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("IO Exception wrapped with Application Exception", e.Message);
                Assert.AreEqual("Security Exception Wrapped with IO Exception", e.InnerException.Message);
                Assert.AreEqual("Original Exception: Security Exception wrapped by IO Exception which is wrapped by Application Exception", e.InnerException.InnerException.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
                Assert.AreEqual("FileNotFoundException", e.InnerException.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.InnerException.GetType().Name);
            }

            Assert.AreEqual(true, exceptionRaised);

        }



        [TestMethod]
        public void WrapWrapRethrowTest()
        {
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception rethrown");
            rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Wrap Rethrow");
            Assert.IsTrue(rethrow);
        }


        [TestMethod]
        public void WrapReplaceHandlerTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
                Exception originalException = new System.IO.FileNotFoundException("Original Exception: FileNotFound Exception wrapped by Security Exception which is replaced by Data Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap Replace Handler");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception wrapped with DataException", e.Message);
                Assert.AreEqual("DataException", e.GetType().Name);
                Assert.IsNull(e.InnerException);
            }

            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void WrapExceptionCreateDirect()
        {
            Exception originalException = new System.ArithmeticException("Arithmetic Exception is not handled");
            WrapHandler handler = new WrapHandler("Wrapped Exception: Arithmetic Exception wrapped with Security Exception", Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"));
            Exception wrappedException = handler.HandleException(originalException, new Guid());
            Assert.IsTrue(wrappedException is System.Security.SecurityException);
            Assert.AreEqual(wrappedException.Message, "Wrapped Exception: Arithmetic Exception wrapped with Security Exception");
            Assert.IsTrue(wrappedException.InnerException is ArithmeticException);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void WrapHandlerExceptionTest()
        {
            Exception originalException = new System.ArithmeticException("Arithmetic Exception is not handled");
            WrapHandler handler = new WrapHandler("Replaced Exception: Arithmetic Exception replaced with Security Exception", Type.GetType("System.String, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"));
        }


        #endregion

    }
}
