using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;

using System.IO;
using System.Data.SqlClient;
using System.Data.Common;
using System.Management;
using System.Diagnostics;

namespace ExceptionHandlingLoggingTests
{
    /// <summary>
    /// Summary description for CustomHandlerLoggingTestFixture
    /// </summary>
    [TestClass]
    public class CustomHandlerLoggingTestFixture
    {
        string connString = "server=.\\sqlexpress;database=Logging;Integrated Security=true";
        private const string fileName = "CustomHandler.txt";
        private bool fileLogged;
        public CustomHandlerLoggingTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        [TestInitialize()]
        public void Initialize()
        {
            Microsoft.Practices.EnterpriseLibrary.Data.Database db = DatabaseFactory.CreateDatabase();
            DbCommand command = db.GetStoredProcCommand("ClearLogs");
            db.ExecuteNonQuery(command);

            if (File.Exists(fileName))
                File.Delete(fileName);

        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void LoggingDBSinkCustomHandlerTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged in database");
            try
            {
                ExceptionPolicy.HandleException(originalException, "Log To Database Custom Handler");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception logged in database", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);

            }
            Assert.AreEqual(3, CustomExceptionHandlerLogging.attributes.Count);
            Assert.AreEqual("Microsoft Office", CustomExceptionHandlerLogging.attributes["ProductName"]);
            Assert.AreEqual("25", CustomExceptionHandlerLogging.attributes["Price"]);
            Assert.AreEqual("12", CustomExceptionHandlerLogging.attributes["Quantity"]);
           
            Assert.IsFalse(rethrow);

            string title;
            using (SqlConnection dbConnection = new SqlConnection(connString))
            {
                dbConnection.Open();
                SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=222", dbConnection);

                object retValue = cmdRetrieveData.ExecuteScalar();
                if (retValue == null)
                {
                    Assert.Fail("No Log Entry in Database");
                }

                cmdRetrieveData.Dispose();

                title = (string)retValue;
            }


            Assert.AreEqual("Log To Database Custom Handler", title);
            Assert.AreEqual(true, exceptionRaised);

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Security Exception logged in database") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);


        }

        [TestMethod]
        public void LoggingFileSinkCustomHandlerTest()
        {
            bool rethrow = false;
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged to flat file with Custom Handler");

            ExceptionPolicy.HandleException(originalException, "Log To Flat File Custom Handler");
            if (rethrow) throw originalException;
            Assert.IsFalse(rethrow);

            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //if (File.ReadAllText("FlatFileTestOutput\\SimpleEntLibLog.txt").Contains("Security Exception logged to flat file with Custom Handler") == false)
                    //Assert.Fail("Exception not logged to flat file");
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

            
            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Security Exception logged to flat file with Custom Handler") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);

         
        }

        [TestMethod]
        public void LoggingEmailSinkCustomHandlerTest()
        {
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged to Email Sink with Custom Handler");

            ExceptionPolicy.HandleException(originalException, "Log To Email Sink Custom Handler");
            if (rethrow) throw originalException;
            Assert.IsFalse(rethrow);

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Security Exception logged to Email Sink with Custom Handler") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);


        }

        [TestMethod]
        public void LoggingEventSinkCustomHandlerTest()
        {
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged in event sink");
            originalException.Data.Add("key1", "value1");
            rethrow = ExceptionPolicy.HandleException(originalException, "Log To Event Sink Custom Handler");

            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {
                int entryCount = eventlog.Entries.Count;
                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(231), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Security Exception logged in event sink") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);


        }


        [TestMethod]
        public void CustomHandlerLoggingDBSinkTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged in database");
            try
            {
                ExceptionPolicy.HandleException(originalException, "Custom Handler Log To Database");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception logged in database", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);

            }
            Assert.IsFalse(rethrow);

            string title;
            using (SqlConnection dbConnection = new SqlConnection(connString))
            {
                dbConnection.Open();
                SqlCommand cmdRetrieveData = new SqlCommand("select title from log where eventid=232", dbConnection);

                object retValue = cmdRetrieveData.ExecuteScalar();
                if (retValue == null)
                {
                    Assert.Fail("No Log Entry in Database");
                }

                cmdRetrieveData.Dispose();

                title = (string)retValue;
            }


            Assert.AreEqual("Custom Handler Log To Database", title);
            Assert.AreEqual(true, exceptionRaised);

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Security Exception logged in database") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);


        }

        [TestMethod]
        public void CustomHandlerLoggingFileSinkTest()
        {
            bool rethrow = false;
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Custom Handler and Security Exception logged to flat file");

            ExceptionPolicy.HandleException(originalException, "Custom Handler Log To Flat File");
            if (rethrow) throw originalException;
            Assert.IsFalse(rethrow);

            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //if (File.ReadAllText("FlatFileTestOutput\\SimpleEntLibLog.txt").Contains("Custom Handler and Security Exception logged to flat file") == false)
                    //Assert.Fail("Exception not logged to flat file");

               // File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);


            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Custom Handler and Security Exception logged to flat file") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);


        }

        [TestMethod]
        public void CustomHandlerLoggingEmailSinkTest()
        {
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Custom Handler and Security Exception logged to Email Sink");

            ExceptionPolicy.HandleException(originalException, "Custom Handler Log To Email Sink");
            if (rethrow) throw originalException;
            Assert.IsFalse(rethrow);

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Custom Handler and Security Exception logged to Email Sink") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);


        }

        [TestMethod]
        public void CustomHandlerLoggingEventSinkTest()
        {
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Custom Handler and Security Exception logged in event sink");
            rethrow = ExceptionPolicy.HandleException(originalException, "Custom Handler Log To Event Sink");

            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(234), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }


            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Custom Handler and Security Exception logged in event sink") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, fileLogged);


        }
    }
}
