using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.IO;
using System.Diagnostics;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Data.SqlClient;
using System.Data;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.Data.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.AzMan;
using System.Security.Principal;

namespace FileSourceBlockTests
{
    /// <summary>
    /// Summary description for ReplaceHandlerTestFixture
    /// </summary>
    [TestClass]
    public class FileSourceBlockTestFixture
    {

        String connStr;
        String itemsXMLfile;
        FileConfigurationSource exceptionConfigSource;
        FileConfigurationSource dataConfigSource;
        FileConfigurationSource azManSecurityConfigSource;
        FileConfigurationSource cacheSecurityConfigSource;
        FileConfigurationSource authRuleSecurityConfigSource;
        FileConfigurationSource cacheConfigSource;
        public FileSourceBlockTestFixture()
        {
 
        }

        [TestInitialize()]
        public void Initialize()
        {
            exceptionConfigSource = new FileConfigurationSource(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ExceptionHandlingConfiguration.Config"));
            dataConfigSource = new FileConfigurationSource(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "DataConfiguration.Config"));
            azManSecurityConfigSource = new FileConfigurationSource(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "SecurityAzManConfiguration.config"));
            cacheSecurityConfigSource = new FileConfigurationSource(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "SecurityCachingConfiguration.config"));
            authRuleSecurityConfigSource = new FileConfigurationSource(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "SecurityAuthRuleConfiguration.config"));
            cacheConfigSource = new FileConfigurationSource(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "CachingConfiguration.config"));
            itemsXMLfile = "Items.xml";

        }

 
        [TestCleanup()]
        public void Cleanup()
        {
        }


        #region "Replace Test Cases"
        [TestMethod]
        public void ReplaceThrowNewSameAssemblyTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
               Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception");
               ExceptionPolicyImpl epi = EnterpriseLibraryFactory.BuildUp<ExceptionPolicyImpl>("Replace ThrowNew Same Assembly", exceptionConfigSource); 
                
                rethrow = epi.HandleException(originalException);
               if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: Security Exception Replaced with Verification Exception", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }
         
            Assert.AreEqual(true, exceptionRaised);
        }




        [TestMethod]
        public void DBCommandQuerySqlTest()
        {
            //Expected Data
            DataSet dsExpectedData = new DataSet();
            dsExpectedData.ReadXml(itemsXMLfile);
            int count = 0;
            DatabaseProviderFactory factory = new DatabaseProviderFactory(dataConfigSource);

            Database db = factory.Create("SQLTest"); 
            string sqlCommand = "Select ItemDescription from Items order by ItemId";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
                while (dataReader.Read())
                {
                    Assert.AreEqual(dsExpectedData.Tables[0].Rows[count][0].ToString(), dataReader["ItemDescription"].ToString().Trim());
                    count++;
                }
            }

            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        public void DBCommandQueryGenericTest()
        {
            //Expected Data
            DataSet dsExpectedData = new DataSet();
            dsExpectedData.ReadXml(itemsXMLfile);
            int count = 0;
            DatabaseProviderFactory factory = new DatabaseProviderFactory(dataConfigSource);

            Database db = factory.Create("GenericSQLTest");
            string sqlCommand = "Select ItemDescription from Items order by ItemId";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
            {
                while (dataReader.Read())
                {
                    Assert.AreEqual(dsExpectedData.Tables[0].Rows[count][0].ToString(), dataReader["ItemDescription"].ToString().Trim());
                    count++;
                }
            }

            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }


        [TestMethod]
        [ExpectedException(typeof(ApplicationException))]
        public void DataExceptionTest()
        {
            
            DatabaseProviderFactory factory = new DatabaseProviderFactory(dataConfigSource);

            Database db = factory.Create("SQLTest");
            string sqlCommand = "Select ItemDescription from Item order by ItemId";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            try
            {
                using (IDataReader dataReader = db.ExecuteReader(dbCommandWrapper))
                {
                    while (dataReader.Read())
                    {

                    }
                }
            }
            catch (Exception e)
            {
                ExceptionPolicyImpl epi = EnterpriseLibraryFactory.BuildUp<ExceptionPolicyImpl>("Data Exception Policy", exceptionConfigSource);

                bool rethrow = epi.HandleException(e);
                if (rethrow) throw e;
            }


            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        
        #endregion
        [TestMethod]
        public void SecurityAzManTaskAuthorizeTest()
        {
            AuthorizationProviderFactory authFactory = new AuthorizationProviderFactory(azManSecurityConfigSource);
            IAuthorizationProvider azManProvider = authFactory.Create("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();

            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Authorize Purchase");
            Assert.IsTrue(isAuthorized);

        }


        [TestMethod]
        public void SecurityInMemoryAddIdentityTest()
        {
            SecurityCacheProviderFactory cacheFactory = new SecurityCacheProviderFactory(cacheSecurityConfigSource);
            ISecurityCacheProvider securityCache = cacheFactory.Create("CacheProvider");
            Assert.IsNotNull(securityCache);
            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }


        [TestMethod]
        public void SecurityIsolatedAddIdentityTest()
        {
            SecurityCacheProviderFactory cacheFactory = new SecurityCacheProviderFactory(cacheSecurityConfigSource);
            ISecurityCacheProvider securityCache = cacheFactory.Create("CacheProviderIsolated");
            Assert.IsNotNull(securityCache);

            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }

        [TestMethod]
        public void SecurityDBAddIdentityTest()
        {
            SecurityCacheProviderFactory cacheFactory = new SecurityCacheProviderFactory(cacheSecurityConfigSource);
            ISecurityCacheProvider securityCache = cacheFactory.Create("CacheProviderDB");
            Assert.IsNotNull(securityCache);

            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }

        [TestMethod]
        public void SecurityAuthRuleIdentityTest()
        {
            IIdentity identity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal principal = new GenericPrincipal(identity, roles); 
            
            AuthorizationProviderFactory authFactory = new AuthorizationProviderFactory(authRuleSecurityConfigSource);
            AuthorizationRuleProvider authRuleProvider = authFactory.Create("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void CachingInMemoryStoreTest()
        {
            CacheManagerFactory factory = new CacheManagerFactory(cacheConfigSource);
            CacheManager itemCache = factory.Create("DefaultCacheManager");
            if (itemCache != null)
                itemCache.Flush();
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

        }

       

        [TestMethod]
        public void CachingIsolatedStoreTest()
        {
            CacheManagerFactory factory = new CacheManagerFactory(cacheConfigSource);
            CacheManager itemCache = factory.Create("IsolatedCacheManager");
            if (itemCache != null)
                itemCache.Flush();
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
        }

        [TestMethod]
        public void DBStoreAddDefaultTest()
        {
            int count = 0;
            CacheManagerFactory factory = new CacheManagerFactory(cacheConfigSource);
            CacheManager itemCache = factory.Create("DBCacheManager");
            if (itemCache != null)
                itemCache.Flush();
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = "server=.\\sqlexpress;initial catalog=caching;integrated security=SSPI";
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1') and PartitionName = 'Partition3'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();
            conn.Close();
        }
    }
}
