using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Messaging;
using System.Diagnostics;
using System.Configuration; 
using System.Management;
using System.Net;
using System.IO;
using System.Data.Common;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration; 
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration; 
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database.Configuration; 
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

using Microsoft.Win32;
using CustomClasses;
using CustomClasses.Configuration;    
namespace ConfigurationSourceTests
{
    /// <summary>
    /// Summary description for DictionarySourceTraceListenersTestFixture
    /// </summary>
    [TestClass]
    public class DictionarySourceTraceListenersTestFixture
    {
        private bool logged = false;
        private string loggedMessage = "";
        private ManagementScope scope = null;
        private ManagementEventWatcher watcher = null;
        private DictionaryConfigurationSource source;

        public DictionarySourceTraceListenersTestFixture()
        {
            source = LoggingConfigurationSource.BuildConfigurationSource(); 
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region FormattedEventLogTraceListener tests
        [TestMethod]
        public void LogToEventLog()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                LogEntry logEntry = new LogEntry();
                logEntry.Message = "LogToEventLog";
                logEntry.Categories.Add("General");   
                int initialCount = log.Entries.Count;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(logEntry);
                }
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message));
            }
        }

        [TestMethod]
        public void LogToEventLogWithCustomLogEntry()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                CustomLogEntry logEntry = new CustomLogEntry();
                logEntry.Message = "LogToEventLogWithCustomLogEntry";
                logEntry.CategoryID = 101;
                logEntry.Categories.Add("FormatterCategory");  
                int initialCount = log.Entries.Count;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(logEntry);
                }
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message) && message.Contains("CategoryID: 101"));
            }
        }
        #endregion

        #region MsmqListener Tests
        [TestMethod]
        public void LogToMsmq()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(new LogEntry("To test Msmq listener using Logger facade", "MsmqCategory", 1, 1, TraceEventType.Critical, "", null));
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using Logger facade"));
            }
        }


        [TestMethod]
        public void LogToMsmqUsingCustomLogEntry()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            MsmqTraceListenerData msmqListener = (MsmqTraceListenerData )setting.TraceListeners.Get("Msmq TraceListener") ;
            string actualFormatter = msmqListener.Formatter;
            msmqListener.Formatter = "CustomFormatter";  
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                CustomLogEntry log = new CustomLogEntry();
                log.CategoryID = 10;
                log.Message = "To test Msmq listener using MsmqListener directly";
                log.Categories.Add("MsmqCategory");
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using MsmqListener directly"));
                Assert.IsTrue(content.Contains("CategoryID: 10"));
            }
            msmqListener.Formatter = actualFormatter;
        }

        [TestMethod]
        public void LogToMsmqUsingMachineNameFormat()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            MsmqTraceListenerData msmqListener = (MsmqTraceListenerData)setting.TraceListeners.Get("Msmq TraceListener");
            string machineName = System.Environment.MachineName;
            string originalQueuePath = msmqListener.QueuePath;
            string actualFormatter = msmqListener.Formatter;
            msmqListener.Formatter = "CustomFormatter";  
            msmqListener.QueuePath = machineName + @"\Private$\EntlibTest";
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                CustomLogEntry log = new CustomLogEntry();
                log.CategoryID = 10;
                log.Message = "To test Msmq listener using MsmqListener directly";
                log.Categories.Add("MsmqCategory");
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using MsmqListener directly"));
                Assert.IsTrue(content.Contains("CategoryID: 10"));
            }
            msmqListener.Formatter = actualFormatter;
            msmqListener.QueuePath = originalQueuePath; 
        }

        [TestMethod]
        public void LogToMsmqUsingTCPIPFormat()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            MsmqTraceListenerData msmqListener = (MsmqTraceListenerData)setting.TraceListeners.Get("Msmq TraceListener");
            string hostName = Dns.GetHostName();
            IPHostEntry ipEntry = Dns.GetHostByName(hostName);
            IPAddress[] addr = ipEntry.AddressList;
            string ipAddress = addr[0].ToString();
            string originalQueuePath = msmqListener.QueuePath;
            string actualFormatter = msmqListener.Formatter;
            msmqListener.Formatter = "CustomFormatter";
            msmqListener.QueuePath = "Formatname:DIRECT=TCP:" + ipAddress + @"\Private$\EntlibTest";

            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                CustomLogEntry log = new CustomLogEntry();
                log.CategoryID = 10;
                log.Message = "To test Msmq listener using MsmqListener directly";
                log.Categories.Add("MsmqCategory");
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using MsmqListener directly"));
                Assert.IsTrue(content.Contains("CategoryID: 10"));
            }
            msmqListener.Formatter = actualFormatter;
            msmqListener.QueuePath = originalQueuePath; 
        }

        [TestMethod]
        public void LogToPublicQueue()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            MsmqTraceListenerData msmqListener = (MsmqTraceListenerData)setting.TraceListeners.Get("Msmq TraceListener");
            string hostName = Dns.GetHostName();
            string originalQueuePath = msmqListener.QueuePath;
            msmqListener.QueuePath = hostName + @"\EntlibTestPublic";
            using (MessageQueue queue = new MessageQueue(hostName + @"\EntlibTestPublic"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogEntry log = new LogEntry();
                log.Message = "To test Msmq listener using Public queue";
                log.Categories.Add("MsmqCategory");
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains(log.Message));
            }
            msmqListener.QueuePath = originalQueuePath; 
        }

        [TestMethod]
        public void LogToPublicQueueUsingTCPIPFormat()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            MsmqTraceListenerData msmqListener = (MsmqTraceListenerData)setting.TraceListeners.Get("Msmq TraceListener");
            string hostName = Dns.GetHostName();
            IPHostEntry ipEntry = Dns.GetHostByName(hostName);
            IPAddress[] addr = ipEntry.AddressList;
            string ipAddress = addr[0].ToString();
            string originalQueuePath = msmqListener.QueuePath;
            msmqListener.QueuePath = "Formatname:DIRECT=TCP:" + ipAddress + @"\EntlibTestPublic";
            using (MessageQueue queue = new MessageQueue(hostName + @"\EntlibTestPublic"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogEntry log = new LogEntry();
                log.Message = "To test Msmq listener using Public queue";
                log.Categories.Add("MsmqCategory");
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains(log.Message));
            }
            msmqListener.QueuePath = originalQueuePath; 
        }

        #endregion

        #region DatabaseListener Tests
        [TestMethod]
        public void LogLengthyMesssageToDb()
        {
            LogEntry log = new LogEntry("", "DatabaseCategory", 4, 1000, TraceEventType.Critical, "", null);
            log.Title = "Enterprise Library";
            log.Message = "Enterprise Library is a major new release of the Microsoft patterns & practices application blocks.";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }

            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source); 
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
        }

        [TestMethod]
        public void LogMesssageCategoryAndPriorityToDb()
        {
            LogEntry log = new LogEntry();
            log.Message = "LogMesssageCategoryAndPriorityToDatabaseSink";
            log.Categories.Add("DatabaseCategory");
            log.Priority = 4;
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }

            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, "LogMesssageCategoryAndPriorityToDatabaseSink");
        }

        [TestMethod]
        public void LogToDbListenerUsingCustomLogEntry()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData DBListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string actualFormatter = DBListener.Formatter;  
            DBListener.Formatter = "CustomFormatter"; 

            CustomLogEntry log = new CustomLogEntry();
            log.CategoryID = 5;
            log.Message = "LogToDatabaseUsingCustomLogEntryObject";
            log.Categories.Add("DatabaseCategory");  
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 FormattedMessage FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(messageContents.Contains("CategoryID: 5"));
            Assert.IsTrue(messageContents.Contains("LogToDatabaseUsingCustomLogEntryObject"));
            DBListener.Formatter = actualFormatter;   
        }


        [TestMethod]
        public void LogToDbWithMultipleCategory()
        {
            LogEntry log = new LogEntry();
            log.Message = "Log To Database using Multiple Category";
            log.Categories.Add("DatabaseCategory");
            log.Categories.Add("General");
            log.Priority = 4;
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog eventlog = new EventLog(logName))
            {
                eventlog.Entries[(eventlog.Entries.Count) - 1].Message.Contains(log.Message);
            }
        }

        [TestMethod]
        public void SetContextItemsToDbListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "SetContextItemsToDbListener";
            log.Categories.Add("DatabaseCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.SetContextItem("key1", "value1");
                writer.SetContextItem("key2", "value2");
                writer.Write(log);
            }
            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 FormattedMessage FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(messageContents.Contains("key1") && messageContents.Contains("value1"));
            Assert.IsTrue(messageContents.Contains("key2") && messageContents.Contains("value2"));
        }

        [TestMethod]
        public void LogToDbListenerApplySwitchFilter()
        {
            LogEntry log = new LogEntry();

            log.Message = "LogToDbListenerApplySwitchFilter";
            log.Categories.Add("ErrorCategory");
            log.Priority = 5;
            log.EventId = 100;
            log.Severity = TraceEventType.Information;

            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT count(logId) FROM Log");
            int initialCount = Convert.ToInt32(db.ExecuteScalar(command));
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            int finalCount = Convert.ToInt32(db.ExecuteScalar(command));
            Assert.IsTrue(initialCount == finalCount);
        }

        [TestMethod]
        public void LogToDbListenerAllowSwitchFilter()
        {
            LogEntry log = new LogEntry();
            log.Message = "LogToDbListenerAllowSwitchFilter";
            log.Categories.Add("ErrorCategory");
            log.Priority = 3;
            log.EventId = 100;
            log.Severity = TraceEventType.Critical;
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
        }

        [TestMethod]
        public void LogToDbListenerApplySwitchOff()
        {
            LogEntry log = new LogEntry();
            log.Message = "LogToDbListenerApplySwitchOff";
            log.Categories.Add("SwitchOffCategory");
            log.Priority = 5;
            log.EventId = 100;
            log.Severity = TraceEventType.Information;
            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT count(logId) FROM Log");
            int initialCount = Convert.ToInt32(db.ExecuteScalar(command));
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            int finalCount = Convert.ToInt32(db.ExecuteScalar(command));
            Assert.IsTrue(initialCount == finalCount);
        }


        [TestMethod]
        public void UsingLogWriterToLogToDbListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "Logged using LogWriter";
            log.Categories.Add("DatabaseCategory");
            log.Priority = 2;
            log.EventId = 100;
            log.Severity = TraceEventType.Information;
            log.Title = "UsingLogWriterToLogToDbListener";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            DatabaseProviderFactory Dbfactory = new DatabaseProviderFactory(source);
            Database db = Dbfactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
        }

        #endregion

        #region WMIListener Tests
        [TestMethod]
        public void LogToWmi()
        {
            LogEntry entry = new LogEntry();
            entry.Categories.Add("WMICategory");
            entry.Message = "LogToWmi";
            MonitorWmiEvent();
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }

            System.Threading.Thread.Sleep(1000);
            StopMonitorWmiEvent();
            Assert.IsTrue(logged);
            Assert.IsTrue(loggedMessage.Contains("LogToWmi"), loggedMessage);
        }


        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            loggedMessage = args.NewEvent.GetText(TextFormat.Mof);
            logged = true;
        }

        private void MonitorWmiEvent()
        {
            if (scope == null)
            {
                scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
                scope.Options.EnablePrivileges = true;
                EventQuery eq = new EventQuery("SELECT * FROM LogEntryV20");
                watcher = new ManagementEventWatcher(scope, eq);
            }
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            logged = false;
            loggedMessage = "";
        }

        private void StopMonitorWmiEvent()
        {
            watcher.EventArrived -= new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Stop();
            watcher.Dispose();
        }
        #endregion

        #region FlatFileListener Tests
        [TestMethod]
        public void LogToFlatFile()
        {
            LogEntry log = new LogEntry();
            log.Categories.Add("FileCategory");
            log.Priority = 4;
            log.Title = "LogToFlatFile";
            log.Message = "To test the logwriter using dictionary source for flat file";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }

            Assert.IsTrue(File.Exists("trace.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace.log", log.Message ));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace.log", log.Title));
        }


        [TestMethod]
        public void LogToFlatFileWithCustomLogEntry()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FlatFileTraceListenerData flatfileListener = (FlatFileTraceListenerData)setting.TraceListeners.Get("FlatFile TraceListener");
            string actualFormatter = flatfileListener.Formatter;
            string actualfileName = flatfileListener.FileName;  
            flatfileListener.Formatter = "CustomFormatter";
            flatfileListener.FileName = "trace1.log";
            CustomLogEntry log = new CustomLogEntry();
            log.Categories.Add("FileCategory");  
            log.CategoryID = 100;
            log.Message = "To test FlatFile tracelistner directly for Custom LogEntry";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", log.Message));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "CategoryID: 100"));
            flatfileListener.Formatter = actualFormatter;
            flatfileListener.FileName = actualfileName; 
        }

        [TestMethod]
        public void LogToFlatFileWithStackTrace()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test TraceOutputOption with stack trace";
            log.Title = "LogToFlatFileWithStackTrace";
            log.Categories.Add("FileCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            Assert.IsTrue(File.Exists("stacktrace.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent ("stacktrace.log", log.Message));
            Assert.IsTrue(HelperClass.VerifyFileContent("stacktrace.log", "Callstack=   at System.Environment.GetStackTrace(Exception e, Boolean needFileInfo)"));
        }

        #endregion

        #region SystemDiagnosticsTraceListener Tests
        [TestMethod]
        public void LogUsingSystemDiagnosticsTraceListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test SystemDiagnosticsTraceListener for system and entlib listeners";
            log.Title = "LogUsingSystemDiagnosticsTraceListener";
            log.Categories.Add("SystemLogCategory");

            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog eventlog = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                MonitorWmiEvent();
                int initialCount = eventlog.Entries.Count;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                StopMonitorWmiEvent();

                int finalCount = eventlog.Entries.Count;
                string message = eventlog.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue(message.Contains(log.Message));

                Assert.IsTrue(File.Exists("trace.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace.log", log.Message));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace.log", log.Title));

                System.Threading.Thread.Sleep(2000);
                Assert.IsTrue(logged);
                Assert.IsTrue(loggedMessage.Contains(log.Message));
            }

        }

        #endregion

        #region CustomListener Tests
        [TestMethod]
        public void LogToRegistryListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test CustomTrace Listener that derives from CustomTraceListener";
            log.Title = "LogToRegistryListener";
            log.Categories.Add("RegistryCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"Software\EntlibTestApp");
            string content = (string)key.GetValue("LogEntry");
            key.Close();
            Assert.IsTrue(content.Contains(log.Message));
        }

        [TestMethod]
        public void LogToExtendedFileListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test extension of flatfile Listener ";
            log.Title = "LogToExtendedFileListener";
            log.Categories.Add("ExtendedFileCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            Assert.IsTrue(HelperClass.VerifyFileContent("trace.log", log.Message));
        }
        #endregion


    }
}
