using System;
using System.Collections.Generic;
using System.Text;
using System.Messaging;
using System.Diagnostics;
using System.Configuration;
using System.Management;
using System.Net;
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Data.Common;
using CustomClasses;
using CustomClasses.Configuration; 
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;  

namespace ConfigurationSourceTests
{
    public class LoggingConfigurationSource
    {

        public static DictionaryConfigurationSource BuildConfigurationSource()
        {
            DictionaryConfigurationSource source = new DictionaryConfigurationSource();
            
            LoggingSettings settings = new LoggingSettings("LoggingTests", true, "General");
            settings.LogWarningWhenNoCategoriesMatch = true;
            settings.TracingEnabled = true;

            //Filter Configuration
            CategoryFilterData categoryFilter = new CategoryFilterData();
            categoryFilter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Filters.CategoryFilter, Microsoft.Practices.EnterpriseLibrary.Logging");
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("DenyCategory"));
            categoryFilter.Name = "CategoryFilter";
            PriorityFilterData priorityFilter = new PriorityFilterData("Priority", 1);
            priorityFilter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Filters.PriorityFilter, Microsoft.Practices.EnterpriseLibrary.Logging");
            priorityFilter.MaximumPriority = 4;
            LogEnabledFilterData logEnabledFilter = new LogEnabledFilterData("LogEnabledFilter", true);
            logEnabledFilter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Filters.LogEnabledFilter, Microsoft.Practices.EnterpriseLibrary.Logging");
            CustomLogFilterData sevFilter = new CustomLogFilterData("CustomSevFilter", Type.GetType("CustomClasses.CustomSeverityFilter,CustomClasses"));
            sevFilter.Attributes.Add("key1", "Information");
            sevFilter.Attributes.Add("key2", "Error");
            sevFilter.Attributes.Add("key3", "Warning");
            sevFilter.Attributes.Add("key4", "Start");
            sevFilter.Attributes.Add("key5", "Stop");
            sevFilter.Attributes.Add("key6", "Critical");

            //Formatter Configuration
            //TextFormatterData textFormatter = new TextFormatterData("TextFormat", "Timestamp: {timestamp}\r\nMessage: {message}\r\nCategory: {category}\r\nPriority: {priority}\r\nEventId: {eventid}\r\nSeverity: {severity}\r\nTitle:{title}\r\nMachine: {machine}\r\nApp Domain: {appDomain}\r\nProcess Id: {processId}\r\nProcess Name: {processName}\r\nThread Name: {threadName}\r\nWin32 ThreadId: {win32ThreadId}\r\nExtended Properties: {dictionary({key} - {value}\r\n)}");
            //textFormatter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Formatters.TextFormatter, Microsoft.Practices.EnterpriseLibrary.Logging");
            //BinaryLogFormatterData binaryFormattter = new BinaryLogFormatterData("Binary Formatter");
            //CustomFormatterData categoryFormatter = new CustomFormatterData("CustomFormatter", Type.GetType("CustomClasses.CustomFormatter,CustomClasses"));

            TextFormatterData textFormatter = new TextFormatterData("TextFormat", "EntLibLog{newline}{tab}{message}{newline}{priority}{newline}{tab}{severity}{newline}{tab}{eventid}{newline}{tab}{timestamp}{newline}{tab}{title}{newline}{appDomain}{newline}{processId}{newline}{processName}{newline}{machine}{newline}{threadName}{newline}{activity}{newline}{win32ThreadId}{newline}Extended Properties: {dictionary({key} - {value})}");
            textFormatter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Formatters.TextFormatter, Microsoft.Practices.EnterpriseLibrary.Logging");
            BinaryLogFormatterData binaryFormattter = new BinaryLogFormatterData("Binary Formatter");
            CustomFormatterData categoryFormatter = new CustomFormatterData("CustomFormatter", Type.GetType("CustomClasses.CustomFormatter,CustomClasses"));

            //Trace Listener Configuration
            FormattedEventLogTraceListenerData eventListener = new FormattedEventLogTraceListenerData();
            eventListener.Name = "Formatted EventLog TraceListener";
            eventListener.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FormattedEventLogTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventListener.Formatter = "TextFormat";
            eventListener.Source = "Enterprise Library Logging Test";
            FormattedEventLogTraceListenerData eventListener1 = new FormattedEventLogTraceListenerData();
            eventListener1.Name = "Formatted EventLog TraceListener1";
            eventListener1.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FormattedEventLogTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventListener1.Formatter = "CustomFormatter";
            eventListener1.Source = "Enterprise Library Logging Test";
            CustomTraceListenerData customListener = new CustomTraceListenerData("Custom Trace Listener", Type.GetType("CustomClasses.CustomRegistryListener,CustomClasses"), "");
            customListener.Formatter = "CustomFormatter";
            customListener.Attributes.Add("regKey", @"SOFTWARE\EntlibTestApp");
            customListener.Attributes.Add("regName", "LogEntry");
            FormattedDatabaseTraceListenerData dbListener = new FormattedDatabaseTraceListenerData("Database Trace Listener", "WriteLog", "AddCategory", "SqlServer", "TextFormat");
            FlatFileTraceListenerData flatfileListener = new FlatFileTraceListenerData("FlatFile TraceListener", "trace.log", "-------------", "-----------", "TextFormat");
            FlatFileTraceListenerData flatfileListener1 = new FlatFileTraceListenerData("FlatFile TraceListenerWithStackTrace", "stacktrace.log", "-------------", "-----------", "", TraceOptions.Callstack);
            MsmqTraceListenerData msmqListener = new MsmqTraceListenerData("Msmq TraceListener", @".\Private$\EntlibTest", "TextFormat");
            MsmqTraceListenerData msmqListener1 = new MsmqTraceListenerData("Msmq TraceListener1", @".\Private$\EntlibTest", "Binary Formatter");
            CustomTraceListenerData registryListener = new CustomTraceListenerData("Registry Listener", Type.GetType("CustomClasses.CustomRegistryListener,CustomClasses"), "");
            registryListener.Formatter = "TextFormat";
            registryListener.Attributes.Add("regKey", @"SOFTWARE\EntlibTestApp");
            registryListener.Attributes.Add("regName", "LogEntry");
            SystemDiagnosticsTraceListenerData systemListener = new SystemDiagnosticsTraceListenerData("System.Diagnostics EventLog TraceListener", Type.GetType("System.Diagnostics.EventLogTraceListener, System, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"), "Enterprise Library Logging Test");
            SystemDiagnosticsTraceListenerData systemfileListener = new SystemDiagnosticsTraceListenerData("System.Diagnostics FileTraceListener", Type.GetType("System.Diagnostics.TextWriterTraceListener,System, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"), "trace.log");
            SystemDiagnosticsTraceListenerData systemWmiListener = new SystemDiagnosticsTraceListenerData("System.Diagnostics WmiTraceListener", Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.WmiTraceListener,Microsoft.Practices.EnterpriseLibrary.Logging"), "");
            WmiTraceListenerData wmiListener = new WmiTraceListenerData("WMI TraceListener");
            ExtendedFlatFileTraceListenerData extendedListener = new ExtendedFlatFileTraceListenerData();
            extendedListener.Name = "ExtendedFlatFileListener";
            extendedListener.FileSize = 10;
            extendedListener.FileName = "trace.log";
            extendedListener.Formatter = "TextFormat";
            extendedListener.Type = Type.GetType("CustomClasses.ExtendedFlatFileListener,CustomClasses");

            //Category Configuration
            TraceSourceData defaultsource = new TraceSourceData();
            defaultsource.Name = "General";
            defaultsource.DefaultLevel = System.Diagnostics.SourceLevels.All;
            TraceListenerReferenceData eventLogRef = new TraceListenerReferenceData();
            eventLogRef.Name = "Formatted EventLog TraceListener";
            defaultsource.TraceListeners.Add(eventLogRef);
            TraceSourceData dbSource = new TraceSourceData();
            dbSource.Name = "DatabaseCategory";
            dbSource.DefaultLevel = System.Diagnostics.SourceLevels.All;
            TraceListenerReferenceData dbRef = new TraceListenerReferenceData();
            dbRef.Name = "Database Trace Listener";
            dbSource.TraceListeners.Add(dbRef);
            TraceSourceData denySource = new TraceSourceData();
            denySource.Name = "DenyCategory";
            denySource.DefaultLevel = System.Diagnostics.SourceLevels.All;
            denySource.TraceListeners.Add(eventLogRef);
            TraceSourceData errorSource = new TraceSourceData();
            errorSource.Name = "ErrorCategory";
            errorSource.DefaultLevel = System.Diagnostics.SourceLevels.Error;
            errorSource.TraceListeners.Add(dbRef);
            TraceSourceData eventLogSource = new TraceSourceData();
            eventLogSource.Name = "EventLogCategory";
            eventLogSource.DefaultLevel = System.Diagnostics.SourceLevels.All;
            eventLogSource.TraceListeners.Add(eventLogRef);
            TraceSourceData fileSource = new TraceSourceData();
            fileSource.Name = "FileCategory";
            fileSource.DefaultLevel = System.Diagnostics.SourceLevels.All;
            TraceListenerReferenceData fileRef = new TraceListenerReferenceData("FlatFile TraceListener");
            TraceListenerReferenceData fileRef1 = new TraceListenerReferenceData("FlatFile TraceListenerWithStackTrace");
            fileSource.TraceListeners.Add(fileRef);
            fileSource.TraceListeners.Add(fileRef1);
            TraceSourceData formatterSource = new TraceSourceData();
            formatterSource.Name = "FormatterCategory";
            formatterSource.DefaultLevel = System.Diagnostics.SourceLevels.All;
            TraceListenerReferenceData customRef = new TraceListenerReferenceData("Custom Trace Listener");
            TraceListenerReferenceData eventLogRef1 = new TraceListenerReferenceData("Formatted EventLog TraceListener1");
            TraceListenerReferenceData msmqRef1 = new TraceListenerReferenceData("Msmq TraceListener1");
            formatterSource.TraceListeners.Add(customRef);
            formatterSource.TraceListeners.Add(eventLogRef1);
            formatterSource.TraceListeners.Add(msmqRef1);
            TraceSourceData msmqSource = new TraceSourceData("MsmqCategory", SourceLevels.All);
            TraceListenerReferenceData msmqRef = new TraceListenerReferenceData("Msmq TraceListener");
            msmqSource.TraceListeners.Add(msmqRef);
            TraceSourceData registrySource = new TraceSourceData("RegistryCategory", SourceLevels.All);
            TraceListenerReferenceData regRef = new TraceListenerReferenceData("Registry Listener");
            registrySource.TraceListeners.Add(regRef);
            TraceSourceData switchOffSource = new TraceSourceData("SwitchOffCategory", SourceLevels.Off);
            switchOffSource.TraceListeners.Add(dbRef);
            TraceSourceData systemLogSource = new TraceSourceData("SystemLogCategory", SourceLevels.All);
            TraceListenerReferenceData sysRef = new TraceListenerReferenceData("System.Diagnostics EventLog TraceListener");
            TraceListenerReferenceData sysRef1 = new TraceListenerReferenceData("System.Diagnostics FileTraceListener");
            TraceListenerReferenceData sysRef2 = new TraceListenerReferenceData("System.Diagnostics WmiTraceListener");
            systemLogSource.TraceListeners.Add(sysRef);
            systemLogSource.TraceListeners.Add(sysRef1);
            systemLogSource.TraceListeners.Add(sysRef2);
            TraceSourceData wmiSource = new TraceSourceData("WMICategory", SourceLevels.All);
            TraceListenerReferenceData wmiRef = new TraceListenerReferenceData("WMI TraceListener");
            wmiSource.TraceListeners.Add(wmiRef);
            TraceSourceData extendedSource = new TraceSourceData("ExtendedFileCategory", SourceLevels.All);
            extendedSource.TraceListeners.Add(new TraceListenerReferenceData("ExtendedFlatFileListener"));

            //Special Sources Configuration
            SpecialTraceSourcesData specialSource = new SpecialTraceSourcesData(new TraceSourceData(), new TraceSourceData(), defaultsource);
            settings.SpecialTraceSources = specialSource;

            //Adding the Filter, Category, Formatter and Listener configuration to the logging setting
            settings.LogFilters.Add(categoryFilter);
            settings.LogFilters.Add(priorityFilter);
            settings.LogFilters.Add(logEnabledFilter);
            settings.LogFilters.Add(sevFilter);

            settings.Formatters.Add(textFormatter);
            settings.Formatters.Add(binaryFormattter);
            settings.Formatters.Add(categoryFormatter);

            settings.TraceListeners.Add(eventListener);
            settings.TraceListeners.Add(eventListener1);
            settings.TraceListeners.Add(customListener);
            settings.TraceListeners.Add(dbListener);
            settings.TraceListeners.Add(flatfileListener);
            settings.TraceListeners.Add(flatfileListener1);
            settings.TraceListeners.Add(msmqListener);
            settings.TraceListeners.Add(msmqListener1);
            settings.TraceListeners.Add(registryListener);
            settings.TraceListeners.Add(systemfileListener);
            settings.TraceListeners.Add(systemListener);
            settings.TraceListeners.Add(systemWmiListener);
            settings.TraceListeners.Add(wmiListener);
            settings.TraceListeners.Add(extendedListener);

            settings.TraceSources.Add(defaultsource);
            settings.TraceSources.Add(dbSource);
            settings.TraceSources.Add(denySource);
            settings.TraceSources.Add(errorSource);
            settings.TraceSources.Add(eventLogSource);
            settings.TraceSources.Add(fileSource);
            settings.TraceSources.Add(formatterSource);
            settings.TraceSources.Add(msmqSource);
            settings.TraceSources.Add(registrySource);
            settings.TraceSources.Add(switchOffSource);
            settings.TraceSources.Add(systemLogSource);
            settings.TraceSources.Add(wmiSource);
            settings.TraceSources.Add(extendedSource);

            source.Add("loggingConfiguration", settings);

            //Data Configuration Setting
            ConnectionStringsSection connectionSection = new ConnectionStringsSection();
            connectionSection.ConnectionStrings.Add(new ConnectionStringSettings("SqlServer", "Database=Logging;Server=(local)\\SQLEXPRESS;Integrated Security=SSPI;", "System.Data.SqlClient"));
            connectionSection.ConnectionStrings.Add(new ConnectionStringSettings("InvalidInstance", "Database=InvalidDatabase;Server=(local);Integrated Security=SSPI;", "System.Data.SqlClient"));
            source.Add("connectionStrings", connectionSection);

            DatabaseSettings DBSettings = new DatabaseSettings();
            DBSettings.DefaultDatabase = "SqlServer";
            source.Add("dataConfiguration", DBSettings);

            InstrumentationConfigurationSection instrumentation = new InstrumentationConfigurationSection(true, true, true);
            source.Add("instrumentationConfiguration", instrumentation);   

            return source;
        }
    }
}
