using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;     

namespace CustomClasses
{
    public class HelperClass
    {
        public static string GetEventLogName()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            return logName;
        }

        public static bool VerifyFileContent(string fileName, string content)
        {
            using (System.IO.FileStream logReader = new System.IO.FileStream(fileName, System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.ReadWrite))
            {
                long fileSize = logReader.Length;
                byte[] fileBytes = new byte[fileSize];
                int amountOfBytes = logReader.Read(fileBytes, 0, (int)fileSize);
                System.Text.UTF7Encoding utf7 = new System.Text.UTF7Encoding();
                System.String fileContent = utf7.GetString(fileBytes, 0, amountOfBytes);
                return fileContent.Contains(content);
            }
        }

        public static int GetCounterValue(string counterName, string category)
        {
            //Verify the perf counter
            string categoryName = "Enterprise Library Logging Counters";
            string instanceName = new AppDomainNameFormatter().CreateName(category);
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }

    }
}
