using System;
using System.Text;
using System.Data.Common;
using System.Management;
using System.Management.Instrumentation;
using System.Net;
using System.IO;
using System.Diagnostics; 
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Messaging;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters; 
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Win32; 
using CustomClasses;   
namespace LoggerFacade
{
    /// <summary>
    /// Summary description for TraceListenersFixture
    /// </summary>
    [TestClass]
    public class TraceListenersFixture
    {
        private bool logged = false;
        private string loggedMessage = "";
        private ManagementScope scope = null;
        private ManagementEventWatcher watcher = null;
        public TraceListenersFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
       
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region FormattedEventLogTraceListener tests
        [TestMethod]
        public void LogToEventLogDirect()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                LogEntry logEntry = new LogEntry();
                logEntry.Message = "LogToEventLogDirect";
                FormattedEventLogTraceListener eventLogListener = new FormattedEventLogTraceListener(log, new TextFormatter());
                int initialCount = log.Entries.Count;
                eventLogListener.TraceData(null, "Critical", TraceEventType.Critical, 1, logEntry);
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1); 
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message));
            }
        }

        [TestMethod]
        public void LogToEventLogUsingEvtLogCstr()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                LogEntry logEntry = new LogEntry();
                logEntry.Message = "LogToEventLogDirect";
                FormattedEventLogTraceListener eventLogListener = new FormattedEventLogTraceListener(log);
                 
                int initialCount = log.Entries.Count;
                eventLogListener.TraceData(null, "Critical", TraceEventType.Critical, 1, logEntry);
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message));
            }
        }

        [TestMethod]
        public void LogToEventLogUsingEvtSrcCstr()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                LogEntry logEntry = new LogEntry();
                logEntry.Message = "LogToEventLogDirect";
                FormattedEventLogTraceListener eventLogListener = new FormattedEventLogTraceListener("Enterprise Library Logging Test");

                int initialCount = log.Entries.Count;
                eventLogListener.TraceData(null, "Critical", TraceEventType.Critical, 1, logEntry);
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message));
            }
        }

        [TestMethod]
        public void LogToEventLogUsingEvtSrcFmtCstr()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                LogEntry logEntry = new LogEntry();
                logEntry.Message = "LogToEventLogDirect";
                FormattedEventLogTraceListener eventLogListener = new FormattedEventLogTraceListener("Enterprise Library Logging Test",new TextFormatter() );

                int initialCount = log.Entries.Count;
                eventLogListener.TraceData(null, "Critical", TraceEventType.Critical, 1, logEntry);
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message));
            }
        }

        [TestMethod]
        public void LogToEventLogUsingEvtSrcEvtLogFmtCstr()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                LogEntry logEntry = new LogEntry();
                logEntry.Message = "LogToEventLogDirect";
                FormattedEventLogTraceListener eventLogListener = new FormattedEventLogTraceListener("Enterprise Library Logging Test",logName, new TextFormatter());

                int initialCount = log.Entries.Count;
                eventLogListener.TraceData(null, "Critical", TraceEventType.Critical, 1, logEntry);
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message));
            }
        }

        [TestMethod]
        public void LogToEventLogWithCustomLogEntry()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName,".","Enterprise Library Logging Test"))
            {
                FormattedEventLogTraceListener eventLogListener = new FormattedEventLogTraceListener(log, new TextFormatter());
                int initialCount = log.Entries.Count;
                eventLogListener.TraceData(null, "Critical", TraceEventType.Critical, 1, new MyObject());
                int finalCount = log.Entries.Count;
                Assert.IsTrue((finalCount - initialCount) == 1);  
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.AreEqual("CustomClasses.MyObject", message);  
            }
        }

        [TestMethod]
        public void LogToEventLogWithCustomObject()
        {
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog log = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                CustomLogEntry logEntry = new CustomLogEntry();
                logEntry.Message = "LogToEventLogWithCustomLogEntry";
                logEntry.CategoryID = 101;
                FormattedEventLogTraceListener eventLogListener = new FormattedEventLogTraceListener(log, new CustomFormatter());
                int initialCount = log.Entries.Count;
                eventLogListener.TraceData(null, "Critical", TraceEventType.Critical, 1, logEntry);
                int finalCount = log.Entries.Count;
                string sourceName = log.Entries[finalCount - 1].Source;
                string message = log.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue((sourceName.Equals("Enterprise Library Logging Test")) && message.Contains(logEntry.Message) && message.Contains("CategoryID: 101"));
            }
        }
#endregion
        
        #region MsmqListener Tests
        [TestMethod]
        public void LogToMsmq()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter(); 
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types; 
                Logger.Write(new LogEntry("To test Msmq listener using Logger facade","MsmqCategory",1,1,TraceEventType.Critical,"",null));
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string) message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using Logger facade"));    
            }
        }

        [TestMethod]
        public void LogToMsmqDirect()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                LogEntry log = new LogEntry("To test Msmq listener using MsmqListener directly", "MsmqCategory", 1, 1, TraceEventType.Critical,"",null);
                MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener", @".\Private$\EntlibTest", new TextFormatter(), MessagePriority.Normal, false, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.None);
                msmqListener.TraceData(null, "Warning", TraceEventType.Critical, 1, log);
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types; 
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using MsmqListener directly"));
            }
        }

        [TestMethod]
        public void LogToMsmqUsingCustomObject()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types; 
                MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener",@".\Private$\EntlibTest", new TextFormatter(), MessagePriority.Normal, true, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.Automatic);
                msmqListener.TraceData(null, "Warning", TraceEventType.Critical, 1, new MyObject());
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("CustomClasses.MyObject"));
            }
        }

        [TestMethod]
        public void LogToMsmqUsingCustomLogEntry()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types; 
                CustomLogEntry log = new CustomLogEntry();
                log.CategoryID = 10;
                log.Message = "To test Msmq listener using MsmqListener directly";
                log.Categories.Add("MsmqCategory");
                MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener", @".\Private$\EntlibTest", new CustomFormatter(), MessagePriority.Normal, true, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.Automatic);
                msmqListener.TraceData(null, "Warning", TraceEventType.Critical, 1, log);
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using MsmqListener directly"));
                Assert.IsTrue(content.Contains("CategoryID: 10"));
            }
        }

        [TestMethod]
        public void LogToMsmqUsingMachineNameFormat()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                CustomLogEntry log = new CustomLogEntry();
                log.CategoryID = 10;
                log.Message = "To test Msmq listener using MsmqListener directly";
                log.Categories.Add("MsmqCategory");
                string machineName = System.Environment.MachineName;  
                MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener", machineName + @"\Private$\EntlibTest", new CustomFormatter(), MessagePriority.Normal, true, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.Automatic);
                msmqListener.TraceData(null, "Warning", TraceEventType.Critical, 1, log);
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using MsmqListener directly"));
                Assert.IsTrue(content.Contains("CategoryID: 10"));


            }
        }

        [TestMethod]
        public void LogToMsmqUsingTCPIPFormat()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                CustomLogEntry log = new CustomLogEntry();
                log.CategoryID = 10;
                log.Message = "To test Msmq listener using MsmqListener directly";
                log.Categories.Add("MsmqCategory");

                string hostName = Dns.GetHostName();
                IPHostEntry ipEntry = Dns.GetHostByName(hostName);
                IPAddress[] addr = ipEntry.AddressList;
                string ipAddress = addr[0].ToString();
                MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener", "Formatname:DIRECT=TCP:" +  ipAddress + @"\Private$\EntlibTest", new CustomFormatter(), MessagePriority.Normal, true, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.Automatic);
                msmqListener.TraceData(null, "Warning", TraceEventType.Critical, 1, log);
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains("To test Msmq listener using MsmqListener directly"));
                Assert.IsTrue(content.Contains("CategoryID: 10"));
            }
        }

        [TestMethod]
        public void LogToPublicQueue()
        {
            string hostName = Dns.GetHostName(); 
            using (MessageQueue queue = new MessageQueue(hostName + @"\EntlibTestPublic"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogEntry log = new LogEntry();
                log.Message = "To test Msmq listener using Public queue";
                log.Categories.Add("MsmqCategory");
                MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener", hostName + @"\EntlibTestPublic", new TextFormatter(), MessagePriority.Normal, true, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.Automatic);
                msmqListener.TraceData(null, "Warning", TraceEventType.Critical, 1, log);
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains(log.Message));
            }
        }

        [TestMethod]
        public void LogToPublicQueueUsingTCPIPFormat()
        {
            string hostName = Dns.GetHostName();
            using (MessageQueue queue = new MessageQueue(hostName + @"\EntlibTestPublic"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogEntry log = new LogEntry();
                log.Message = "To test Msmq listener using Public queue";
                log.Categories.Add("MsmqCategory");
                IPHostEntry ipEntry = Dns.GetHostByName(hostName);
                IPAddress[] addr = ipEntry.AddressList;
                string ipAddress = addr[0].ToString();
                MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener", "Formatname:DIRECT=TCP:"+ ipAddress + @"\EntlibTestPublic", new TextFormatter(), MessagePriority.Normal, true, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.Automatic);
                msmqListener.TraceData(null, "Warning", TraceEventType.Critical, 1, log);
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                string content = (string)message.Body;
                Assert.IsTrue(content.Contains(log.Message));
            }
        }

#endregion

        #region DatabaseListener Tests
        [TestMethod]
        public void LogLengthyMesssageToDb()
        {
            LogEntry log = new LogEntry("", "DatabaseCategory", 4, 1000, TraceEventType.Critical, "", null);
            log.Title = "Enterprise Library";
            log.Message = "Enterprise Library is a major new release of the Microsoft patterns & practices application blocks.";
            Logger.Write(log);
            Database db = DatabaseFactory.CreateDatabase("SqlServer");  
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
        }

        [TestMethod]
        public void LogMesssageCategoryAndPriorityToDb()
        {
            LogEntry log = new LogEntry();
            log.Message = "LogMesssageCategoryAndPriorityToDatabaseSink";
            log.Categories.Add("DatabaseCategory");
            log.Priority = 4;
            Logger.Write(log);
            Microsoft.Practices.EnterpriseLibrary.Data.Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, "LogMesssageCategoryAndPriorityToDatabaseSink");
        }

        [TestMethod]
        public void LogUsingDbListenerDirect()
        {
            FormattedDatabaseTraceListener listener = new FormattedDatabaseTraceListener(DatabaseFactory.CreateDatabase("SqlServer"), "WriteLog", "AddCategory", new TextFormatter());
            LogEntry log = new LogEntry("To test the Database trace listener class direclty without using Logger facade", "DatabaseCategory", 3, 1, TraceEventType.Warning, "", null);   
            listener.TraceData(null, "Error : ", TraceEventType.Critical, 1, log);
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(messageContents.Contains("To test the Database trace listener class direclty without using Logger facade"));
        }


        [TestMethod]
        public void LogToDbListenerUsingCustomObject()
        {
            FormattedDatabaseTraceListener listener = new FormattedDatabaseTraceListener(DatabaseFactory.CreateDatabase("SqlServer"), "WriteLog", "AddCategory", new TextFormatter());
            listener.TraceData(null, "Error : ", TraceEventType.Critical, 1, new MyObject());
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, "CustomClasses.MyObject");
        }

        [TestMethod]
        public void LogToDbListenerUsingCustomLogEntry()
        {
            FormattedDatabaseTraceListener listener = new FormattedDatabaseTraceListener(DatabaseFactory.CreateDatabase("SqlServer"), "WriteLog", "AddCategory", new CustomFormatter());
            CustomLogEntry log = new CustomLogEntry();
            log.CategoryID = 5;
            log.Message = "LogToDatabaseUsingCustomLogEntryObject";
            listener.TraceData(null, "Error : ", TraceEventType.Critical, 1, log);
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 FormattedMessage FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(messageContents.Contains("CategoryID: 5"));
            Assert.IsTrue(messageContents.Contains("LogToDatabaseUsingCustomLogEntryObject"));
        }

        [TestMethod]
        public void LoggerOverloadToDbListener()
        {
            Logger.Write("LoggerWriteOverload", "DatabaseCategory", 4, 200, TraceEventType.Information, "LoggerOverloadToDbListener");
            Microsoft.Practices.EnterpriseLibrary.Data.Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, "LoggerWriteOverload");

        }

        [TestMethod]
        public void LogToDbWithMultipleCategory()
        {
            LogEntry log = new LogEntry();
            log.Message = "Log To Database using Multiple Category";
            log.Categories.Add("DatabaseCategory");
            log.Categories.Add("General");
            log.Priority = 4;
            Logger.Write(log);
            Microsoft.Practices.EnterpriseLibrary.Data.Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog eventlog = new EventLog(logName))
            {
                eventlog.Entries[(eventlog.Entries.Count) - 1].Message.Contains(log.Message);
            }
        }

        [TestMethod]
        public void SetContextItemsToDbListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "SetContextItemsToDbListener";
            log.Categories.Add("DatabaseCategory");
            Logger.SetContextItem("key1", "value1");
            Logger.SetContextItem("key2", "value2");
            Logger.Write(log);
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 FormattedMessage FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(messageContents.Contains("key1") && messageContents.Contains("value1"));
            Assert.IsTrue(messageContents.Contains("key2") && messageContents.Contains("value2"));
        }

        [TestMethod]
        public void LogToDbListenerApplySwitchFilter()
        {
            LogEntry log = new LogEntry();

            log.Message = "LogToDbListenerApplySwitchFilter";
            log.Categories.Add("ErrorCategory");
            log.Priority = 5;
            log.EventId = 100;
            log.Severity = TraceEventType.Information;

            Microsoft.Practices.EnterpriseLibrary.Data.Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT count(logId) FROM Log");
            int initialCount = Convert.ToInt32(db.ExecuteScalar(command));  
            Logger.Write(log);
            int finalCount = Convert.ToInt32(db.ExecuteScalar(command));
            Assert.IsTrue(initialCount == finalCount);  
        }

        [TestMethod]
        public void LogToDbListenerAllowSwitchFilter()
        {
            LogEntry log = new LogEntry();
            log.Message = "LogToDbListenerAllowSwitchFilter";
            log.Categories.Add("ErrorCategory");
            log.Priority = 3;
            log.EventId = 100;
            log.Severity = TraceEventType.Critical;
            Logger.Write(log);
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
        }

        [TestMethod]
        public void LogToDbListenerApplySwitchOff()
        {
            LogEntry log = new LogEntry();
            log.Message = "LogToDbListenerApplySwitchOff";
            log.Categories.Add("SwitchOffCategory");
            log.Priority = 5;
            log.EventId = 100;
            log.Severity = TraceEventType.Information;
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT count(logId) FROM Log");
            int initialCount = Convert.ToInt32(db.ExecuteScalar(command));
            Logger.Write(log);
            int finalCount = Convert.ToInt32(db.ExecuteScalar(command));
            Assert.IsTrue(initialCount == finalCount);
        }


        [TestMethod]
        public void UsingLogWriterToLogToDbListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "Logged using LogWriter";
            log.Categories.Add("DatabaseCategory");
            log.Priority = 2;
            log.EventId = 100;
            log.Severity = TraceEventType.Information;
            log.Title = "UsingLogWriterToLogToDbListener";
            LogWriterFactory factory = new LogWriterFactory();
            LogWriter writer = factory.Create();
            writer.Write(log);
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 Message FROM Log ORDER BY LogId DESC");
            string messageContents = Convert.ToString(db.ExecuteScalar(command));
            Assert.AreEqual(messageContents, log.Message);
        }

        #endregion

        #region WMIListener Tests
        [TestMethod]
        public void LogToWmi()
        {
            LogEntry entry = new LogEntry();
            entry.Categories.Add("WMICategory");
            entry.Message = "LogToWmi";
            MonitorWmiEvent();
            Logger.Write(entry);
            System.Threading.Thread.Sleep(1000);
            StopMonitorWmiEvent(); 
            Assert.IsTrue(logged);
            Assert.IsTrue(loggedMessage.Contains("LogToWmi"), loggedMessage);
        }

        [TestMethod]
        public void LogToWmiDirect()
        {
            WmiTraceListener wmiListener = new WmiTraceListener();
            LogEntry log = new LogEntry("To test the Wmi trace listener class direclty without using Logger facade", "WmiCategory", 3, 1, TraceEventType.Warning, "", null);
            MonitorWmiEvent(); 
            wmiListener.TraceData(null, "Error : ", TraceEventType.Critical, 1, log);
            System.Threading.Thread.Sleep(1000);
            StopMonitorWmiEvent();
            Assert.IsTrue(logged);
            Assert.IsTrue(loggedMessage.Contains("To test the Wmi trace listener class direclty without using Logger facade"), loggedMessage);
        }

        [TestMethod]
        public void LogToWmiUsingCustomObject()
        {
            WmiTraceListener wmiListener = new WmiTraceListener();
            MonitorWmiEvent();
            wmiListener.TraceData(null, "Error : ", TraceEventType.Critical, 1, new MyObject() );
            System.Threading.Thread.Sleep(1000);
            StopMonitorWmiEvent();
            Assert.IsTrue(logged);
            Assert.IsTrue(loggedMessage.Contains("CustomClasses.MyObject"));
        }


        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            loggedMessage = args.NewEvent.GetText(TextFormat.Mof);
            logged = true;
        }

        private void MonitorWmiEvent()
        {
            if (scope == null)
            {
                scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
                scope.Options.EnablePrivileges = true;
                EventQuery eq = new EventQuery("SELECT * FROM LogEntryV20");
                watcher = new ManagementEventWatcher(scope, eq);
            }
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            logged = false;
            loggedMessage = "";
        }

        private void StopMonitorWmiEvent()
        {
            watcher.EventArrived -= new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Stop();
            
        }
        #endregion

        #region FlatFileListener Tests
        [TestMethod] 
        public void LogToFlatFile()
        {
            string message = "To test the logger facade for Flat File";
            string category = "FileCategory";
            int priority = 4;
            int eventId = 100;
            TraceEventType severity = TraceEventType.Error;
            string title = "LogToFlatFile";
            Logger.Write(message, category, priority, eventId, severity, title);
            Assert.IsTrue(File.Exists("trace.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace.log", message));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace.log",title)); 
        }

        [TestMethod]
        public void LogToFlatFileDirect()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly","FileCategory",1,1,TraceEventType.Error,"LogToFlatFileDirect",null);
            FlatFileTraceListener listener = new FlatFileTraceListener("trace1.log", new TextFormatter());
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();  
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect")); 
        }

        [TestMethod]
        public void LogToFlatFileDirectDefaultCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            FlatFileTraceListener listener = new FlatFileTraceListener();
            listener.Writer = new TextWriterTraceListener("trace1.log").Writer;   
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
        }

        [TestMethod]
        public void LogToFlatFileUsingFStreamCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (FileStream fs = new FileStream("trace1.log", FileMode.Create,FileAccess.ReadWrite  ))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(fs);
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }

        [TestMethod]
        public void LogToFlatFileUsingFmtCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            FlatFileTraceListener listener = new FlatFileTraceListener(new TextFormatter());
            listener.Writer = new TextWriterTraceListener("trace1.log").Writer;   
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
        }

        [TestMethod]
        public void LogToFlatFileUsingStrWtrCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (StreamWriter writer = new StreamWriter("trace1.log"))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(writer);
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }


        [TestMethod] 
        public void LogToFlatFileUsingFnameCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            FlatFileTraceListener listener = new FlatFileTraceListener("trace1.log");
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
        }

        [TestMethod]
        public void LogToFlatFileUsingFnameHdrFtrCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            FlatFileTraceListener listener = new FlatFileTraceListener("trace1.log","","");
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
        }

        [TestMethod]
        public void LogToFlatFileUsingFnameHdrFtrFmtCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            FlatFileTraceListener listener = new FlatFileTraceListener("trace1.log", "", "",new TextFormatter());
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
        }

        [TestMethod]
        public void LogToFlatFileUsingNameFnameCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            FlatFileTraceListener listener = new FlatFileTraceListener("trace1.log","FlatFileListener");
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
        }

        [TestMethod]
        public void LogToFlatFileUsingNameFnameFmtCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            FlatFileTraceListener listener = new FlatFileTraceListener("trace1.log","FlatFileListener",new TextFormatter() );
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
        }

        [TestMethod]
        public void LogToFlatFileUsingNameFstreamFmtCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (FileStream fs = new FileStream("trace1.log", FileMode.Create, FileAccess.ReadWrite))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(fs, "FlatFileListener",new TextFormatter());
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }

        [TestMethod]
        public void LogToFlatFileUsingNameFstreamCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (FileStream fs = new FileStream("trace1.log", FileMode.Create, FileAccess.ReadWrite))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(fs, "FlatFileListener");
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }

        [TestMethod]
        public void LogToFlatFileUsingFStreamFmtCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (FileStream fs = new FileStream("trace1.log", FileMode.Create, FileAccess.ReadWrite))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(fs, new TextFormatter());
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }

        [TestMethod]
        public void LogToFlatFileUsingNameStrWtrCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (StreamWriter writer = new StreamWriter("trace1.log"))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(writer, "FlatFileListener");
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }

        [TestMethod]
        public void LogToFlatFileUsingNameStrWtrFmtCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (StreamWriter writer = new StreamWriter("trace1.log"))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(writer,"FlatFileListener", new TextFormatter());
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }

        [TestMethod]
        public void LogToFlatFileUsingStrWtrFmtCstr()
        {
            LogEntry log = new LogEntry("To test FlatFile tracelistner directly", "FileCategory", 1, 1, TraceEventType.Error, "LogToFlatFileDirect", null);
            using (StreamWriter writer = new StreamWriter("trace1.log"))
            {
                FlatFileTraceListener listener = new FlatFileTraceListener(writer, new TextFormatter());
                listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
                listener.Flush();
                listener.Close();
                Assert.IsTrue(File.Exists("trace1.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "To test FlatFile tracelistner directly"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "LogToFlatFileDirect"));
            }
        }


        [TestMethod]
        public void LogToFlatFileWithCustomLogEntry()
        {
            CustomLogEntry log = new CustomLogEntry();
            log.CategoryID = 100;
            log.Message = "To test FlatFile tracelistner directly for Custom LogEntry";
            FlatFileTraceListener listener = new FlatFileTraceListener("trace1.log", new CustomFormatter());
            listener.TraceData(null, "Error", TraceEventType.Error, 1, log);
            listener.Flush();
            listener.Close();  
            Assert.IsTrue(File.Exists("trace1.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", log.Message));
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", "CategoryID: 100")); 
        }

        [TestMethod]
        public void LogToFlatFileWithStackTrace()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test TraceOutputOption with stack trace";
            log.Title = "LogToFlatFileWithStackTrace";
            log.Categories.Add("FileCategory");  
            Logger.Write(log);
            Assert.IsTrue(File.Exists("stacktrace.log"));
            Assert.IsTrue(HelperClass.VerifyFileContent("stacktrace.log", log.Message));
            Assert.IsTrue(HelperClass.VerifyFileContent("stacktrace.log", "Callstack=   at System.Environment.GetStackTrace(Exception e, Boolean needFileInfo)")); 
        }


#endregion

        #region SystemDiagnosticsTraceListener Tests
        [TestMethod]
        public void LogUsingSystemDiagnosticsTraceListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test SystemDiagnosticsTraceListener for system and entlib listeners";
            log.Title = "LogUsingSystemDiagnosticsTraceListener";
            log.Categories.Add("SystemLogCategory");

            string logName = "Application";
            if (EventLog.SourceExists("Enterprise Library Logging Test"))
            {
                logName = EventLog.LogNameFromSourceName("Enterprise Library Logging Test", ".");
            }
            else
            {
                EventLog.CreateEventSource("Enterprise Library Logging Test", "Application");
            }
            using (EventLog eventlog = new EventLog(logName, ".", "Enterprise Library Logging Test"))
            {
                MonitorWmiEvent();
                int initialCount = eventlog.Entries.Count;
                Logger.Write(log);

                int finalCount = eventlog.Entries.Count;
                string message = eventlog.Entries[finalCount - 1].Message;
                Assert.IsTrue((finalCount - initialCount) == 1);
                Assert.IsTrue(message.Contains(log.Message));

                Assert.IsTrue(File.Exists("trace2.log"));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace2.log", log.Message));
                Assert.IsTrue(HelperClass.VerifyFileContent("trace2.log", log.Title)); 

                System.Threading.Thread.Sleep(1000);
                StopMonitorWmiEvent();
                Assert.IsTrue(logged);
                Assert.IsTrue(loggedMessage.Contains(log.Message));
            }

        }

        #endregion

        #region CustomListener Tests
        [TestMethod] 
        public void LogToRegistryListener()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test CustomTrace Listener that derives from CustomTraceListener";
            log.Title = "LogToRegistryListener";
            log.Categories.Add("RegistryCategory");
            Logger.Write(log);
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"Software\EntlibTestApp");       
            string content = (string) key.GetValue("LogEntry"); 
            key.Close(); 
            Assert.IsTrue(content.Contains(log.Message));
        }

        [TestMethod]
        public void LogToExtendedFileListenerDirect()
        {
            LogEntry log = new LogEntry();
            log.Message = "To test extension of flatfile Listener ";
            log.Title = "LogToExtendedFileListener";
            log.Categories.Add("ExtendedFileCategory");
            ExtendedFlatFileListener listener = new ExtendedFlatFileListener("trace1.log", "", "", new TextFormatter(), 10);
            listener.TraceData(null, "Extended Flat File Listener", TraceEventType.Warning, 10,log);
            listener.Flush();
            listener.Close();  
            Assert.IsTrue(HelperClass.VerifyFileContent("trace1.log", log.Message));
        }

#endregion
    }
}
