using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.AzMan;
using System.Security.Principal;
using System.Threading;

namespace SecurityAzManFuncTests
{
    /// <summary>
    /// Summary description for AzManAuthorizationTestFixture
    /// </summary>
    [TestClass]
    public class AzManAuthorizationTestFixture
    {
        public AzManAuthorizationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void AzManTaskAuthorizeTest()
        {
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
                WindowsIdentity identity = WindowsIdentity.GetAnonymous();
                
                bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Authorize Purchase");
                Assert.IsTrue(isAuthorized);
            
        }

        [TestMethod]
        public void AzManUnauthorizedTaskTest()
        {
            
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Fire Employee");
            Assert.IsFalse(isAuthorized);
        }

        [TestMethod]
        public void AzManOperationAuthorizeTest()
        {

            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "O:View Purchase Order");
            Assert.IsTrue(isAuthorized);
        }

        [TestMethod]
        public void AzManScopeTaskAuthorizeTest()
        {
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("AzManProviderwithScope");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Manage Extranet");
            Assert.IsTrue(isAuthorized);
        }

        [TestMethod]
        public void AzManUnauthorizedOperationTest()
        {
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "O:Terminate Employee");
            Assert.IsFalse(isAuthorized);
        }

        [TestMethod]
        [ExpectedException(typeof(System.Security.SecurityException))]
		public void AzManInvalidTaskTest()
        {

            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Invalid Task");
        }

        [TestMethod]
        [ExpectedException(typeof(System.Security.SecurityException))]
        public void AzManInvalidOperationTest()
        {

            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "O:Invalid Operation");
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentNullException))]
        public void AzManNullIdentityTest()
        {
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(null), "Fire Employee");
        }

       
    }
}
