using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;

namespace SecurityAzManFuncTests
{
    /// <summary>
    /// Summary description for AzmanDictTestFixture
    /// </summary>
    [TestClass]
    public class AzmanDictTestFixture
    {
        private DictionaryConfigurationSource configSource;
        AuthorizationProvider azManProvider;
        public AzmanDictTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestInitialize()]
        public void MyTestInitialize()
        {
            AzmanDictSource sourceSection = new AzmanDictSource();
            configSource = sourceSection.BuildDictionarySourceSection();
            AuthorizationProviderFactory authFactory = new AuthorizationProviderFactory(configSource);
            azManProvider = authFactory.Create("DefaultAzManProviderDic") as AuthorizationProvider;
        }

        [TestMethod]
        public void AzManTaskAuthorizeDicTest()
        {
            try
            {
                WindowsIdentity identity = WindowsIdentity.GetAnonymous();

                bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Authorize Purchase");
                Assert.IsTrue(isAuthorized);
            }
            catch (Exception)
            {
                Assert.Fail();
            }
        }

        [TestMethod]
        public void AzManUnauthorizedTaskDicTest()
        {

            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Fire Employee");
            Assert.IsFalse(isAuthorized);
        }

        [TestMethod]
        public void AzManOperationAuthorizeDicTest()
        {
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "O:View Purchase Order");
            Assert.IsTrue(isAuthorized);
        }

        [TestMethod]
        public void AzManUnauthorizedOperationDicTest()
        {
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "O:Terminate Employee");
            Assert.IsFalse(isAuthorized);
        }

        [TestMethod]
        [ExpectedException(typeof(System.Security.SecurityException))]
        public void AzManInvalidTaskDicTest()
        {
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Invalid Task");
        }

        [TestMethod]
        [ExpectedException(typeof(System.Security.SecurityException))]
        public void AzManInvalidOperationDicTest()
        {
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "O:Invalid Operation");
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentNullException))]
        public void AzManNullIdentityDicTest()
        {
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(null), "Fire Employee");
        }
    }
}
