using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Security;
using System.Threading;

namespace SecurityCachingFuncTests
{
    /// <summary>
    /// Summary description for SecurityCachingInMemoryTestFixture
    /// </summary>
    [TestClass]
    public class SecurityCachingInMemoryTestFixture
    {
        IIdentity identity;
        private static IPrincipal principal;
        private static object profile;
        public SecurityCachingInMemoryTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestInitialize()]
        public void SetUp()
        {
            identity = new GenericIdentity("testuser", "SecurityCache");
            string[] roles = new string[] { "admin", "managers" };
            principal = new GenericPrincipal(identity, roles);
            profile = "New Profile";
        }

        #region Principal
        [TestMethod]
        public void InMemoryAddPrincipalTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SavePrincipal(principal);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNotNull(cachedPrincipal);
            Assert.AreEqual(cachedPrincipal.Identity.Name, "testuser");
            Assert.IsTrue(cachedPrincipal.IsInRole("admin"));
            Assert.IsTrue(cachedPrincipal.IsInRole("managers"));
            Assert.IsFalse(cachedPrincipal.IsInRole("users"));

        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void InMemoryAddNullPrincipalTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SavePrincipal(null);
            Assert.IsNull(token);
        }

        [TestMethod]
        public void InMemoryGetExpiredPrincipalTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SavePrincipal(principal);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpirePrincipal(token);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNull(cachedPrincipal);

        }

        [TestMethod]
        public void InMemoryAbsoluteExpirePrincipalTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SavePrincipal(principal);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            Thread.Sleep(70000);
            
            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNull(cachedPrincipal);

        }


        #endregion

        #region Identity
        [TestMethod]
        public void InMemoryAddIdentityTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void InMemoryAddNullIdentityTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(null);
            Assert.IsNull(token);
        }

        [TestMethod]
        public void InMemoryGetExpiredIdentityTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpireIdentity(token);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNull(cachedIdentity);
        }

        #endregion

        #region Profile
        [TestMethod]
        public void InMemoryAddProfileTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveProfile(profile);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            object retrievedProfile = securityCache.GetProfile(token);
            Assert.IsNotNull(retrievedProfile);
            Assert.AreEqual(retrievedProfile.ToString(), "New Profile");
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void InMemoryAddNullProfileTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveProfile(null);
            Assert.IsNull(token);
        }

        [TestMethod]
        public void InMemoryGetExpiredProfileTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveProfile(profile);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpireProfile(token);

            object retrievedProfile = securityCache.GetProfile(token);
            Assert.IsNull(retrievedProfile);
        }

        #endregion
        
        [TestMethod]
        public void InMemoryGetNonExistingTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpireIdentity(token);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNull(cachedPrincipal);
        }

        [TestMethod]
        public void InMemoryAddIdentityPrincipalProfileTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.SavePrincipal(principal, token);

            securityCache.SaveProfile(profile, token);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNotNull(cachedPrincipal);
            Assert.AreEqual(cachedPrincipal.Identity.Name, "testuser");
            Assert.IsTrue(cachedPrincipal.IsInRole("admin"));
            Assert.IsTrue(cachedPrincipal.IsInRole("managers"));
            Assert.IsFalse(cachedPrincipal.IsInRole("users"));

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");

            object retrievedProfile = securityCache.GetProfile(token);
            Assert.IsNotNull(retrievedProfile);
            Assert.AreEqual(retrievedProfile.ToString(), "New Profile");

        }


    }
}
