using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Security;


namespace SecurityCoreFuncTests
{
    /// <summary>
    /// Summary description for AuthorizationRuleDictSourceTestFixture
    /// </summary>
    [TestClass]
    public class AuthorizationRuleDictSourceTestFixture
    {
        private AuthorizationRuleProvider authRuleProvider;
        private DictionaryConfigurationSource configSource;
        IIdentity identity;
        private static IPrincipal principal;
        public AuthorizationRuleDictSourceTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestInitialize()]
        public void MyTestInitialize()
        {
            AuthorizationRuleDictSource sourceSection = new AuthorizationRuleDictSource();
            configSource = sourceSection.BuildDictionarySourceSection();
            AuthorizationProviderFactory authFactory = new AuthorizationProviderFactory(configSource);
            authRuleProvider = authFactory.Create("DefaultRuleProviderDic") as AuthorizationRuleProvider;

            identity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "Manager" };
            principal = new GenericPrincipal(identity, roles);
            
        }

        [TestMethod]
        public void AuthorizeIdentityDicTest()
        {
            
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityRuleDic");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeInvalidIdentityDicTest()
        {
            IIdentity invalidIdentity = new GenericIdentity("UnauthorizedUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal invalidPrincipal = new GenericPrincipal(invalidIdentity, roles);

            bool authorized = authRuleProvider.Authorize(invalidPrincipal, "TestIdentityRuleDic");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityNewRoleDicTest()
        {
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles);

            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityRuleDic");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeRoleDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleRuleDic");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void CustomAuthorizeRoleDicTest()
        {
            AuthorizationProviderFactory factory = new AuthorizationProviderFactory(configSource);
            IAuthorizationProvider authRuleProvider = factory.Create("MyAuthorizationProvider");  
            bool authorized = authRuleProvider.Authorize(null,"");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeInvalidRoleDicTest()
        {
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles);

            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestRoleRuleDic");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityOrRoleDicTest()
        {
            // Testing "Identity or Role" rule for authorizing identity.
            IIdentity newIdentity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORRoleRuleDic");
            Assert.IsTrue(authorized, "Authorizing Identity using Identity Or Role rule failed");

            // Testing "Identity or Role" rule for authorizing role.
            newIdentity = new GenericIdentity("NewUser");
            roles = new string[] { "Manager" };
            newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORRoleRuleDic");
            Assert.IsTrue(authorized, "Authorizing Role using Identity Or Role rule failed");
        }

        [TestMethod]
        public void AuthorizeInvalidIdentityOrRoleDicTest()
        {
            // Testing "Identity or Role" rule for authorizing identity.
            IIdentity newIdentity = new GenericIdentity("NewUser");
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORRoleRuleDic");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityAndRoleDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityAndRoleRuleDic");
            Assert.IsTrue(authorized);

            // Testing "Identity And Role" rule with invalid role.
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityAndRoleRuleDic");
            Assert.IsFalse(authorized);

        }

        [TestMethod]
        public void AuthorizeMultipleIdentitiesDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityORIdentityRuleDic");
            Assert.IsTrue(authorized);

            IIdentity newIdentity = new GenericIdentity("Me");
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORIdentityRuleDic");
            Assert.IsTrue(authorized);

            //Testing invalid identity
            newIdentity = new GenericIdentity("InvalidUser");
            roles = new string[] { "NewRole" };
            newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORIdentityRuleDic");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityAndIdentityDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityAndIdentityRuleDic");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeMultipleRolesDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleORRoleRuleDic");
            Assert.IsTrue(authorized);

            IIdentity newIdentity = new GenericIdentity("Me");
            string[] roles = new string[] { "Employee" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestRoleORRoleRuleDic");
            Assert.IsTrue(authorized);

            //Testing invalid identity
            newIdentity = new GenericIdentity("InvalidUser");
            roles = new string[] { "NewRole" };
            newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestRoleORRoleRuleDic");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeRoleAndRoleDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleAndRoleRuleDic");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeRoleOrIdentityDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleOrIdentityRuleDic");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeRuleNameWithSpacesDicTest()
        {
            bool authorized = authRuleProvider.Authorize(principal, "TestRuleName With Spaces Dic");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        [ExpectedException(typeof(SyntaxException))]
        public void AuthorizeRuleWithSpacesDicTest()
        {
            string[] roles = new string[] { "Test Manager" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles);

            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestRule With Spaces Dic");
            Assert.IsTrue(authorized);
        }

    }
}
